/* This file is part of dengue: Dengue fever individual propagation model
Copyright (C) 2012 Harish Padmanabha, Fabio Correa

This project has been developed by Harish Padmanabha<harishpadmanabha@gmail.com> and Fabio Correa<facorread@gmail.com>
This code has been delivered to the Public Library of Science, on February 16, 2015.
We, the developers, grant you a license to review and modify this source code for your own use.
We do not grant you license to distribute or publish this program or any derivative work, any of its output, the ideas or algorithms contained within, in any form.
Anybody who wishes to get a copy of this source code or to get this project under a different license should email the developers directly.

*/

/// \file eta.h Facility for the calculation of the Estimated Time to Arrival or completion of a program.

/// \addtogroup facilities Facilities

//Check for C++0x
#if (__cplusplus <= 199711L) && !defined(__GXX_EXPERIMENTAL_CXX0X__)
#error Please use a C++11 compiler, such as GCC-4.6 or later. On Linux, use a recent distribution. On Windows, use MinGW or Cygwin. On Mac OS, use macports.
#endif


/// \def _wur_
/// \brief Warns about unused results of functions
/// \details _wur_ can be used in most function declarations so that the compiler warns about unused results of these functions. _wur_ is automatically enabled on GCC only; in other compilers it should do nothing.
#if defined(__GXX_EXPERIMENTAL_CXX0X__)
#define _wur_ __attribute__ (( warn_unused_result ))
#else
#define _wur_
#endif

// Boost includes
#include <boost/date_time.hpp>

class etaCls;

/// \brief Class to calculate the Estimated Time to Arrival or completion of the program. \ingroup facilities
/// \details This class receives information of each big step in the run of a program, namely the evaluation of a simulation scenario, and based on the runtime it calculates the ETA of the program.
class etaCls {
	private:
		/// Total number of scenarios reported by the program.
		int mNScenarios;
		/// Current scenario number.
		int mScenario;
		/// Starting time of the program.
		boost::posix_time::ptime mStartingTime;
		/// Estimated time of completion.
		boost::posix_time::ptime mEstimatedTime;
		/// Time duration from current scenario until program completion.
		boost::posix_time::time_duration mDuration;
		/// Currently completed scenario percentage.
		int mCompletedPercent;
	public:
		/// Initializes the object with the current time of program start. \param [in] nScenarios Number of scenarios that the program intends to run.
		void start(const int nScenarios);
		/// Returns the starting time.
		const boost::posix_time::ptime& startingTime() const _wur_ { return mStartingTime; }
		/// Reports a simulation step in the program, to calculate the ETA.
		void step();
		/// Prints the ETA info to \c std::cerr.
		void info() const;
		/// Returns the scenario number that the program is running.
		int scenario() const _wur_ { return mScenario; }
		/// Returns the number of scenarios that the program reported.
		int nScenarios() const _wur_ { return mNScenarios; }
};
