/* This file is part of dengue: Dengue fever individual propagation model
Copyright (C) 2012 Harish Padmanabha, Fabio Correa

This project has been developed by Harish Padmanabha<harishpadmanabha@gmail.com> and Fabio Correa<facorread@gmail.com>
This code has been delivered to the Public Library of Science, on February 16, 2015.
We, the developers, grant you a license to review and modify this source code for your own use.
We do not grant you license to distribute or publish this program or any derivative work, any of its output, the ideas or algorithms contained within, in any form.
Anybody who wishes to get a copy of this source code or to get this project under a different license should email the developers directly.

*/

/// \file gsl.h Implenetation of the interfaces to the GNU Scientific Library.

#include "gsl.h"
#include <list>
#include <ctime>


using std::cerr;
using std::endl;

void discreteArrayCls::set() {
	/// Generates the array of double that is represented by the distroCls.
	// begin() has the least index, rbegin has the greatest index.
	const histCls::const_reverse_iterator rbegin(mHistogram.rbegin());
	std::vector<double> v;
	v.resize(rbegin->first, 0);
	for(auto& item : mHistogram)
		v[item.first] = item.second;
	/// Generates the distribution.
	distro = {v.begin(), v.end()};
}

void discreteArrayCls::produceDistro() {
	mDistro.clear();
	const histCls::const_iterator histEnd(mHistogram.end());
	int norm(0);
	for(auto& item : mHistogram)
		norm += item.second;
	const double invNorm(1.0 / static_cast<double>(norm));
	for(auto& item : mHistogram)
		mDistro[item.first] = item.second * invNorm;
}

void discreteArrayCls::print(std::ofstream& outfile) const {
	for(auto& item : mDistro)
		outfile << item.first << "\t" << item.second << endl;
}

void print(std::ofstream& outfile, const std::vector<discreteArrayCls*>& distributions) {
	/* Prints distribution data in the following format:
	// x   Distro1     Distro2    Distro3
	// 1     0.1          0.2        1
	// 8     0.9          0.6
	// 30                 0.2
	*/
	if(distributions.empty()) {
		cerr << __LINE__ << " " << __PRETTY_FUNCTION__ << " List of distributions is empty." << endl;
		std::exit(1);
	}
	for(discreteArrayCls* distroP : distributions) {
		if(distroP->distroEmpty()) {
			cerr << __LINE__ << " " << __PRETTY_FUNCTION__ << " One of the specified distributions is empty." << endl;
			std::exit(1);
		}
		distroP->distroStart();
	}
	while(true) {
		// This keeps track of whether distributions have data yet to print.
		bool endOfPrint(true);
		/// This function prints the specified probability \c distributions, only at points where at least one distribution has a nonzero value.
		/// At each step, the function looks up the lowest value of the variate \c x.
		int x(std::numeric_limits<int>::max());
		for(discreteArrayCls* distroP : distributions) {
			if(distroP->distroCont()) {
				// Current variate of the current distribution;
				const int distx(distroP->distroItem().first);
				if(distx < x)
					x = distx;
				endOfPrint = false;
			}
		}
		if(endOfPrint)
			return;
		/// Then the function prints the values.
		outfile << x;
		for(discreteArrayCls* distroP : distributions) {
			outfile << "\t";
			if(distroP->distroCont()) {
				// Current item of the current distribution;
				const auto item(distroP->distroItem());
				if(item.first == x) {
					outfile << item.second;
					distroP->distroStep();
				} else
					outfile << "0";
			}
		}
		outfile << endl;
	}
}

void printHist(std::ofstream& outfile, const std::vector<discreteArrayCls*>& histograms) {
	/* Prints histogram data in the following format:
	// x   Freq1     Freq2    Freq3
	// 1     1         2        15
	// 8     9         6
	// 30              2
	*/
	if(histograms.empty()) {
		cerr << __LINE__ << " " << __PRETTY_FUNCTION__ << " List of histograms is empty." << endl;
		std::exit(1);
	}
	for(discreteArrayCls* histP : histograms) {
		if(histP->histEmpty()) {
			cerr << __LINE__ << " " << __PRETTY_FUNCTION__ << " One of the specified histograms is empty." << endl;
			std::exit(1);
		}
		histP->histStart();
	}
	while(true) {
		// This keeps track of whether histograms have data yet to print.
		bool endOfPrint(true);
		/// This function prints the specified \c histograms, only at points where at least one histogram has a nonzero value.
		/// At each step, the function looks up the lowest value of the variate \c x.
		int x(std::numeric_limits<int>::max());
		for(discreteArrayCls* histP : histograms) {
			if(histP->histCont()) {
				// Current variate of the current histogram;
				const int histx(histP->histItem().first);
				if(histx < x)
					x = histx;
				endOfPrint = false;
			}
		}
		if(endOfPrint)
			return;
		/// Then the function prints the values.
		outfile << x;
		for(discreteArrayCls* histP : histograms) {
			outfile << "\t";
			if(histP->histCont()) {
				// Current item of the current histogram;
				const auto item(histP->histItem());
				if(item.first == x) {
					outfile << item.second;
					histP->histStep();
				} else
					outfile << "0";
			}
		}
		outfile << endl;
	}
}

void gslCls::seed() {
	try {
		std::random_device r;
		mGenerator.seed(r());
	} catch(...) {
		mGenerator.seed(std::time(0));
	}
}

void gslCls::seedSalt(const std::mt19937::result_type salt) {
	try {
		std::random_device r;
		mGenerator.seed(r() + salt);
	} catch(...) {
		mGenerator.seed(std::time(nullptr) + salt);
	}
}

void gslCls::seed(const int seed) {
	mGenerator.seed(seed);
}

double gslCls::flat(const double a, const double b) {
	std::uniform_real_distribution<> distro(a, b);
	return distro(mGenerator);
}

size_t gslCls::flat(const size_t a, const size_t b) {
	std::uniform_int_distribution<size_t> distro(a, b - 1);
	return distro(mGenerator);
}

size_t gslCls::flat(const size_t b) {
	std::uniform_int_distribution<size_t> distro(0, b - 1);
	return distro(mGenerator);
}

/// \return an integer random number given by a rectangular distribution in the closed set [a.min(), a.max()].
size_t gslCls::flatClosed(const paramIntervalCls<size_t>& a) {
	std::uniform_int_distribution<size_t> distro(a.min(), a.max());
	return distro(mGenerator);
}

double gslCls::flatM(double mean, double amplitude) {
	const double a2 = amplitude / 2.0;
	std::uniform_real_distribution<> distro(mean - a2, mean + a2);
	return distro(mGenerator);
}

unsigned int gslCls::bernoulli(double p) {
	std::bernoulli_distribution distro(p);
	return distro(mGenerator) ? 1 : 0;
}

bool gslCls::bern(double p) {
	std::bernoulli_distribution distro(p);
	return distro(mGenerator);
}

unsigned int gslCls::binomial(double p, unsigned int n) {
	std::binomial_distribution<unsigned int> distro(n, p);
	return distro(mGenerator);
}

unsigned int gslCls::negBinomial(double p, double n) {
	std::negative_binomial_distribution<> distro(n, p);
	return distro(mGenerator);
}

unsigned int gslCls::geometric(double p) {
	std::geometric_distribution<> distro(p);
	return distro(mGenerator);
}

int gslCls::poisson(const double p) {
	std::poisson_distribution<> distro(p);
	return distro(mGenerator);
}

unsigned long int gslCls::get() {
	return mGenerator();
}

double gslCls::beta(double alpha, double beta) {
	// Taken from GSL 1.15
	std::gamma_distribution<> adistro(alpha, 1);
	std::gamma_distribution<> bdistro(beta, 1);
	double x(adistro(mGenerator));
	double y(bdistro(mGenerator));
	return x / (x + y);
}

double gslCls::weibull(double scale, double shape) {
	std::weibull_distribution<> distro(shape, scale);
	return distro(mGenerator);
}

void gslCls::bivariate_gauss(double& xdiff, double& ydiff, const double sigma) {
	std::normal_distribution<> distro(0, sigma);
	xdiff = distro(mGenerator);
	ydiff = distro(mGenerator);
}

void gslCls::bigauss_displacement(double& x, double& y, const double sigma) {
	std::normal_distribution<> distro(0, sigma);
	x += distro(mGenerator);
	y += distro(mGenerator);
}

double gslCls::weibullh(double x, double scale, double shape) {
	return (shape / scale) * std::pow(x / scale, shape - 1);
}

double gslCls::lognormal(double Mu, double Sigma) {
	std::lognormal_distribution<> distro(Mu, Sigma);
	return distro(mGenerator);
}

double gslCls::gauss(double Sigma) {
	std::normal_distribution<> distro(Sigma);
	return distro(mGenerator);
}

double gslCls::gauss(double Mu, double Sigma) {
	std::normal_distribution<> distro(Mu, Sigma);
	return distro(mGenerator);
}

/// \returns a random number from the exponential distribution.
double gslCls::exponential(const double mean) {
	std::exponential_distribution<> distro(mean);
	return distro(mGenerator);
}

void gslCls::negBinomialParam(double mean, double variance, double& p, double& n) {
	if((mean <= 0) || (variance <= 0) || (mean > variance)) {
		cerr << __LINE__ << " " << __PRETTY_FUNCTION__ << " Range error." << endl;
		std::exit(1);
	}
	p = mean / variance;
	n = mean * p / (1 - p);
}

