/* This file is part of dengue: Dengue fever individual propagation model
Copyright (C) 2012 Harish Padmanabha, Fabio Correa

This project has been developed by Harish Padmanabha<harishpadmanabha@gmail.com> and Fabio Correa<facorread@gmail.com>
This code has been delivered to the Public Library of Science, on February 16, 2015.
We, the developers, grant you a license to review and modify this source code for your own use.
We do not grant you license to distribute or publish this program or any derivative work, any of its output, the ideas or algorithms contained within, in any form.
Anybody who wishes to get a copy of this source code or to get this project under a different license should email the developers directly.

*/

/// \file gsl.h This file provided interfaces to the GNU Scientific Library, but now it uses C++11 to provide numerical functions.

//Check for C++0x
#if (__cplusplus <= 199711L) && !defined(__GXX_EXPERIMENTAL_CXX0X__)
#error Please use a C++11 compiler, such as GCC-4.6 or later. On Linux, use a recent distribution. On Windows, use MinGW or Cygwin. On Mac OS, use macports.
#endif

/// \def _wur_
/// \brief Warns about unused results of functions
/// \details _wur_ can be used in most function declarations so that the compiler warns about unused results of these functions. _wur_ is automatically enabled on GCC only; in other compilers it should do nothing.
#if defined(__GXX_EXPERIMENTAL_CXX0X__)
#define _wur_ __attribute__ (( warn_unused_result ))
#else
#define _wur_
#endif

#include <iostream>
#include <fstream>
#include <random>
#include <vector>
#include <map>
#include <cstdlib>
#include "param.h"

class discreteArrayCls;
class gslCls;

/// Wrapper for a discrete probability distribution with underlying array. \ingroup facilities
class discreteArrayCls {
	public:
		/// Underlying array type.
		typedef std::map<int, int> histCls;
		/// Type for the optional distribution.
		typedef std::map<int, double> distroCls;
	private:
		/// Container for the histogram.
		histCls mHistogram;
		/// Optional distribution.
		distroCls mDistro;
		/// Histogram iterator.
		histCls::const_iterator mHistIt;
		/// Distribution iterator.
		distroCls::const_iterator mDistroIt;
	public:
		/// Resulting discrete distribution.
		std::discrete_distribution<> distro;
		/// Adds an element to the distribution. @param [in] a datum to add.
		void push(int a) { ++mHistogram[a]; }
		/// Processes the histogram for random variate generation.
		void set();
		/// Cleans the distribution and histogram.
		void clear() { mHistogram.clear(); mDistro.clear(); }
		/// Cleans the distribution and histogram to free some memory.
		void optimize() { mHistogram.clear(); mDistro.clear(); }
		/// Produces the optional distribution;
		void produceDistro();
		/// Sets the distribution iterator at the start of the distribution.
		void distroStart() { mDistroIt = mDistro.begin(); }
		/// Steps the distribution iterator.
		void distroStep() { ++mDistroIt; }
		/// Returns \c true if the iterator is at the end of the distribution, \c false if there are elements left to evaluate.
		bool distroCont() const _wur_ { return mDistroIt != mDistro.end(); }
		/// Returns whether the optional distribution is empty.
		bool distroEmpty() const _wur_ { return mDistro.empty(); }
		/// Sets the distribution iterator at the start of the distribution.
		void histStart() { mHistIt = mHistogram.begin(); }
		/// Steps the distribution iterator.
		void histStep() { ++mHistIt; }
		/// Returns \c true if the iterator is at the end of the distribution, \c false if there are elements left to evaluate.
		bool histCont() const _wur_ { return mHistIt != mHistogram.end(); }
		/// Returns whether the optional distribution is empty.
		bool histEmpty() const _wur_ { return mHistogram.empty(); }
		/// Prints the distribution. \param [in] outfile Output file.
		void print(std::ofstream& outfile) const;
		/// Returns the (x, probability) pair that is pointed by the distribution iterator.
		const distroCls::value_type& distroItem() const { return *mDistroIt; }
		/// Returns the (x, frequency) pair that is pointed by the histogram iterator.
		const histCls::value_type& histItem() const { return *mHistIt; }
};

/// Prints a set of distributions to a file. @param [in] outfile Output file. @param [in] distributions List of references to the distributions. \ingroup facilities
void print(std::ofstream& outfile, const std::vector<discreteArrayCls*>& distributions);

/// Prints a set of histograms to a file. @param [in] outfile Output file. @param [in] distributions List of references to the histograms. \ingroup facilities
void printHist(std::ofstream& outfile, const std::vector<discreteArrayCls*>& distributions);

/// Wrapper and service for the GNU Scientific Library. \ingroup facilities
class gslCls {
	private:
		/// Pointer to the random number generator.
		std::mt19937 mGenerator;
	public:
		/// Seeds the RNG with a non-deterministic random value.
		void seed();
		/// Seeds the RNG. @param [in] seed New seed for random number.
		void seed(const int seed);
		/// Seeds the RNG with an internal algorithm and some salt. @param [in] salt Number that is added to the internal calculated seed.
		void seedSalt(const std::mt19937::result_type salt);
		/// \name Discrete random number distributions
		/// @{ Functions that produce random numbers according to discrete probability distributions.

		/// Returns a random number from an arbitrary discrete distribution
		template<class discreteDistro = std::discrete_distribution<> >
		typename discreteDistro::result_type discrete(discreteDistro& a) { return a(mGenerator); }
		/// Returns a random number given by a rectangular distribution in the interval [a,b].
		double flat(const double a, const double b) _wur_;
		/// Returns an integer random number given by a rectangular distribution in the set (a, a + 1, ... , b - 1).
		size_t flat(const size_t a, const size_t b) _wur_;
		/// Returns an integer random number given by a rectangular distribution in the set (0, 1, ... ,  b - 1).
		size_t flat(const size_t b) _wur_;
		/// Returns an integer random number given by a rectangular distribution in the closed set ().
		size_t flatClosed(const paramIntervalCls<size_t>& a) _wur_;
		/// Returns a random number given by a rectangular distribution in the interval [mean-amplitude/2, mean+amplitude/2].
		double flatM(double mean, double amplitude) _wur_;
		/// Returns a number randomly selected between 0 and 1 given by a Bernoulli distribution with probability \c p .
		unsigned int bernoulli(double p) _wur_;
		/// Returns \c true with probability \c p .
		bool bern(double p) _wur_;
		/// Returns a random number given by a binomial distribution with probability \c p and \c n tries.
		unsigned int binomial(double p, unsigned int n) _wur_;
		/// Returns a random number given by a negative binomial distribution with probability \c p and \c n tries.
		unsigned int negBinomial(double p, double n) _wur_;
		/// Returns a random number given by a geometric distribution. \details The geometric distribution gives the number of trial which is the first success = negBinomial(p,1) + 1. The number is in the interval [1,inf).
		unsigned int geometric(double p) _wur_;
		/// Returns a random number according to the Poisson distribution.
		int poisson(const double p) _wur_;
		/// Returns an integer random number from the generator
		unsigned long int get() _wur_;
		/// @}

		/// \name Continuous random number distributions
		/// @{ Functions that produce random numbers according to continuous probability distributions.

		/// Returns a random number from a Beta distribution with shape parameters \c alpha and \c beta.
		double beta(double alpha, double beta) _wur_;
		/// Returns a random number given by a Weibull distribution.
		double weibull(double scale, double shape) _wur_;
		/// \brief Sets x and y displacements according to a bivariate Gaussian distribution with symmetric sigma.
		/// @param [out] xdiff Place to put the x displacement.
		/// @param [out] ydiff Place to put the y displacement.
		/// @param [in] sigma Standard deviation.
		void bivariate_gauss(double& xdiff, double& ydiff, const double sigma);
		/// \brief Displaces x and y according to a bivariate Gaussian distribution with symmetric sigma.
		/// @param [out] x X coordinate.
		/// @param [out] y Y coordinate.
		/// @param [in] sigma Standard deviation.
		void bigauss_displacement(double& x, double& y, const double sigma);
		/// Returns the Weibull hazard function evaluated on \c x.
		double weibullh(double x, double scale, double shape) _wur_;
		/// Returns a random number given by a lognormal distribution.
		//(Mean \f$ e^{\left(\mu + \sigma^2\right)/2} \f$, variance \f$ \left(e^{\sigma^2} -1 \right)e^{2\mu + \sigma^2} \f$).
		double lognormal(double Mu, double Sigma) _wur_;
		/// Returns a random number given by a normal distribution.
		//(Mean \f$ \mu \f$, Variance \f$ \sigma^2 \f$).
		double gauss(double Mu, double Sigma) _wur_;
		/// Returns a random number given by a normal distribution.
		//(Mean \f$ \mu \f$, Variance \f$ \sigma^2 \f$).
		double gauss(double Sigma) _wur_;
		/// Returns a Monte Carlo integral.
		double integral(double(*f)(double*, size_t, void*), double x1, double y1, double x2, double y2, void* params) _wur_;
		/// Returns a random number from the exponential distribution.
		double exponential(const double mean) _wur_;
		/// Parameterizes the GSL negative binomial distribution. @param [in] mean Mean of the data to parameterize. @param [in] variance Variance of the data. @param [out] p Probability of success. @param [out] n Expected number of successes.
		static void negBinomialParam(double mean, double variance, double& p, double& n);
		/// @}

		/// \name Distributions that select random elements from containers
		/// @{ Using random number distributions

		/// Returns a number from a rectangular distribution in the double interval (a.min(), a.max())
		template<class T> double flatd(const T& a) _wur_;
		/// Returns a random element from a container, using a flat distribution, not checking for container size. @param [in,out] container Element container.
		template<class T>
		typename T::value_type& insecureFlat(T& container) _wur_;
		/// Returns a random element from a container, using a flat distribution. @param [in,out] container Element container.
		template<class T>
		typename T::value_type& flat(T& container) _wur_;
		/// Returns a random element from a container, using a flat distribution, not checking for container size. @param [in,out] container Element container.
		template<class T>
		const typename T::value_type& insConstFlat(const T& container) _wur_;
		/// Returns a random element from a container, using a flat distribution. @param [in,out] container Element container.
		template<class T>
		const typename T::value_type& constFlat(const T& container) _wur_;
		/// Returns a random element from a sequential container, using a flat distribution. @param [in,out] container Element container.
		template<class T>
		const typename T::value_type& constSeqFlat(const T& container) _wur_;
		/// Returns an iterator pointing to a random element from a sequential container, using a flat distribution. @param [in,out] container Element container.
		template<class T> typename T::iterator iterSeqFlat(T& container) _wur_;
		/// Returns a random element from a list or similar container without operator[], using a flat distribution. @param [in,out] container Element container.
		template<class T> typename T::value_type& insSeqFlat(T& container) _wur_;
		/// Returns a random element from a map, using a flat distribution. @param [in,out] container Element container.
		template<class T>
		const typename T::mapped_type& constFlat2(const T& container) _wur_;
		/// @}

};

template<class T> double gslCls::flatd(const T& a) {
	return flat(a.min(), a.max());
}

template<class T>
typename T::value_type& gslCls::insecureFlat(T& container) {
	return container[flat(container.size())];
}

template<class T>
typename T::value_type& gslCls::flat(T& container) {
	const size_t size(container.size());
	if(size > 0)
		return container[flat(size)];
	else {
		cerr << __LINE__ << " " << __PRETTY_FUNCTION__ << " Empty container." << endl;
		std::exit(1);
	}
}

template<class T>
const typename T::value_type& gslCls::insConstFlat(const T& container) {
	return container[flat(container.size())];
}

template<class T>
const typename T::value_type& gslCls::constFlat(const T& container) {
	const int size(container.size());
	if(size)
		return container[flat(size)];
	else {
		cerr << __LINE__ << " " << __PRETTY_FUNCTION__ << " Empty container." << endl;
		std::exit(1);
	}
}

template<class T>
const typename T::value_type& gslCls::constSeqFlat(const T& container) {
	if(container.empty()) {
		cerr << __LINE__ << " " << __PRETTY_FUNCTION__ << " Empty container." << endl;
		std::exit(1);
	}
	typename T::iterator i(container.begin());
	for(size_t index(flat(container.size())); index; --index)
		++i;
	return *i;
}

template<class T> typename T::iterator gslCls::iterSeqFlat(T& container) {
	if(container.empty()) {
		cerr << __LINE__ << " " << __PRETTY_FUNCTION__ << " Empty container." << endl;
		std::exit(1);
	}
	typename T::iterator i(container.begin());
	for(size_t index(flat(container.size())); index; --index)
		++i;
	return i;
}

template<class T> typename T::value_type& gslCls::insSeqFlat(T& container) {
	typename T::iterator i(container.begin());
	for(size_t index(flat(container.size())); index; --index)
		++i;
	return *i;
}

template<class T>
const typename T::mapped_type& gslCls::constFlat2(const T& container) {
	const int size(container.size());
	if(!(size > 0)) {
		cerr << __LINE__ << " " << __PRETTY_FUNCTION__ << " Empty container." << endl;
		std::exit(1);
	}
	typename T::const_iterator i(container.begin());
	int index(flat(size));
	while(index > 0) {
		++i;
		--index;
	}
	return i->second;
}
