/* This file is part of dengue: Dengue fever individual propagation model
Copyright (C) 2012 Harish Padmanabha, Fabio Correa

This project has been developed by Harish Padmanabha<harishpadmanabha@gmail.com> and Fabio Correa<facorread@gmail.com>
This code has been delivered to the Public Library of Science, on February 16, 2015.
We, the developers, grant you a license to review and modify this source code for your own use.
We do not grant you license to distribute or publish this program or any derivative work, any of its output, the ideas or algorithms contained within, in any form.
Anybody who wishes to get a copy of this source code or to get this project under a different license should email the developers directly.

*/

//Check for C++0x
#if (__cplusplus <= 199711L) && !defined(__GXX_EXPERIMENTAL_CXX0X__)
#error This source code requires c++0x; it has been tested with GCC-4.4.4 and later.
#endif

#include <iostream>
#include <fstream>
#include <cmath>
#include <vector>
#include <list>
#include <set>
#include <algorithm>
#include "gsl.h"
#include "results.h"
#include "eta.h"

// Boost defines and includes
#include <boost/algorithm/string/predicate.hpp>
#include <boost/algorithm/string/trim.hpp>
#include <boost/accumulators/accumulators.hpp>
#include <boost/accumulators/statistics/stats.hpp>
#include <boost/accumulators/statistics/rolling_sum.hpp>
#include <boost/container/stable_vector.hpp>

using std::cout;
using std::cerr;
using std::endl;

/// Designates the health status of an individual.
enum class ePStatus {
	/// Susceptible.
	S,
	/// Exposed.
	E,
	/// Infected.
	I,
	/// Retired
	R
};

/// Designates the health status of a vector.
enum class eVStatus {
	/// Susceptible.
	S,
	/// Exposed.
	E,
	/// Infectious.
	I
};

// Template declarations
/// Contains results containers. @tparam C Contained class, generally an instantiation of this template. @tparam rangeCls Specifies the ranges.
template<class C, class rangeCls>
class tableCls;

//Class declarations
/// Represents the position of a mosquito or a person.
class pointCls;
/// Contains variables for the detection of post-epidemic times in each simulations.
class epiExtinctionCls;
/// Defines the basic measurements of a post-epidemic outbreak. @tparam targetPCR Minimum number of PCR+ people that defines the outbreak as a valid measurement.
template<const int targetPCR> class postEpiCls;
/// Daily measurement containers for all simulation objects. Their contents are cleared every day.
class dayMeasureCls;
/// Overall measurement containers for all simulation objects. Their contents are cleared every 5 years.
class fiveYearMeasureCls;
/// Overall measurement containers for all simulation objects. Their contents are cleared at the end of each simulation.
class overallMeasureCls;
/// Represents an introduction of the dengue infection.
class introCls;
/// Contains infection information, to be copied among persons and mosquitoes during risk bites.
class infectionInfoCls;
/// Represents a person.
class pCls;
/// Typedef for a vector of persons.
typedef std::vector<pCls*> pPersonVCls;
/// Represents a mosquito.
class vCls;
/// Represents the bounding box of a home, a block, or a town.
class envelopeCls;
/// Represents an age group for persons.
class ageGroupCls;
/// Represents a home.
class homeCls;
/// Container for daily results.
class dayResCls;
/// Container for results for an age group.
class ageGroupResCls;
/// Container for overall results, which are taken at the end of life or the end of each simulation instance.
class overallResCls;
/// Container for simulation results for each scenario.
class resultsCls;
/// Represents a simulation.
class simCls;

//Class definitions
/// Represents the position of a mosquito or a person.
class pointCls {
	public:
		/// X coordinate.
		double x;
		/// Y coordinate.
		double y;
		/// Constructor.
		pointCls(const double X, const double Y) : x(X), y(Y) {}
		/// Deafult constructor.
		pointCls() : x(0), y(0) {}
		/// Returns the distance of two points. @param [in] p1 First point. @param [in] p2 Second point.
		static double distance(const pointCls& p1, const pointCls& p2) _wur_;
};

/// Contains variables for the detection of post-epidemic times.
class epiExtinctionCls {
public:
	enum class stateCls {
		/// No epidemic has been detected.
		nonEpidemic,
		/// Epidemic is in progress.
		inProgress,
		/// Epidemic has extinguished but the town is not post-epidemic yet because the minimum number of days is not reached.
		extinguished,
		/// The town is in post-epidemic state.
		postEpidemic
	};
	/// Status of the epidemic.
	stateCls state;
	/// Day in which the simulation is declared postEpidemic.
	size_t postEpidemicDay;
	/// Default constructor initializes the status.
	epiExtinctionCls() : state(stateCls::nonEpidemic) {}
};

class dayMeasureCls {
	public:
		/// Total number of non-buffer persons.
		counterCls h;
		/// Susceptible persons in all the simulation.
		counterCls s;
		/// Number of persons in PCR+ state in all the simulation.
		counterCls pcr;
		/// Number of persons in PCR+ that are NOT introduced.
		counterCls pcr2plus;
		/// Number of PCR+ cases that appeared today.
		counterCls pcrCases;
		/// List of introduction IDs that generated the current PCR positives
		std::set<introCls> pcrIntroID;
		/// Default initializer.
		dayMeasureCls() {}
};

/// Overall measurement containers for all simulation objects. Their contents are cleared every 5 years.
class fiveYearMeasureCls {
	public:
		/// Total number of secondary and higher cases on people that is part of results.
		counterCls secondaryAndHigherpcrCases;
		/// Default initializer.
		fiveYearMeasureCls() {}
};

/// Overall measurement containers for all simulation objects. Their contents are cleared at the end of each simulation.
class overallMeasureCls {
	public:
		/// Days that there was no PCR+ people, from the PostEpidemic time that the infection was present in the simulation.
		counterCls pcr0days;
		/// Days that there were no PCR+ cases reported, from the PostEpidemic time that the infection was present in the simulation.
		counterCls pcr0casesDays;
		/// Total number of PCR+ cases.
		counterCls pcrCases;
		/// Days that PCR>=16.
		counterCls pcr16days;
		/// List of introduction IDs that generated days with 16+ PCR+
		std::set<introCls> pcr16intro;
		/// Days that PCR>=22.
		counterCls pcr22days;
		/// List of introduction IDs that generated days with 22+ PCR+
		std::set<introCls> pcr22intro;
		/// Number of post-epidemic introductions.
		counterCls postEpiIntroductions;
		/// Number of post-epidemic introductions into inhabitant persons.
		counterCls postEpiInhabitantIntroductions;
		/// PCR epidemic extinction measures.
		epiExtinctionCls PCRextinction;
		/// Counts the number of homes that produce more than 15 pupae.
		counterCls freq15pupae;
		/// Counts the number of homes that produce more than 15 pupae.
		counterCls freq50pupae;
		/// Counts the number of homes that produce more than 15 pupae.
		counterCls freq100pupae;
		/// Default constructor sets some values.
		overallMeasureCls() {}
};

/// Represents an introduction of the dengue infection.
class introCls {
	public:
		/// Specifies introduction types.
		enum class Type {
			noIntroduction,
			inhabitant,
			visitor1day,
			mosquito
		};
	private:
		/// Type of introduction.
		Type mType;
		/// ID of the mosquito/person that received the introduced infection.
		int mIntroducedID;
		/// Time of infection.
		size_t mIntroTime;
		/// Niche breadth at the day of introduction.
		double mNicheBreadth;
	public:
		/// Constructor. @param [in] type Type of introduction. @param [in] ID Identification of the individual that received the introduction. @param [in] introTime Time of introduction.
		explicit introCls(Type type, int ID, int introTime);
		/// Default constructor, set all variables to zero.
		explicit introCls() : mType(Type::noIntroduction), mIntroducedID(0), mIntroTime(0), mNicheBreadth(std::numeric_limits<double>::quiet_NaN()) {}
		/// Comparison function for associative containers.
		bool operator< (const introCls& a) const _wur_;
		/// Prints the heading of the introductionID. @param [in] outfile Output file.
		static void printHeading(std::ofstream& outfile) { outfile << "\tintroType\tintroID\tintroTime"; }
		/// Prints the current introductionID. @param [in] outfile Output file.
		void printValue(std::ofstream& outfile) const;
		/// Returns whether the primary infected individual was a person.
		bool isPerson() const _wur_ { return (mType == Type::inhabitant) || (mType == Type::visitor1day); }
		/// Returns whether the primary infected individual was an inhabitant person.
		bool isInhabitantPerson() const _wur_ { return mType == Type::inhabitant; }
		/// Returns whether the primary infected individual was a person.
		bool isMosquito() const _wur_ { return mType == Type::mosquito; }
		/// Returns the ID of the primary infected individual.
		int id() const _wur_ { return mIntroducedID; }
		/// Returns the time.
		size_t day() const _wur_ { return mIntroTime; }
		/// Returns whether the introduction is valid.
		bool isValid() const _wur_ { return mType != Type::noIntroduction; }
		/// Returns whether the introduction day is post-epidemic.
		bool isPostEpidemic() const _wur_;
		/// Returns the niche breadth at the introduction day.
		double nicheBreadth() const _wur_ { return mNicheBreadth; }
};

/// Contains infection information, to be copied among persons and mosquitoes during risk bites.
class infectionInfoCls {
	public:
		/// ID of the introduction that was transmitted to the current individual.
		introCls introID;
		/// Infection generation. Generation 1 introduced people and the mosquitoes infected by them. Generation 2 is secondary people and the mosquitoes infected by them, etc.
		size_t infectionGeneration;
		/// ID of the person that infected the mosquito that infected the current person; it is the person of the previous generation.
		int prevGenPersonId;
};

/// Represents a person.
class pCls {
	private:
		/// Defines the state variables of the person, which are updated every simulation step.
		class statusCls {
			public:
				/// Health status.
				ePStatus health;
				/// State variable of persons relating to viremia.
				bool pcr;
		};
		/// Stores the state variables of the person for the current day.
		statusCls mState;
		/// Stores the state variables of the person for the next day.
		statusCls mNewState;
		/// Contains infection information, to be copied among persons and mosquitoes during risk bites.
		infectionInfoCls mInfectionInfo;
		/// Stores whether the cases should be reported for the current health status.
		bool mMeasureCases;
		/// Day to change from exposed to infectious.
		size_t mIDay;
		/// Inhabitant weight for mosquito bites, which is different for a person that goes out of home, than for a person that stays at home, than for other special cases.
		double mInhabitantW;
		/// Pointer to the home
		homeCls* pHome;
		/// Permanent set of home contacts.
		std::vector<homeCls*> mHomeSet;
		/// Unique ID
		int mID;
		/// Age in days.
		int mAgeDays;
		/// Age index to age group.
		size_t mAgeIndex;
		/// Determines whether the person should be removed for death or other reason.
		bool mRemove;
		/// Bernoulli probability distance-independent component for choosing homes for the permanent set of contacts.
		indPCls <> pIndep;
		/// Incubation time
		indPCls <> t_h;
		/// Recovery probability, from viremic into recovered state.
		indPCls <> p_r;
		/// Probability of the PCR becoming negative.
		indPCls <> p_pcroff;
		/// Parameter of the Poisson distribution of number of visits that \c this person does in each day.
		indPCls<> nVisitsLambda;
		/// Returns whether the person is older than 15 years.
		bool olderThan15() const _wur_ { return mAgeDays >= 15 * 365; }
		/// Samples individual parameters
		void sampleindpars();
		/// Prepares the value of lambda for daily visits.
		void nVisitsLambdaPrepare();
		/// Calculates the age index.
		void calculateAgeIndex();
		/// Updates the age index.
		void updateAgeIndex();
	public:
		/// Death probability for each age group, as reported by Harish.
		static std::vector<double> deathP;
		/// Fraction variation of deathP for each trial.
		static paramIntervalCls<> deathPvariationPerTrial;
		/// Current death probability as fraction of deathP(field); same fraction for all age groups.
		static paramCls<> deathPfraction;
		/// Death probability for each age group, for the current trial.
		static std::vector<double> deathPcurrent;
		/// Limits to the Bernoulli probability distance-independent component for choosing homes for the permanent set of contacts
		static paramIntervalCls<> pIndepPerP;
		/// Mean Bernoulli probability constant for choosing homes for the permanent set of contacts.
		static paramCls <> Cd;
		/// Probability weight of visiting the booked home each day.
		static paramCls <> visitW;
		/// Limits to the incubation time distribution across persons.
		static paramIntervalCls<> t_h_perP;
		/// Per person rectangular distribution limits to the daily probability of a person recovering from the infectious state.
		static paramIntervalCls<> p_r_perP;
		/// SEIS model is used instead of the traditional SEIR model.
		static paramCls<bool> seis;
		/// All infections are allowed, not just secondary.
		static paramCls<bool> allInfections;
		/// Per person rectangular distribution limits of time of recovering into the PCR- state.
		static paramIntervalCls<> p_pcroff_perP;
		/// Individual probability of getting infected in a bite from an infectious mosquito
		static paramCls<> p_inf_h;
		/// Rate parameter of the Poisson distribution of number of visits that a young person does in each day
		static paramCls<> nVisitsLambdaYoung;
		/// Rate parameter of the Poisson distribution of number of visits that an adult person does in each day
		static paramCls<> nVisitsLambdaAdult;
		/// Contains pointers to all persons in the town to which the infection can be introduced.
		static pPersonVCls introducible;
		/// Last infection generation.
		static size_t maxIgeneration;
		/// ID counter
		static int idCounter;
		/// General purpose calculator of minimum \c int values.
		static intMinCls mIntMin;
		/// Holds the count of the size of each age group.
		static std::vector<counterCls> ageGroupHcount;
		/// Prepares the values of the death probability.
		static void deathPprepare();
		/// \return the value of the pIndep probability component.
		double pindep() const _wur_ { return pIndep.value(); }
		/// \return the value of the Cd probability component.
		double cd() const _wur_ { return Cd.value(); }
		/// If the person is infectious, it returns a pointer to the infection info. Otherwise, it returns \c nullptr.
		const infectionInfoCls* isInfectious() const _wur_;
		/// Returns whether the person is in exposed of infectious state.
		bool hasInfection() const _wur_ { return (mState.health == ePStatus::E) || (mState.health == ePStatus::I); }
		/// Returns whether the person is in exposed of infectious state, or is PCR+.
		bool hasInfectionOrPCR() const _wur_ { return (mState.health == ePStatus::E) || (mState.health == ePStatus::I) || mState.pcr; }
		/// Returns the health status of the person.
		ePStatus health() const _wur_ { return mState.health; }
		/// Returns the age.
		int ageDays() const _wur_ { return mAgeDays; }
		/// Changes the age and corresponding age index.
		void ageDays(int newAgeDays, size_t newAgeIndex) { mAgeDays = newAgeDays; mAgeIndex = newAgeIndex; }
		/// Prepares the homes to visit.
		void setVisitHomes();
		/// Books visits to homes in the mHomeSet.
		void bookVisits();
		/// Performs the consequences of a bite action in which the mosquito is viremic. @param [in] info Mosquito and infection information.
		void riskBite(const infectionInfoCls& info);
		/// Performs interactions among individuals.
		void interact();
		/// Advances the simulation on one step depending on the clock settings.
		void simStep();
		/// Constructor sets the initial age and home. If \c ageDays >= 0, which is the normal town construction, the home represents the premise in which the person resides. If \c ageDays < 0, which corresponds to the infection introduction in a one-day visit, the home represents the home that \c this is visiting and mHome is set to \c nullptr because it is irrelevant. \param [in] home Home as described above. @param [in] ageDays Initial age of the person, as described above.
		pCls(homeCls* const home, const int ageDays);
		/// Returns whether the person is subject to surveys.
		bool isPartOfResults() const _wur_;
		/// Returns unique human ID
		int id() const {return mID;};
		/// \returns a reference to the home set.
		std::vector<homeCls*>& homeSet() _wur_ { return mHomeSet; }
		/// Returns a pointer to the home
		homeCls* ownHome() const { return pHome; }
		/// Introduction of the infection by deterministically changing the status to infectious (Base function).
		void deterministicIntroIbase();
		/// Introduction of the infection by deterministically changing the status to infectious.
		void deterministicIntroI();
		/// Prepares the set of pointers to persons that can receive the primary infection.
		void prepareIntroducible();
		/// \return the infection generation of the person.
		int infectionGeneration() const _wur_ { return mInfectionInfo.infectionGeneration; }
		/// Surveys the health status of persons.
		void measureDay();
		/// Reports results for all the life cycle and the end of simulation; these are results that are related to the static containers.
		void measureOverall() const;
		/// Returns the visit weight of the person for mosquito bites.
		static double visitw() _wur_ { return visitW.value(); }
		/// Returns the inhabitant weight of the person for mosquito bites.
		double inhabitantw() const _wur_ { return mInhabitantW; }
		/// Returns whether the person should be removed for death or another reason.
		bool remove() const { return mRemove; }
		/// Prepares the removal of the person from the simulation.
		void removePrepareBase();
		/// Prepares the removal of the person from the simulation, and removes the person from her home.
		void removePrepare();
		/// Returns the first day of viremia.
		size_t viremiaDay() const _wur_ { return mIDay; }
		/// Returns the infection info.
		const infectionInfoCls& infectionInfo() const _wur_ { return mInfectionInfo; }
		/// Returns the age index corresponding to simCls::age and simCls::ageDays.
		size_t ageIndex() const _wur_ { return mAgeIndex; }
		/// Sets the visits lambda.
		void setVisitsLambda(const double lambda) { nVisitsLambda.set(lambda); }
		/// Returns the visits lambda.
		double visitsLambda() const _wur_ { return nVisitsLambda.value(); }
		/// Calculates the age group sizes.
		void ageGroupH() const { if(isPartOfResults()) ++ageGroupHcount[mAgeIndex]; }
};

class vCls {
	private:
		/// Defines the state variables of the mosquito, which are updated every simulation step.
		class statusCls {
			public:
				/// Health status.
				eVStatus health;
				/// \c true if the mosquito is seeking blood, \c false if it is in the gravid state.
				bool bloodSeeking;
				/// Mosquito position. Needs to be based on double for quick GSL processing.
				pointCls position;
				/// Pointer to the home in which the mosquito is
				homeCls* pHome;
		};
		/// Stores the state variables of the mosquito for the current day.
		statusCls mState;
		/// Stores the state variables of the mosquito for the next day.
		statusCls mNewState;
		/// Contains infection information, to be copied among persons and mosquitoes during risk bites.
		infectionInfoCls mInfectionInfo;
		/// Day to change from exposed to infectious.
		size_t mIDay;
		/// Day of the next blood seeking status.
		size_t mBloodSeekingDay;
		/// Determines whether the mosquito should be removed because it moves to another home.
		bool mRemove;
		/// Unique integer ID
		int mID;
		/// Incubation time
		indPCls <> t_m;
		/// Number of trials in the binomial distribution of bites.
		indPCls <size_t> biteLimit;
		/// Probability of the host-seeking female to get full (successful bite) and become gravid.
		indPCls <> p_full;
		/// Bernoulli probability of mosquito survival each day.
		indPCls <> s_v;
		/// Marks the mosquito for removal.
		void markForRemoval() { mRemove = true; }
		/// Returns the unique ID of the current mosquito home
		int pHomeID() const;
		/// Performs displacement of the mosquito.
		void fly();
		/// Returns the health status.
		eVStatus status() const _wur_ { return mState.health; }
		/// Samples individual parameters
		void sampleindpars();
		/// \returns whether the mosquito is in the non-buffer area.
		bool isPartOfResults() const _wur_;
	public:
		/// Incubation time
		static paramCls<> t_m_mean;
		static paramCls<> t_m_width;
		/// Gonotrophic time: Time the mosquito stays in gravid status
		static paramCls<> g;
		/// Number of trials in the binomial distribution of bites, per mosquito.
		static paramIntervalCls<size_t> biteLimitPerV;
		/// Probability of the host-seeking female to get full (successful bite) and become gravid.
		static paramCls<> p_full_mean;
		static paramCls<> p_full_width;
		/// Probability that each mosquito that bites a viremic person to become infected
		static paramCls<> p_inf_m;
		/// Bite probability for the same person as in the latest bite.
		static paramCls<> p_same;
		/// Standard deviation of the Gaussian flight distribution. Needs to be double for quick GSL processing.
		static paramCls<> sigma;
		/// Sigma value for houses that have no useful containers
		static paramCls<> sigma_mean;
		static paramCls<> sigma_width;
		/// Bernoulli probability of mosquito survival each day.
		static paramCls<> s_v_Alpha;
		static paramCls<> s_v_Beta;
		static paramCls<> s_v_mean;
		static paramCls<> s_v_width;
		/// ID counter, to maintain uniqueness
		static int idcounter;
		/// General purpose home pointer
		static homeCls* staticPHome;
		/// Introduction of the infection by deterministically changing the status to infectious.
		void deterministicIntroI();
		/// Returns whether the mosquito is blood-seeking infectious
		bool isInfectious() const _wur_	{ return mState.bloodSeeking && (mState.health == eVStatus::I); }
		/// Returns whether the mosquito is exposed of infectious.
		bool hasInfection() const _wur_ { return (mState.health == eVStatus::E) ||  (mState.health == eVStatus::I); }
		/// Performs mosquito interactions.
		void interact();
		/// Steps mosquito evolution one day.
		void simStep();
		/// Sets the home pointer
		void setHome(homeCls* home);
		/// Reports results for all the life cycle.
		void measureOverall();
		/// Reports results for the current day.
		void measureDay();
		/// Constructor sets the age and initial status.
		vCls(homeCls* home);
		/// Returns the position
		const pointCls& position() const { return mState.position; }
		/// Returns the intended flight destination
		const pointCls& newPosition() const { return mNewState.position; }
		/// Returns unique mosquito ID
		int id() const {return mID;};
		/// Returns a pointer to the current mosquito home
		const homeCls* home() const _wur_ { return mState.pHome; }
		/// Returns whether the mosquito can have dengue infection introduced, and belongs to the chosen infection home, as specified in \c vCls::staticPHome.
		bool isIntroducible() const _wur_ { return (mState.health == eVStatus::S) && (mState.pHome == staticPHome); }
		/// Returns whether the mosquito should be removed because it moves to another home.
		bool remove() const { return mRemove; }
};

/// \brief Represents the bounding box of a home, a block, or a town.
/// This bounding box is optimized for determining whether a mosquito or a person is located inside the bounding box. For this class to be efficient, the stored data is the box center, its width, and height.
class envelopeCls {
	private:
		/// Center of the envelope.
		pointCls mCenter;
		/// Half width of the envelope, in the x axis.
		double mHalfWidth;
		/// Half length of the envelope, in the y axis.
		double mHalfLength;
	public:
		/// Constructor sets the initial coordinates.
		envelopeCls(pointCls center, double width, double length) : mCenter(center), mHalfWidth(0.5 * width), mHalfLength(0.5 * length) {}
		/// Default constructor.
		envelopeCls() : mCenter(), mHalfWidth(0), mHalfLength(0) {}
		/// Returns a reference to the center of the envelope.
		pointCls& center() _wur_ { return mCenter; }
		/// Returns a const reference to the center of the envelope.
		const pointCls& center() const _wur_ { return mCenter; }
		/// Returns the minimum coordinate in the x axis
		double minX() const _wur_ { return mCenter.x - mHalfWidth; }
		/// Returns the maximum coordinate in the x axis
		double maxX() const _wur_ { return mCenter.x + mHalfWidth; }
		/// Returns the minimum coordinate in the y axis
		double minY() const _wur_ { return mCenter.y - mHalfLength; }
		/// Returns the maximum coordinate in the y axis
		double maxY() const _wur_ { return mCenter.y + mHalfLength; }
		/// Expands the envelope to include another envelope, from the perspective of the X coordinates only.
		void expandToIncludeX(const envelopeCls& rhs);
		/// Expands the envelope to include another envelope, from the perspective of the Y coordinates only.
		void expandToIncludeY(const envelopeCls& rhs);
		/// Expands the envelope to include another envelope.
		void expandToInclude(const envelopeCls& rhs);
		/// Returns whether a point is contained within \c this. @param [in] p Point.
		bool contains(const pointCls& p) const _wur_;
		/// Draws the envelope. @param [in] color Color of the envelope border.
		void draw(const std::string& color = "black") const;
};

/// Represents an age group for persons.
class ageGroupCls {
	private:
		/// Age groups that are being used in the simulation: index: age index; int data: population for each age index.
		std::vector<int> mGroup;
		/// Sum of the population in the age group.
		counterCls mSum;
		/// Pointer to the vector of ages.
		std::vector<int>* pAge;
	public:
		/// Default constructor sets the vector of ages to nullptr.
		ageGroupCls() : pAge(nullptr) {}
		/// Constructor sets the vector of ages.
		ageGroupCls(std::vector<int>& age) : pAge(&age) {}
		/// Reserves space.
		void reserve(size_t space) { mGroup.reserve(space); }
		/// Registers the population for the next age group.
		void operator()(int persons) { mGroup.emplace_back(persons); mSum += persons; }
		/// Registers a zero population datum.
		void zero() { mGroup.emplace_back(0); }
		/// Returns the total number of persons.
		int H() const { return mSum.value(); }
		/// Merges another ageGroupCls element to \c this. @param [in] a Element to emerge into \c this. @param [in] skip Number of elements from a to skip from the start. Useful for specialized algorithms.
		void merge(const ageGroupCls& a, size_t skip);
		/// Returns \c true if there is no age data.
		bool empty() const { return mGroup.empty(); }
		/// Clears the age group.
		void clear() { mGroup.clear(); mSum.start(); }
		/// Typedef for the iterator.
		typedef std::vector<int>::iterator iterator;
		/// Typedef for the const iterator.
		typedef std::vector<int>::const_iterator const_iterator;
		/// Returns an iterator to the begining of the age groups.
		iterator begin() { return mGroup.begin(); }
		/// Returns a const iterator to the end of the age groups.
		iterator end() { return mGroup.end(); }
		/// Returns \c true if the number of persons does not match \c expectedH. @param [in] expectedH Expected number of persons in the age group.
		bool operator!=(const int expectedH) const _wur_ { return mSum.value() != expectedH; }
		/// Returns an iterator to the beginning of the age vector.
		std::vector<int>::iterator ageBegin() const { return pAge->begin(); }
		/// Returns an iterator to the end of the age vector.
		std::vector<int>::const_iterator ageEnd() const { return pAge->end(); }
		/// Prints the age group to the specified output file. @param [in] outfile Output file.
		void print(std::ofstream& outfile) const;
};

/// Represents a home.
class homeCls {
	public:
		/// Typedef for a distance-dependent visit zone
		typedef std::vector<homeCls*> visitzCls;
	private:
		/// Home identification.
		int mId;
		/// \c true if the home is non-buffer, part of results.
		bool mIsPartOfResults;
		/// Pointers to home inhabitants.
		std::list<std::list<pCls>::iterator> mInhabitant;
		/// Array of persons scheduled for visit
		std::vector<pCls*> mVisit;
		/// Home envelope, specifying the center and bounding box
		envelopeCls mEnvelope;
		/// Age groups in the home. size() = 0 means not censated.
		ageGroupCls mAgeGroups;
		/// Number of pupae for the home.
		int mNPupae;
		/// Number of containers
		int mNContainers;
		/// Number of positive containers, with larvae
		int mNPositiveContainers;
		/// Distribution of bite probabilities considering inhabitants and visitors
		std::discrete_distribution<> mVisitDistro;
		/// Points to the visit zone of the home
		visitzCls* pVisitZone;
		/// Poisson rate of female mosquito emergences each day at each home.
		indPCls<> b_v;
		/// Annual birth probability per inhabited home
		static indPCls<> yearBirthP;
	public:
		/// Home length for theoretical simulations.
		static paramCls<> theoLength;
		/// Home half-length for theoretical simulations.
		static paramCls<> theoHalfLength;
		/// Probability of a produced pupa to be female (males are discarded from the sim).
		static paramCls<> femaleP;
		/// # days until pupal emergence
		static paramCls<> pupalemergence;
		/// Fraction of pupae that survive
		static paramCls<> pupalsurvival;
		/// Neighborhood niche breadth timeline.
		static paramArrayCls<> nicheBreadthTL;
		/// Timeline of the probability of infection introduction, with time interval 3 weeks.
		static paramArrayCls<> p_introTL;
		/// Connectivity coefficient to the probability of infection introduction.
		static paramArrayCls<> p_introConn;
		/// Connectivity coefficient to the probability of infection introduction.
		static paramCls<> p_intro;
		/// Mean years that a family dwells in a home. After this, they are completely replaced by a new family.
		static paramArrayCls<> t_dwell;
		/// Joint scenario number for p_introConn and yearBirthP.
		static paramArrayCls<> scenarioNumber;
		/// Probability of a family to move out of the town and be replaced by a new family
		static paramCls<> p_migration;
		/// # days after emergence before mosquito disperses
		static paramCls<> refr_non_disp;
		/// Additional # days mosquito can disperse before becoming host-seeking
		static paramCls<> disp_non_biting;
		/// Per-trial, rectangular distribution of annual birth probability per inhabited home
		static paramIntervalCls<> yearBirthPacrossTrials;
		/// Daily birth probability per inhabited home
		static paramCls<> birthP;
		/// Prepares the value of the p_migration parameter
		static void p_migrationPrepare() { p_migration.set((t_dwell.value() == std::numeric_limits<double>::max()) ? 0 : 1.0 / (365.0 * t_dwell.value())); }
		/// Prepares the value of the birthP parameter
		static void birthPprepare();
		/// Prepares the value of the p_intro parameter.
		static void p_introPrepare() { p_intro.set(p_introConn.value() * p_introTL.value()); }
		/// Prepares the value of the p_intro parameter at the start of a simulation or measurement.
		static void p_introStart() { nicheBreadthTL.start(); p_introTL.start(); p_introPrepare(); }
		/// Prepares the value of the p_intro parameter at a time step.
		static void p_introStep();
		/// Constructor for the theoretical case. @param [in] id Home identification. @param [in] vgroup Associated mosquito group. @param [in] minx west x coordinate. @param [in] miny south y coordinate. @param [in] maxx east x coordinate. @param [in] maxy north y coordinate.
		homeCls(const int id, const envelopeCls& envelope, const bool isPartOfResults);
		/// Fills in missing data from field data.
		void fill();
		/// Sets the adult production from the distributions.
		void setEmerges();
		/// Assigns an adult the lambda of young persons.
		void assignYoungLambda();
		/// Creates the group of persons of a home, according to the age groups.
		void createPersons();
		/// Restarts the home.
		void start();
		/// Returns whether a mosquito will be inside the home.
		bool willBeInside(vCls& mosquito) const;
		/// Tells the home the location of its visit zone.
		void setVisitZone(visitzCls* zone) { pVisitZone = zone; }
		/// \return the pointer to the visit zone
		visitzCls* visitZone() const { return pVisitZone; }
		/// Performs interactions, adjusts visit distributions.
		void interact();
		/// Returns the yearly birth rate.
		static double yearbirthP() _wur_ { return yearBirthP.value(); }
		/// Birth of one person.
		void birth();
		/// Advances the simulation on one step depending on the clock settings.
		void simStep();
		/// Returns the position of the home
		const pointCls& position() const _wur_ { return mEnvelope.center(); }
		/// Returns the envelope of the home
		const envelopeCls& envelope() const _wur_ { return mEnvelope; }
		/// Reports the position or ID of the home to cerr.
		void reportPosition() const;
		/// Returns a random person from the home.
		pCls* randomPerson() _wur_;
		/// Returns the number of persons
		int H() const { return mInhabitant.size(); }
		/// \returns whether the home has persons or visitors.
		bool hasPeople() const _wur_ { return !(mInhabitant.empty() && mVisit.empty()); }
		/// \returns whether the home has inhabitants.
		bool hasInhabitants() const _wur_ { return !mInhabitant.empty(); }
		/// Draws the home.
		void draw() const;
		/// Books a visit to \c this for the current day. @param [in] p Person who is coming over.
		void bookVisit(pCls* const p);
		/// Reports results for the current day.
		void measureDay();
		/// Reports results for the current scenario.
		void measureOverall() const;
		/// \return the home identification.
		int id() const _wur_ { return mId; }
		/// Returns whether the persons and mosquitoes inside the home are part of the simulation results.
		bool isPartOfResults() const _wur_ { return mIsPartOfResults; }
		/// Saves survey data. @param [in] ageGroups Age groups for the home. @param [in] nPupae Number of pupae. @param [in] nContainers Number of containers. @param [in] nPositiveContainers Number of positive containers.
		void save(ageGroupCls&& ageGroups, const int nPupae, const int nContainers, const int nPositiveContainers);
		/// Removes a person from the inhabitants of the home. @param [in] person Reference to the person to remove.
		void remove(pCls* person);
		/// Introduces an infection to the home.
		void introI();
		/// Returns the current number of pupae produced by containers at \c this home.
		int nPupae() const _wur_ { return mNPupae; }
};

/// Container for daily results.
class dayResCls {
	public:
		/// Fraction of susceptibles.
		meanMeasureCls sFrac;
		/// Number of persons in PCR+ state.
		meanMeasureCls pcr;
};

/// Container for overall results, which are taken at the end of life or the end of each simulation instance.
class overallResCls {
	public:
		/// 1 if the scenario is post-epidemic, 0 if no post-epidemic could be achieved.
		meanMeasureCls postEpidemic;
		/// 1 if the post-epidemic condition was achieved before 10 years, 0 otherwise.
		meanMeasureCls postEpidemic10yr;
		/// Time duration of the pre-epidemic time. Day that the post-epidemic was confirmed.
		meanMeasureCls preEpidemicTime;
		/// Fraction of time that there was no PCR+ people, respect to the total time of the infection: all generations until extinction.
		meanMeasureCls pcr0days;
		/// Fraction of time that there were no PCR+ cases reported, respect to the total time of the infection: all generations until extinction.
		meanMeasureCls pcr0casesDays;
		/// Total number of PCR+ cases.
		meanMeasureCls pcrCases;
		/// Number of days with PCR>=16.
		meanMeasureCls pcr16days;
		/// Average neighborhood niche breadth of days with PCR>=16.
		meanMeasureCls pcr16nicheBreadth;
		/// Average number of introduction IDs responsible for the PCR+ on days that 16 or more PCR+ were reported.
		meanMeasureCls pcr16introductions;
		/// Number of days with PCR>=22.
		meanMeasureCls pcr22days;
		/// Average neighborhood niche breadth of days with PCR>=22.
		meanMeasureCls pcr22nicheBreadth;
		/// Average number of introduction IDs responsible for the PCR+ on days that 22 or more PCR+ were reported.
		meanMeasureCls pcr22introductions;
		/// Number of post-epidemic introductions.
		meanMeasureCls postEpiIntroductions;
		/// Number of post-epidemic introductions into inhabitant persons.
		meanMeasureCls postEpiInhabitantIntroductions;
		/// Counts the number of homes that produce more than 15 pupae.
		meanMeasureCls freq15pupae;
		/// Counts the number of homes that produce more than 15 pupae.
		meanMeasureCls freq50pupae;
		/// Counts the number of homes that produce more than 15 pupae.
		meanMeasureCls freq100pupae;
		/// Number of visits done per person per day.
		meanMeasureCls nVisits;
		/// Birth rate of the trial.
		meanMeasureCls yearBirthRate;
		/// Current death probability as fraction of deathP(field); same fraction for all age groups.
		meanMeasureCls deathPfraction;
		/// Number of persons at the start of simulation, start of the pre-epidemic time.
		meanMeasureCls h0;
		/// Number of persons at the end of simulation.
		meanMeasureCls hFinal;
		/// Proportional human population variation from beginning to end of simulation.
		meanMeasureCls hVariationFrac;
		/// Typedef for a set of results for each age group.
		typedef std::vector<meanMeasureCls> ageGroupResCls;
		/// Population per age group, for times 0, 10 years, 20 years. Respective indices 0, 1, 2.
		std::vector<ageGroupResCls> ageGroup;
		/// Set of measurements that are taken every 5 years.
		class fiveYearResCls {
			public:
				/// Total number of secondary and higher cases on people that is part of results.
				meanMeasureCls secondaryAndHigherpcrCases;
				/// Average number of introduction IDs responsible for the PCR+ on days that 22 or more PCR+ were reported.
				meanMeasureCls pcr22introductions;
		};
		/// Typedef for a set of measurements that are taken every 5 years.
		typedef std::list<fiveYearResCls> fiveYearResVectorCls;
		/// Set of measurements that are taken every 5 years.
		fiveYearResVectorCls fiveYearMeasurementVector;
};

/// Container for simulation results for each scenario.
class resultsCls {
	public:
		/// Daily results.
		std::vector<dayResCls> dayRes;
		/// Overall results.
		overallResCls overallRes;
		/// Initializes results containers.
		void init();
		/// \return that this is the results class.
		static bool isTableCls() _wur_ { return false; }
		/// \return itself, for recursion.
		resultsCls& result() _wur_ { return *this; }
		/// For recursion.
		static void start() {}
		/// For recursion.
		static void start(const std::initializer_list< void* >&) {}
		/// For recursion.
		static bool step() _wur_ { return true; }
		/// For recursion.
		static bool step(const std::initializer_list< void* >&) _wur_ { return true; }
		/// For recursion.
		static bool stepOmitted(const std::initializer_list< void* >&) _wur_ { return true; }
		/// For recursion.
		static void printValue(std::ofstream&, const bool) {}
		/// For recursion.
		static void printHzParamName(std::ofstream&, const size_t, const std::initializer_list< void* >&) {}
		/// For recursion.
		static bool printHzParamValue(std::ofstream&, size_t, const size_t, const std::initializer_list< void* >&) _wur_ { return true; }
		/// For recursion.
		static std::string fileNameValue() _wur_ { return std::string(); }
		/// For recursion.
		static void produceFileName(const std::initializer_list< void* >&) {}
		/// For recursion.
		static void produceFileName(std::stringstream&, const std::initializer_list< void* >&) {}
		/// For recursion.
		static void printHeading(std::ofstream&, const bool) {}
		/// For recursion.
		static void info() {}
		/// For recursion.
		static int nScenarios() _wur_ { return 1; }
		/// For recursion. WOW!
		static bool cont() _wur_ { return false; }
};

/// Results typedef for simCls::patch
typedef tableCls<resultsCls, paramArrayCls<int> > results1Cls;
/// Results typedef for homeCls::scenarioNumber
typedef tableCls<results1Cls, paramArrayCls<> > results2Cls;
/// Results typedef for simCls::trial
typedef tableCls<results2Cls, paramRangeCls<int> > results3Cls;

class simCls {
	private:
		/// Results container.
		results3Cls mRes;
		/// Mosquitoes
		std::list<vCls> mMosquito;
		/// People
		std::list<pCls> mPerson;
		/// Results pointer.
		resultsCls* pRes;
		/// Day results pointer.
		std::vector<dayResCls>::iterator pDayRes;
		/// Overall results pointer.
		overallResCls* pOverallRes;
		/// Holds resampling pools and other data for an input patch. @tparam cont Type of container of data, such as \c std::vector or \c std::list.
		template< template<class T, class Alloc = std::allocator<T> > class cont > class perPatchCls {
			public:
				/// Homes.
				std::list<homeCls> mHome;
				/// Resampling pool for the number of people according to age groups.
				cont<ageGroupCls> ageGroups;
				/// Resampling pool for the number of pupae for positive containers.
				cont<int> nPupaePosConts;
				/// Resampling pool for the number of water containers.
				cont<int> nContainers;
				/// Resampling pool for the number of positive containers.
				cont<int> nPositiveContainers;
				/// Auxiliary function to move data from a temporary pool into their definitive, fast read equivalents.	@tparam destCls Type of the destination container, generally std::vector. @tparam srcCls Type of the source container, typically std::list. @param [out] dest Destination container. @param [in] source Source container, will be emptied.
				template<class destCls, class srcCls> static void move(destCls& dest, srcCls&& source) {
					dest.resize(source.size());
					std::move(source.begin(), source.end(), dest.begin());
				}
				/// Moves the pool data from a temporary container into its definitive, fast read equivalent, so this is \c poolCls<std::vector>. @tparam srcCls Class of the source container, typically \c poolCls<std::list>.
				template<class srcCls> void movePools(srcCls&& source) {
					move(ageGroups, std::move(source.ageGroups));
					move(nPupaePosConts, std::move(source.nPupaePosConts));
					move(nContainers, std::move(source.nContainers));
					move(nPositiveContainers, std::move(source.nPositiveContainers));
				}
		};
		/// Typedef for holding homes, resampling pools and other data for each input patch.
		typedef std::map<int, perPatchCls<std::vector> > patchMap;
		/// Holds homes, resampling pools and other data for each input patch.
		patchMap mPerPatch;
		/// Points to the homes, resampling pools and other data for the current input patch.
		patchMap::mapped_type* pPatch;
		/// Holds the number of persons at the start of pre-epidemic time.
		int mH0;
		/// Holds the time that the post-epidemic time started, from the beginning of the sim; it is the length of the pre-epidemic time.
		int mPreEpidemicTime;
		/// Typedef for a set of results for each age group.
		typedef std::vector<int> ageGroupSizeCls;
		/// Holds the age group sizes for each of the age indices, as defined in the documentation of overallResCls::ageGroup.
		std::vector<ageGroupSizeCls> mAgeGroupSizeV;
		/// Contains five-year results to be held until measureOverall();
		std::vector<overallResCls::fiveYearResCls> mFiveYearRes;
		/// \returns the distance between two homes. @param [in] h1 First home. @param [in] h1 Second home.
		static double distance(homeCls& h1, homeCls& h2) _wur_;
		/// Group of distance-dependent visit zones
		std::vector<homeCls::visitzCls> mVisitZone;
		/// Front width of homes
		static paramCls <> homefront;
		/// Side width of homes
		static paramCls <> homeside;
		/// Simulation time of the model (in days).
		static paramCls<size_t> simTime;
		/// Parameters to be omitted from the TL horizontal outputs, becoming the spreadsheet identifiers
		static const std::initializer_list< void* > TLomitParams;
		/// Maintains the ETA calculation.
		etaCls mETA;
		/// Envelope for the simulation, determines the limits of mosquito flights
		envelopeCls mEnvelope;
		/// Auxiliary function to ignore a number of cells from survey files. @param [in] input Survey file. @param [out] nFields Number of fields to ignore.
		static void ignore(std::ifstream& input, int nFields);
		/// Auxiliary function to parse a cell from survey files. @tparam dataT Data type that is expected to be read from the survey file. @param [in] input Survey file. @param [out] result Variable that will hold the result if data is read.
		template<class dataT> static void readField(std::ifstream& input, dataT& result);
		/// Auxiliary function to parse a cell from survey files: specialization for strings. @tparam dataT Data type that is expected to be read from the survey file. @param [in] input Survey file. @param [out] result Variable that will hold the result if data is read.
		static void readField(std::ifstream& input, std::string& result);
		/// Auxiliary function to parse a cell from survey files: specialization for bool. @tparam dataT Data type that is expected to be read from the survey file. @param [in] input Survey file. @param [out] result Variable that will hold the result if data is read.
		static void readField(std::ifstream& input, bool& result);
		/// Auxiliary function to seek to the beginning of the next line of a survey file. @param [in] input Survey file.
		static void nextLine(std::ifstream& input);
		/// Reads field data from surveys files.
		void readSurveyFiles();
		/// Prints the description of a simulation.
		void printDescription() const;
		/// Starts a new table.
		void printTLheading();
		/// Prepares the simulations pointers according to the scenario needs.
		void setResultsPointers();
		/// Prepares the simulation for each scenario.
		void scenPrepare();
		/// Initializes the scenario-related variables.
		void scenStart();
		/// Prepares the next scenario of the program. \returns whether the program should continue: if true, it reports end-of-simulation.
		bool scenStep() _wur_;
		/// Tells whether the scenario variation is to continue.
		bool scenContinue;
		/// Tells whether the scenario variation is to continue.
		bool scenarioContinue() const { return scenContinue; }
		/// Starts taking all the measurements.
		void startMeasure();
		/// Measures the size of each age group. @param [in] index Number of age group survey, as defined in the documentation of overallResCls::ageGroup.
		void ageGroupH(const size_t ageIndex);
		/// Measures and stores results for the current day.
		void measureDay();
		/// Prints the data for one day of the timelines of persons
		void printTL();
		/// Returns the non-buffer population size.
		int nonBufferH();
		/// Reports results for all the life cycle.
		void measureOverall();
		/// Prints the overall results heading.
		void printOverallHeading();
		/// Prints overall results.
		void printOverall();
		/// Restarts a simulation without affecting parameters.
		void start();
		/// Performs interactions among individuals and homes.
		void interact();
		/// Advances the simulation on one step depending on the clock settings.
		void simStep();
		/// Draws the status of the town through gnuplot.
		void draw() const;
		/// Prepares the set of pointers to all the persons that can receive the primary infection.
		void prepareIntroducible();
		/// Introduces an infection.
		void introI();
	public:
		/// Typedef for persons iterator
		typedef std::list<pCls>::iterator pItCls;
		/// Allows for area of each house to be varied
		static paramCls <> scalerange;
		/// Deterministic introduction
		static paramCls<bool> deterministic;
		/// Registers simulation trials
		static paramRangeCls<int> trial;
		/// Number of days per vector scenario.
		static paramCls<int> daysPerPupaScenario;
		/// Day in which program starts the infection introduction attempts.
		static paramCls<size_t> introDay;
		/// Patch number.
		static paramArrayCls<int> patch;
		/// Returns a number of pupae from the resampling pool
		int poolPupae() const _wur_;
		/// Returns the current results pointer.
		resultsCls& result();
		/// Runs all simulations.
		void operator()();
		/// Builds the homes that take part in the simulation.
		void buildHomes();
		/// Lower limits of the age groups that have been defined at the input file for the Fachada cluster.
		static std::vector<int> age;
		/// Number of age groups in the simulation.
		static size_t nAgeGroups;
		/// Lower limits in days of the age groups that have been defined at the input file for the Fachada cluster.
		static std::vector<int> ageDays;
		/// Lower limits of the age groups that have been defined at the input file for year 2009.
		static std::vector<int> age09;
		/// Lower limits of the age groups that have been defined at the input file for year 2010.
		static std::vector<int> age10;
		/// Produces the set of homes for visit; @param [in] person Person who is to visit homes from the set.
		void setVisitHomes(pCls& person);
		/// Decide whether a mosquito can travel to a home. @param [in] mosquito Mosquito that wants to travel.
		bool canTravel(vCls& mosquito);
		/// Emerges adult female mosquitoes. @param [in] amount Number of mosquitoes. @param [in] home Home in which the mosquitoes emerge.
		void emerge(int nMosquitoes, homeCls& home);
		/// \return a reference to the overall results structure.
		overallResCls& overallRes() { return *pOverallRes; }
		/// Creates a new person. @param [in] home Calling Home. @param [in] ageDays Age of the new person, in days. \return a pointer to the new person.
		pItCls newPerson(homeCls* const home, const int ageDays) _wur_;
		/// Introduces an infectious visit for one day.
		void introVisit(homeCls* const home);
		/// Removes a person from the simulation. @param [in] it Iterator to the person. We pass by ref hoping C++ will forward it.
		void remove(pItCls& it) { it->removePrepareBase(); mPerson.erase(it); }
		/// \return an age group from the pool.
		const ageGroupCls& ageGroupsPool() const _wur_;
		/// \return a number of pupae from the pool.
		int nPupaePool() const _wur_;
		/// \return a number of containers from the pool.
		int nContainersPool() const _wur_;
		/// \return a number of positive containers from the resampling pool.
		int nPositiveContainersPool() const _wur_;
		/// Prepares the set of mosquitoes that can receive the primary infection at the home pointed to by \c vCls::staticPHome. @param [in] mosquito Container that is to be filled with pointers to the mosquitoes that are susceptible of having the infection introduced.
		void prepareIntroducible(std::list<vCls*>& mosquito);
		/// Returns the total number of persons, including buffer homes.
		size_t hWithBuffer() const _wur_ { return mPerson.size(); }
};

// Simulation parameters
/// Specifies whether case measurements are performed
const bool trackCases(true);
/// Specifies whether overall measurements are performed
const bool enableOverall(true);
/// Specifies whether time series measurements are performed
const bool enableTL(true);
/// Specifies whether the program must do all simulations and save all results to memory before starting output, or results will be written to hard disk as soon as they are available from each simulation.
const bool keepResults(true);
#if (!defined FABIOID) || (FABIOID == 0)
const bool enableHeadings(true);
#else
const bool enableHeadings(false);
#endif
// ptag - tag for fast CTRL+F search
//Static class members
std::vector<int> simCls::age {0, 6, 16, 26, 36, 46};
size_t simCls::nAgeGroups(simCls::age.size());
std::vector<int> simCls::ageDays {0, 2190, 5840, 9490, 13140, 16790};
std::vector<int> simCls::age09 {0, 1, 2, 3, 4, 5, 6, 11, 16, 51, 61, 71};
std::vector<int> simCls::age10 {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 16, 21, 51, 61, 71};
std::vector<double> pCls::deathP {0.0000125140143578, 0.0000078396655926, 0.0000095127053182, 0.000011246228738, 0.0000120335795618, 0.0000731259890631};
paramIntervalCls<> pCls::deathPvariationPerTrial(0.8, 1.2, "deathPvariationPerTrial", "Fractional variation of deathP for each trial");
paramCls<> pCls::deathPfraction(0, "deathPfraction", "Current death probability as fraction of deathP(field); same fraction for all age groups");
std::vector<double> pCls::deathPcurrent(deathP);
paramIntervalCls<> pCls::t_h_perP(4.5, 7, "t_h_perP", "Minimum incubation time of Dengue fever in humans");
paramIntervalCls<> pCls::p_r_perP(0.2, 1.0 / 3.0, "p_r_perP", "Per person rectangular distribution limits to the daily probability of a person recovering from the infectious state");
paramCls<bool> pCls::seis(false, "SEIS", "SEIS model is used instead of the traditional SEIR model");
paramCls<bool> pCls::allInfections(true, "allInfections", "All infections are allowed, not just secondary");
paramIntervalCls<> pCls::p_pcroff_perP(1.0 / 10.0, 1.0 / 6.0, "p_pcroff_perP", "Per person rectangular distribution limits of time of recovering into the PCR- state");
paramCls<> pCls::p_inf_h(0.75, "p_inf_h", "Probability of a person bitten by an infectious mosquito to become infected");
paramIntervalCls<> pCls::pIndepPerP(0.05, 0.25, "pIndepPerP", "Limits to the Bernoulli probability distance-independent component for choosing homes for the permanent set of contacts");
paramCls<> pCls::Cd(0.2, "Cd", "Bernoulli probability constant for choosing homes for the permanent set of contacts");
paramCls<> pCls::visitW(0.2, "visitW", "Bernoulli probability weight of visiting a home");
paramCls<> pCls::nVisitsLambdaYoung(3, "nVisitsLambdaYoung", "Rate parameter of the Poisson distribution of number of visits that a young person does in each day");
paramCls<> pCls::nVisitsLambdaAdult(1.5, "nVisitsLambdaAdult", "Rate parameter of the Poisson distribution of number of visits that an adult person does in each day");
pPersonVCls pCls::introducible;
size_t pCls::maxIgeneration;
int pCls::idCounter(0);
intMinCls pCls::mIntMin;
std::vector<counterCls> pCls::ageGroupHcount;
paramCls<> vCls::t_m_mean(12.5, "t_m_mean", "Mean Incubation time of Dengue fever virus in mosquitoes");
paramCls<> vCls::t_m_width(5, "t_m_width", "Full width for Incubation time of Dengue fever virus in mosquitoes");
paramCls<> vCls::g(0, "g", "Time of the gravid stage of the gonotrophic cycle");
paramIntervalCls<size_t> vCls::biteLimitPerV(9, 11, "biteLimitPerV", "Limits across mosquitoes in the number of bites a mosquito can inflict each day");
paramCls<> vCls::p_full_mean(0.375, "p_full_mean", "Max Bernoulli probability of a mosquito getting full at each bite");
paramCls<> vCls::p_full_width(0.25, "p_full_width", "Min Bernoulli probability of a mosquito getting full at each bite");
paramCls<> vCls::p_inf_m(0.75, "p_inf_m", "Probability that each mosquito that bites a viremic person to become infected");
paramCls<> vCls::p_same(0, "p_same", "Probability of a mosquito to bite the same person again at the same day");
paramCls<> vCls::sigma(3, "sigma", "Standard deviation of the Gaussian flight distribution");
paramCls<> vCls::sigma_mean(2.9, "sigma_mean", "Mean of the sigma distribution for homes with no habitable containers");
paramCls<> vCls::sigma_width(1.2, "sigma_width", "Sigma distribution width for homes with no habitable containers");
paramCls<> vCls::s_v_Alpha(6, "s_v_Alpha", "Alpha par for Bernoulli probability of mosquito survival each day");
paramCls<> vCls::s_v_Beta(6, "s_v_Beta", "Beta par for Bernoulli probability of mosquito survival each day");
paramCls<> vCls::s_v_mean(0.84, "s_v_mean", "Mean value of the distribution of Bernoulli probabilities of mosquito survival each day");
paramCls<> vCls::s_v_width(0.1, "s_v_width", "Full width of the distribution of Bernoulli probabilities of mosquito survival each day");
int vCls::idcounter(0);
homeCls* vCls::staticPHome(nullptr);
indPCls<> homeCls::yearBirthP(0.1);
paramCls<> homeCls::femaleP(0.5, "femaleP", "Probability of a produced pupa to be female");
paramCls<> homeCls::pupalemergence(2.7, "pupalemergence", "# days until pupal emergence");
paramCls<> homeCls::pupalsurvival(0.94, "pupalsurvival", "Fraction of pupae that survive");
paramArrayCls<> homeCls::nicheBreadthTL({0.0344827586206897, 0.0804597701149425, 0.118226600985222, 0.0410509031198686, 0.0827586206896552, 0.137931034482759, 0.174833635813672, 0.0947574992991309, 0.100724519256959, 0.0755443660177733, 0.149925037481259, 0.114330408760117, 0.132413793103448, 0.230523627075351, 0.160919540229885, 0.193789070377092, 0.314625445897741, 0.367816091954023, 0.342938119981105, 0.381158385484481, 0.548239817373543, 0.152773127999107, 0.316892186817983, 0.296607793664144, 0.169278996865204, 0.250319284802043, 0.272187035733834, 0.231738969225065, 0.241639067471914, 0.17035072207486, 0.169938107869142, 0.237297239230852, 0.173850574712644, 0.237110016420361, 0.316751460335406, 0.397877984084881, 0.439114183739426, 0.338303821062442, 0.418538437203417, 0.395183360700602, 0.280759620189905, 0.247025610002016, 0.187739463601533, 0.153605015673981, 0.172413793103448, 0.173297966401415, 0.147126436781609, 0.103448275862069, 0.164203612479475, 0.0846394984326019, 0.0689655172413793, 0.220689655172414, 0.103448275862069, 0.11264367816092, 0.180229885057471, 0.146400483968542, 0.0563023572959283, 0.0620689655172414, 0.0846394984326019, 0.136116152450091, 0.213969938107869, 0.209671026555688, 0.126156433978133, 0.114942528735632, 0.169278996865204, 0.10678531701891, 0.194867682437851, 0.221300742034046, 0.224278104850014, 0.358454793221946, 0.237297239230852, 0.225979243388015, 0.35632183908046, 0.310861951437369, 0.252873563218391, 0.505747126436782, 0.399014778325123, 0.283371647509578, 0.143236074270557, 0.173850574712644, 0.191570881226054, 0.224806201550388, 0.0689655172413793, 0.169278996865204, 0.169278996865204, 0.153682418050234, 0.137931034482759, 0.126436781609195, 0.18544061302682, 0.155909224874742, 0.128912466843501, 0.206896551724138, 0.0919540229885057, 0.103448275862069, 0.194867682437851, 0.202758620689655, 0.261261261261261, 0.18544061302682, 0.184546615581098, 0.164300202839757, 0.172919034988, 0.203532380151388, 0.147126436781609, 0.19000062810125, 0.238541223215553, 0.199848635353105, 0.255427841634738, 0.189819376026273, 0.0306513409961686, 0.0777011494252874, 0.276011344976862, 0.253918495297806, 0.144427786106947, 0.138752052545156, 0.143891017454236, 0.149425287356322, 0.198686371100164, 0.106984969053935, 0.165977011494253, 0.163473818646232, 0.137931034482759, 0.0620689655172414, 0.103448275862069, 0.0517241379310345, 0.0344827586206896, 0.0625798212005109, 0.0344827586206897, 0.0114942528735632, 0.0574712643678161, 0.0804597701149425, 0.0574712643678161, 0.0846394984326019, 0.0517241379310345, 0.0410509031198686, 0.0410509031198686, 0.0574712643678161, 0.103448275862069, 0.154533844189017, 0.114266396213658, 0.123152709359606, 0.0965152344462689, 0.106403940886699, 0.230827209346147, 0.316947909024211, 0.423410743607788, 0.476137335862813, 0.669904214559387, 0.748798241509102, 0.692771685480887, 0.706280844705711, 0.743379548406796, 0.887491910881022, 0.662853407419512, 0.611694152923538, 0.655259989053093, 0.508763348822043, 0.430458023268001, 0.450975579825746, 0.388319353836595, 0.248275862068965, 0.231738969225065, 0.18425635667015, 0.236453201970443, 0.231738969225065, 0.238541223215553, 0.283371647509578, 0.212156633547633, 0.191570881226054, 0.173727422003284, 0.225287356321839, 0.236453201970443, 0.147126436781609, 0.153605015673981, 0.114942528735632}, "nicheBreadthTL", "Niche breadth timeline");
paramArrayCls<> homeCls::p_introTL({0.00143946026067999, 0.000499812590513885, 0.00100762218247599, 0.00167937030412665, 0.00212867551707282, 0.0011537137455179, 0.00122636480739739, 0.000919785495773843, 0.00182540250448549, 0.00139202242664542, 0.00161219549196159, 0.00280672536939686, 0.00195926535481443, 0.00235946617290309, 0.00383070157303373, 0.00447832081100441, 0.00417542068765736, 0.00464076903469835, 0.00667505809906787, 0.00186007924461925, 0.00385830012910111, 0.00361132882472764, 0.00206104537324635, 0.00304774610748911, 0.00331399548110254, 0.00282152268838484, 0.00294206068816162, 0.00207409409355813, 0.00206907033623809, 0.0028891970419126, 0.0021167063208263, 0.0028869175228082, 0.00385658672300124, 0.00484433741574996, 0.00534640607212285, 0.0041189960837701, 0.00509588741366872, 0.00481152920468034, 0.00341837042339983, 0.00300764418502215, 0.00228580958061683, 0.0018702078386865, 0.00209921288015832, 0.00210997807441554, 0.00179132832440176, 0.00125952772809499, 0.00199925036205554, 0.00103052268662317, 0.000839685152063326, 0.00268699248660264, 0.00125952772809499, 0.0013714857483701, 0.00219437719739216, 0.00178248953332741, 0.000685505674989552, 0.000755716636856994, 0.00103052268662317, 0.00165727332644078, 0.00260517701024776, 0.00255283589334195, 0.00153600942450609, 0.00139947525343888, 0.00206104537324635, 0.00130015765480773, 0.0023725987436208, 0.00269443273478548, 0.00273068342134415, 0.00436434292953877, 0.0028891970419126, 0.0027513955225376, 0.00433837328566052, 0.00378487939196915, 0.00307884555756553, 0.00615769111513107, 0.00485817837979496, 0.00345017299147798, 0.00174396146966999, 0.0021167063208263, 0.00233245875573146, 0.00273711322823743, 0.000839685152063326, 0.00206104537324635, 0.00206104537324635, 0.00187115024626457, 0.00167937030412665, 0.00153942277878277, 0.00225782007554806, 0.00189826258735684, 0.00156956532270299, 0.00251905545618998, 0.0011195802027511, 0.00125952772809499, 0.0023725987436208, 0.00246867434706618, 0.00318096942741107, 0.00225782007554806, 0.00224693526802131, 0.00200042639168028, 0.00210536441973387, 0.00247809520486982, 0.00179132832440176, 0.00231334024134022, 0.00290434308769229, 0.00243324401060875, 0.00310994500764195, 0.0023111334185362, 0.000373193400917034, 0.000946045271324681, 0.00336055810858245, 0.00309156805986952, 0.00175847107932103, 0.00168936655593693, 0.0017519356876383, 0.00181931782947054, 0.0024190929380872, 0.00130258850512388, 0.00202084226596574, 0.00199036480489085, 0.00167937030412665, 0.000755716636856994, 0.00125952772809499, 0.000629763864047495, 0.000419842576031663, 0.000761936526872278, 0.000419842576031663, 0.000139947525343888, 0.000699737626719439, 0.000979632677407215, 0.000699737626719439, 0.00103052268662317, 0.000629763864047495, 0.000499812590513885, 0.000499812590513885, 0.000699737626719439, 0.00125952772809499, 0.00188151672962338, 0.00139124304606571, 0.00149943777154165, 0.00117511493503042, 0.00129551423461199, 0.0028104216155125, 0.00385897857118465, 0.0051552098621575, 0.00579717904337141, 0.00815637502291735, 0.00911694409665167, 0.0084347964219887, 0.00859927630804888, 0.00905096915259855, 0.010805599812075, 0.00807052838096883, 0.0074476422183008, 0.00797807521146137, 0.00619441493383832, 0.00524101355550772, 0.00549082372568101, 0.0047279569372935, 0.00302286654742797, 0.00282152268838483, 0.00224340123960353, 0.00287892052135998, 0.00282152268838483, 0.00290434308769229, 0.00345017299147798, 0.00258309923897447, 0.00233245875573146, 0.00211520688305476, 0.0027429714967402, 0.00287892052135998, 0.00179132832440176, 0.0018702078386865, 0.00139947525343888}, "p_introTL", "Timeline of the probability of infection introduction, with time interval 3 weeks");
paramArrayCls<> homeCls::p_introConn({0.001}, "p_introConn", "Connectivity coefficient to the probability of infection introduction");
paramCls<> homeCls::p_intro(0, "p_intro", "Probability of infection introduction per home per day");
paramArrayCls<> homeCls::t_dwell({12}, "t_dwell", "Mean time that a family dwells in a home (years)");
paramArrayCls<> homeCls::scenarioNumber({1}, "scenarioNumber", "Scenario number");
paramCls<> homeCls::p_migration(0.0004335804958904, "p_migration", "Daily probability of a family to move out of the town and be replaced by a new family");
paramCls<> homeCls::refr_non_disp(2, "refr_non_disp", "# days after emergence before mosquito disperses");
paramCls<> homeCls::disp_non_biting(1, "disp_non_biting", "Additional # days mosquito can disperse before becoming host-seeking");
paramIntervalCls<> homeCls::yearBirthPacrossTrials(0.15, 0.15, "yearBirthPacrossTrials", "Per-trial, rectangular distribution of annual birth probability per inhabited home");
paramCls<> homeCls::birthP(0, "dayBirthP", "Daily birth probability per inhabited home");
paramCls<bool> simCls::deterministic(false, "deterministic", "Deterministic introduction");
paramCls <> simCls::homefront(5, "homefront", "Front width of homes");
paramCls <> simCls::homeside(8, "homeside", "Side width of homes");
paramCls <> simCls::scalerange(1, "scale", "Scale factor for individual house edge size (house area increases as scale^2)");
paramCls<size_t> simCls::simTime(20 * 365, "simTime", "Simulation time of the model (in days)");
const std::initializer_list< void* > simCls::TLomitParams{};
#if defined FABIOID && FABIOID == 0
#define TRIAL_MIN 1
#define TRIALS 1
#elif defined FABIOID && FABIOID == 1
#define TRIAL_MIN 144
#define TRIALS 143
#elif defined FABIOID && FABIOID == 2
#define TRIAL_MIN 287
#define TRIALS 143
#elif defined FABIOID && FABIOID == 3
#define TRIAL_MIN 430
#define TRIALS 143
#elif defined FABIOID && FABIOID == 4
#define TRIAL_MIN 573
#define TRIALS 143
#elif defined FABIOID && FABIOID == 5
#define TRIAL_MIN 716
#define TRIALS 143
#elif defined FABIOID && FABIOID == 6
#define TRIAL_MIN 859
#define TRIALS 142
#else
#define TRIAL_MIN 1
#define TRIALS 10
#endif
paramRangeCls<int> simCls::trial(TRIAL_MIN, 1, TRIALS, "Trial", "Number of simulation trials");
paramCls<int> simCls::daysPerPupaScenario(30, "daysPerPupaScenario", "Number of days per pupal production scenario");
paramCls<size_t>simCls::introDay(19, "introDay", "Day in which program starts the infection introduction attempts");
paramArrayCls<int> simCls::patch({27}, "patch", "Number of patches in which the town is divided");

//Static template members
template<> paramArrayCls<int> & results1Cls::range(simCls::patch);
template<> paramArrayCls<> & results2Cls::range(homeCls::scenarioNumber);
template<> paramRangeCls<int> & results3Cls::range(simCls::trial);

//Variables
/// Based on enableOverall, to determine the time in which measurements and results can be calculated by the program.
bool enableOverallToday(false);
/// Based on enableTL, to determine the time in which results can be calculated by the program and stored in the containers.
bool enableTLResults(false);
/// Random number generator.
gslCls gsl;
/// Simulation clock.
dayClockCls simClock;
/// Simulation clock for the measurement time.
dayClockCls measurementClock;
/// Daily measurement containers for all simulation objects. Their contents are cleared every day.
dayMeasureCls dayMeasures;
/// Overall measurement containers for all simulation objects. Their contents are cleared every 5 years.
fiveYearMeasureCls fiveYearMeasures;
/// Overall measurement containers for all simulation objects. Their contents are cleared at the end of each simulation.
overallMeasureCls overallMeasures;
/// Determines whether a dengue infection has been introduced.
bool introduceInfection(false);
/// Simulation.
simCls sim;
/// Timeline output file.
std::ofstream tlfile;
/// Overall output file.
std::ofstream overallfile;
/// Description output file.
std::ofstream descrfile;
/// Gnuplot file.
std::ofstream gnuplot;

double pointCls::distance(const pointCls& p1, const pointCls& p2) {
	const double dx(p1.x - p2.x);
	const double dy(p1.y - p2.y);
	return std::sqrt(dx * dx + dy * dy);
}

std::ostream& operator<< (std::ostream& out, const pointCls& c) {
	out << "(" << c.x << "," << c.y << ")";
	return out;
}

introCls::introCls(introCls::Type type, int ID, int introTime) : mType(type), mIntroducedID(ID), mIntroTime(introTime), mNicheBreadth(homeCls::nicheBreadthTL.value()) {}

bool introCls::operator<(const introCls& a) const {
	if(mIntroTime == a.mIntroTime) {
		if(mType == a.mType)
			return mIntroducedID < a.mIntroducedID;
		return mType < a.mType;
	}
	return mIntroTime < a.mIntroTime;
}

void introCls::printValue(std::ofstream& outfile) const {
	outfile << "\t";
	switch(mType) {
		case Type::noIntroduction:
			cerr << __FILE__ << " " << __LINE__ << " " << __PRETTY_FUNCTION__ << "() invalid infection introduction." << endl;
			std::exit(1);
		case Type::inhabitant:
			outfile << "Inhabitant";
			break;
		case Type::visitor1day:
			outfile << "Visitor";
			break;
		case Type::mosquito:
			outfile << "Mosquito";
	}
	outfile << "\t" << mIntroducedID << "\t" << mIntroTime;
}

bool introCls::isPostEpidemic() const {
	return (overallMeasures.PCRextinction.state == epiExtinctionCls::stateCls::postEpidemic) && (mIntroTime >= overallMeasures.PCRextinction.postEpidemicDay);
}

void pCls::sampleindpars() {
	t_h.set(gsl.flatd(t_h_perP));
	p_r.set(gsl.flatd(p_r_perP));
	p_pcroff.set(linearRelation(p_r_perP, p_pcroff_perP, p_r.value()));
	pIndep.set(gsl.flatd(pIndepPerP));
}

void pCls::nVisitsLambdaPrepare() {
	if(olderThan15())
		nVisitsLambda.set(nVisitsLambdaAdult.value());
	else
		nVisitsLambda.set(nVisitsLambdaYoung.value());
}

void pCls::calculateAgeIndex() {
	// This needs an overflow:
	mAgeIndex = std::numeric_limits<size_t>::max();
	for(int ageDays : simCls::ageDays) {
		if(mAgeDays < ageDays)
			break;
		++mAgeIndex;
	}
	nVisitsLambdaPrepare();
}

void pCls::updateAgeIndex() {
	const size_t nextAgeIndex(mAgeIndex + 1);
	if((nextAgeIndex < simCls::ageDays.size()) && (mAgeDays == simCls::ageDays[nextAgeIndex])) {
		++mAgeIndex;
		nVisitsLambdaPrepare();
	}
}

void pCls::deathPprepare() {
	deathPfraction.set(gsl.flatd(deathPvariationPerTrial));
	// Modify death probabilities according to the birth probability.
	std::vector<double>::iterator curit(deathPcurrent.begin());
	for(double deathp : deathP)
		(*(curit++)) = deathp * deathPfraction.value();
}

const infectionInfoCls* pCls::isInfectious() const {
	if(mState.health != ePStatus::I)
		return nullptr;
	if(allInfections.value() || (mInfectionInfo.infectionGeneration == 1))
		return &mInfectionInfo;
	else
		return nullptr;
}

void pCls::setVisitHomes() {
	sim.setVisitHomes(*this);
}

void pCls::bookVisits() {
	mInhabitantW = 1;
	if(!mHomeSet.empty()) {
		int nvisits(gsl.poisson(nVisitsLambda.value()));
		if(isPartOfResults())
			sim.overallRes().nVisits(nvisits);
		if(nvisits)
			mInhabitantW -= visitw();
		for(; nvisits; --nvisits)
			gsl.flat(mHomeSet)->bookVisit(this);
	} else if(isPartOfResults())
		sim.overallRes().nVisits(0);
}

void pCls::riskBite(const infectionInfoCls& info) {
	if((mState.health == ePStatus::S) && gsl.bern(p_inf_h.value()) && (mNewState.health == ePStatus::S)) {
		mNewState.health = ePStatus::E;
		mInfectionInfo = info;
		++mInfectionInfo.infectionGeneration;
		if(maxIgeneration < mInfectionInfo.infectionGeneration)
			maxIgeneration = mInfectionInfo.infectionGeneration;
		mIDay = simClock.day() + t_h.value();
	}
}

void pCls::interact() {
	bookVisits();
	switch(mState.health) {
		case ePStatus::S:
			// in case of putting instructions here, you need to check the precedence of interact() and riskBite()
			break;
		case ePStatus::E:
			if(simClock.day() == mIDay) {
				mNewState.health = ePStatus::I;
				mNewState.pcr = true;
				mMeasureCases = true;
			}
			break;
		case ePStatus::I:
			if(gsl.bern(p_r.value())) {
				if(seis.value())
					mNewState.health = ePStatus::S;
				else
					mNewState.health = ePStatus::R;
			}
			break;
		case ePStatus::R:
			break;
	}
	if(mState.pcr && gsl.bern(p_pcroff.value()))
		mNewState.pcr = false;
	if(gsl.bern(deathPcurrent[mAgeIndex]))
		mRemove = true;
}

void pCls::simStep() {
	mState = mNewState;
	++mAgeDays;
	if(mAgeDays % 365 == 0)
		updateAgeIndex();
}

pCls::pCls(homeCls* const home, const int ageDays) : mID(++idCounter), mAgeDays(ageDays) {
	calculateAgeIndex();
	sampleindpars();
	if(ageDays >= 0) {
		pHome = home;
		mRemove = false;
		setVisitHomes();
		mNewState.health = ePStatus::S;
		mNewState.pcr = false;
		mMeasureCases = false;
	} else {
		pHome = nullptr;
		mRemove = true;
		mHomeSet.emplace_back(home);
		mInfectionInfo.introID = introCls(introCls::Type::visitor1day, id(), simClock.day());
		mNewState.health = ePStatus::I;
		mMeasureCases = true;
		deterministicIntroIbase();
	}
	mState = mNewState;
}

bool pCls::isPartOfResults() const {
	// A 1-day visitor is part of the results, too.
	return (!pHome) || pHome->isPartOfResults();
}

void pCls::deterministicIntroIbase() {
	mState.health = ePStatus::I;
	mState.pcr = true;
	mNewState.pcr = true;
	mInfectionInfo.infectionGeneration = 1;
	mIDay = simClock.day();
	mMeasureCases = true;
	if(introduceInfection)
		introduceInfection = false;
}

void pCls::deterministicIntroI() {
	mInfectionInfo.introID = introCls(introCls::Type::inhabitant, id(), simClock.day());
	mNewState.health = ePStatus::I;
	deterministicIntroIbase();
}

void pCls::prepareIntroducible() {
	if(isPartOfResults() && (mState.health == ePStatus::S))
		introducible.emplace_back(this);
}

void pCls::measureDay() {
	if(isPartOfResults()) {
		if(mState.pcr) {
			++dayMeasures.pcr;
			if(infectionGeneration() > 1)
				++dayMeasures.pcr2plus;
			dayMeasures.pcrIntroID.insert(mInfectionInfo.introID);
		}
		++dayMeasures.h;
		switch(mState.health) {
			case ePStatus::S:
				if(enableTL)
					++dayMeasures.s;
				break;
			case ePStatus::E:
				break;
			case ePStatus::I:
				if(mMeasureCases) {
					if(enableOverall && (infectionGeneration() > 1))
						++fiveYearMeasures.secondaryAndHigherpcrCases;
					if(enableOverallToday) {
						++dayMeasures.pcrCases;	//I case is the same as PCR case, it's the same day;
						++overallMeasures.pcrCases;
						if(mInfectionInfo.introID.isPostEpidemic() && (infectionGeneration() == 1)) {
							++overallMeasures.postEpiIntroductions;
							if(mInfectionInfo.introID.isInhabitantPerson())
								++overallMeasures.postEpiInhabitantIntroductions;
						}
					}
					mMeasureCases = false;
				}
				break;
			case ePStatus::R:
				break;
		}
	}
}

void pCls::measureOverall() const {
}

void pCls::removePrepareBase() {
	measureOverall();
}

void pCls::removePrepare() {
	removePrepareBase();
	if(pHome) {
		pHome->remove(this);
		if((nVisitsLambdaYoung.value() != nVisitsLambdaAdult.value()) && olderThan15() && (nVisitsLambda.value() == nVisitsLambdaYoung.value()))
			pHome->assignYoungLambda();
	}
}

int vCls::pHomeID() const {
	return mState.pHome->id();
}

void vCls::fly() {
	const bool printFlight(false);
	if(printFlight)
		cerr << "mosquito " << this << " " << mState.position;
	pointCls& newPos(mNewState.position);
	homeCls* pHome(mState.pHome);
	while(true) {
		if(pHome)
			newPos = pHome->position();
		else
			newPos = mState.position;
		newPos.x += gsl.gauss(sigma.value());
		newPos.y += gsl.gauss(sigma.value());
		if(printFlight)
			cerr << "->" << newPos;
		if(pHome && pHome->willBeInside(*this)) {
			setHome(pHome);
			break;
		} else {
			if(sim.canTravel(*this)) {
				if(printFlight) {
					if(pHome)
						pHome->reportPosition();
					else
						cerr << "street";
					cerr << " to ";
					if(mNewState.pHome)
						mNewState.pHome->reportPosition();
					else
						cerr << "street";
				}
				break;
			} else if(printFlight)
				cerr << " cannot travel";
		}
	}
	if(printFlight)
		cerr << "\n";
}

void vCls::sampleindpars() {
	t_m.set(gsl.flatM(t_m_mean.value(), t_m_width.value()));
	biteLimit.set(gsl.flatClosed(biteLimitPerV));
	p_full.set(gsl.flatM(p_full_mean.value(), p_full_width.value()));
	sigma.set(gsl.flatM(sigma_mean.value(), sigma_width.value()));
	s_v.set(s_v_mean.value() + s_v_width.value() * (-0.5 + gsl.beta(s_v_Alpha.value(), s_v_Beta.value())));
}

bool vCls::isPartOfResults() const {
	return mState.pHome && mState.pHome->isPartOfResults();		// Returns whether mosquito is currently in a home with index other than -1
}

void vCls::deterministicIntroI() {
	mState.health = eVStatus::I;
	mNewState.health = eVStatus::I;
	mInfectionInfo.infectionGeneration = 1;
	mInfectionInfo.introID = introCls(introCls::Type::mosquito, id(), simClock.day());
	if(introduceInfection)
		introduceInfection = false;
}

void vCls::interact() {
	homeCls* pHome(mState.pHome);
	// In the following, the program needs to check mNewState.bloodSeeking because it will change.
	if(mNewState.bloodSeeking) {
		if(pHome && pHome->hasPeople()) {
			pCls* bitPerson(pHome->randomPerson());
			for(size_t trial(biteLimit.value()); trial; --trial) {
				if(!gsl.bern(p_same.value())) {
					bitPerson = pHome->randomPerson();
				}
				if(g.value() && gsl.bern(p_full.value())) {
					mNewState.bloodSeeking = false;
					mBloodSeekingDay = simClock.day() + g.value();
				}
				switch(mState.health) {
					case eVStatus::S: {
						const infectionInfoCls* info(bitPerson->isInfectious());
						if(info && gsl.bern(p_inf_m.value())) {
							mNewState.health = eVStatus::E;
							mInfectionInfo = *info;
							// The infecter is registered; when \c this infects a new person, prevGenPersonId becomes the infecter of the previous generation.
							mInfectionInfo.prevGenPersonId = bitPerson->id();
							mIDay = simClock.day() + t_m.value();
						}
						break;
					}
					case eVStatus::E:
						break;
					case eVStatus::I:
						bitPerson->riskBite(mInfectionInfo);
						break;
				}
			}
		}
	} else if(simClock.day() >= mBloodSeekingDay)
		mNewState.bloodSeeking = true;
	if((mState.health == eVStatus::E) && (simClock.day() == mIDay))
		mNewState.health = eVStatus::I;
	if(gsl.bern(s_v.value()))
		fly();
	else
		markForRemoval();
}

void vCls::simStep() {
	mState = mNewState;
}

void vCls::setHome(homeCls* home) {
	mNewState.pHome = home;
}

void vCls::measureOverall() {
	if(enableOverall && (mInfectionInfo.introID.isMosquito()) && (mInfectionInfo.introID.isPostEpidemic()) && (mInfectionInfo.infectionGeneration == 1))
		++overallMeasures.postEpiIntroductions;
}

void vCls::measureDay() {
}

vCls::vCls(homeCls* home) : mBloodSeekingDay(simClock.day() + g.value()), mRemove(false), mID(++idcounter) {
	mState.health = eVStatus::S;
	mState.bloodSeeking = g.value() ? false : true;
	mState.pHome = home;
	mState.position = home->position();
	mNewState = mState;
	sampleindpars();
}

void envelopeCls::expandToIncludeX(const envelopeCls& rhs) {
	const double rhsminx(rhs.minX());
	const double rhsmaxx(rhs.maxX());
	double minx(minX());
	double maxx(maxX());
	if(rhsminx < minx)
		minx = rhsminx;
	if(rhsmaxx > maxx)
		maxx = rhsmaxx;
	mCenter.x = 0.5 * (maxx + minx);
	mHalfWidth = 0.5 * (maxx - minx);
}

void envelopeCls::expandToIncludeY(const envelopeCls& rhs) {
	const double rhsminy(rhs.minY());
	const double rhsmaxy(rhs.maxY());
	double miny(minY());
	double maxy(maxY());
	if(rhsminy < miny)
		miny = rhsminy;
	if(rhsmaxy > maxy)
		maxy = rhsmaxy;
	mCenter.y = 0.5 * (maxy + miny);
	mHalfLength = 0.5 * (maxy - miny);
}

void envelopeCls::expandToInclude(const envelopeCls& rhs) {
	expandToIncludeX(rhs);
	expandToIncludeY(rhs);
}

bool envelopeCls::contains(const pointCls& p) const {
	using std::abs;
	return (abs(p.x - mCenter.x) <= mHalfWidth) && (abs(p.y - mCenter.y) <= mHalfLength);
}

void envelopeCls::draw(const std::string& color) const {
	gnuplot << "set object rectangle center " << mCenter.x << "," << mCenter.y << " size " << 2 * mHalfWidth << "," << 2 * mHalfLength << " fs empty border rgb '" << color << "' lw 1\n";
}

void ageGroupCls::merge(const ageGroupCls& a, size_t skip) {
	mSum += a.mSum;
	auto it(a.mGroup.begin());
	for(; skip; ++it, --skip)
		mSum -= *it;
	mGroup.insert(mGroup.end(), it, a.mGroup.end());
}

void ageGroupCls::print(std::ofstream& outfile) const {
	// Print the pAge pointer for spreadsheet sorting.
	outfile << pAge << "\t" << mSum.value();
	for(int h : mGroup) outfile << "\t" << h;
}

void homeCls::birthPprepare() {
	yearBirthP.set(gsl.flatd(yearBirthPacrossTrials));
	birthP.set(yearBirthP.value() / 365.0);
}

void homeCls::p_introStep() {
	p_introTL.step();
	nicheBreadthTL.step();
	if(!p_introTL.cont()) {
		p_introTL.start();
		nicheBreadthTL.start();
	}
	p_introPrepare();
}

homeCls::homeCls(const int id, const envelopeCls& envelope, const bool isPartOfResults) : mId(id), mIsPartOfResults(isPartOfResults), mEnvelope(envelope), mNPupae(-1), mNContainers(-1), mNPositiveContainers(-1) {}

bool homeCls::willBeInside(vCls& mosquito) const {
	return mEnvelope.contains(mosquito.newPosition());
}

void homeCls::fill() {
	if(mAgeGroups.empty())
		mAgeGroups = sim.ageGroupsPool();
	if(mIsPartOfResults) {
		if(mNContainers == -1)
			mNContainers = sim.nContainersPool();
		if((mNPositiveContainers == -1) && (mNPupae == -1)) {
			mNPositiveContainers = sim.nPositiveContainersPool();
			if(mNPositiveContainers)
				mNPupae = sim.nPupaePool();
			else
				mNPupae = 0;
		} else if(mNPositiveContainers == -1) {
			do
				mNPositiveContainers = sim.nPositiveContainersPool();
			while((mNPupae > 0) && (mNPositiveContainers == 0));
		} else if(mNPupae == -1) {
			if(mNPositiveContainers)
				mNPupae = sim.nPupaePool();
			else
				mNPupae = 0;
		} else if((mNPupae > 0) && (mNPositiveContainers == 0)) {
			cerr << "Home " << id() << " has " << mNPupae << " pupae but no positive containers. Correcting.\n";
			while(mNPositiveContainers == 0)
				mNPositiveContainers = sim.nPositiveContainersPool();
		}
	} else {
		mNContainers = 0;
		mNPupae = 0;
		mNPositiveContainers = 0;
	}
}

void homeCls::setEmerges() {
	if(mNPositiveContainers)
		mNPupae = sim.poolPupae();
	b_v.set((pupalsurvival.value() * femaleP.value() / (pupalemergence.value() + refr_non_disp.value())) * mNPupae);
}

void homeCls::assignYoungLambda() {
	// Assign the lambda of an adult to a young person.
	if(!mInhabitant.empty())
		gsl.insSeqFlat(mInhabitant)->setVisitsLambda(pCls::nVisitsLambdaYoung.value());
}

void homeCls::createPersons() {
	mInhabitant.clear();
	const auto end(mAgeGroups.end());
	auto it(mAgeGroups.begin());
	const auto ageEnd(mAgeGroups.ageEnd());
	auto ageIt(mAgeGroups.ageBegin());
	auto ageItNext(ageIt);
	++ageItNext;
	size_t ageLowLimitDays(*ageIt);
	while(it != end) {
		const size_t ageHighLimitYears((ageItNext == ageEnd) ? 90 : *ageItNext);
		for(size_t s(*it); s; --s) {
			const int ageDays(gsl.flat(ageLowLimitDays, ageHighLimitYears * 365));
			mInhabitant.emplace_back(sim.newPerson(this, ageDays));
		}
		ageLowLimitDays = ageHighLimitYears * 365;
		++it;
		++ageIt;
		++ageItNext;
	}
	assignYoungLambda();
}

void homeCls::start() {
	setEmerges();
	mVisit.clear();
	createPersons();
}

void homeCls::interact() {
	// Distribution of probability of being bitten, including visitors and inhabitants.
	std::vector<double> bitedistro;
	bitedistro.reserve(mInhabitant.size() + mVisit.size());
	// Visitors are included first
	for(pCls* person : mVisit) bitedistro.emplace_back(person->visitw());
	// Inhabitants are included next
	for(auto& person : mInhabitant)
		bitedistro.emplace_back(person->inhabitantw());
	mVisitDistro = {bitedistro.begin(), bitedistro.end()};
}

void homeCls::birth() {
	if(hasInhabitants())
		mInhabitant.emplace_back(sim.newPerson(this, 0));
}

void homeCls::simStep() {
	/// Female mosquitoes emerge.
	sim.emerge(gsl.poisson(b_v.value()), *this);
	/// Person births
	if(gsl.bern(birthP.value()))
		birth();
	/// Migrations
	if(gsl.bern(p_migration.value())) {
		// The current family goes away:
		for(auto& person : mInhabitant) sim.remove(person);
		mInhabitant.clear();
		mAgeGroups = sim.ageGroupsPool();
		createPersons();
	}
}

void homeCls::reportPosition() const {
	cerr << " home " << id() << position();
}

pCls* homeCls::randomPerson() {
	const size_t index(gsl.discrete(mVisitDistro));
	if(index < mVisit.size())
		return mVisit[index];
	else {
		auto it(mInhabitant.begin());
		size_t ind(index - mVisit.size());
		while(ind) {
			--ind;
			++it;
		}
		return &(**it);
	}
}

void homeCls::draw() const {
	if(!isPartOfResults())
		mEnvelope.draw("blue");
	else
		mEnvelope.draw();
}

void homeCls::bookVisit(pCls* const p) {
	mVisit.emplace_back(p);
}

void homeCls::measureDay() {
	if(mIsPartOfResults) {
		if(enableTLResults) {
			if(mNPupae > 15)
				++overallMeasures.freq15pupae;
			if(mNPupae > 50)
				++overallMeasures.freq50pupae;
			if(mNPupae > 100)
				++overallMeasures.freq100pupae;
		}
	}
	mVisit.clear();
}

void homeCls::measureOverall() const {

}

void homeCls::save(ageGroupCls&& ageGroups, const int nPupae, const int nContainers, const int nPositiveContainers) {
	mAgeGroups = std::move(ageGroups);
	mNContainers = nContainers;
	mNPupae = nPupae;
	mNPositiveContainers = nPositiveContainers;
}

void homeCls::remove(pCls* person) {
	const auto end(mInhabitant.end());
	for(auto it(mInhabitant.begin()); it != end; ++it) {
		if(&(**it) == person) {
			mInhabitant.erase(it);
			break;
		}
	}
}

void homeCls::introI() {
	if(isPartOfResults()) {
		if(gsl.bern(p_intro.value())) {
			static std::vector<pCls*> susceptibles;	//Holds pointers to susceptible inhabitants of the home. Static to save mallocs.
			susceptibles.clear();
			susceptibles.reserve(mInhabitant.size());
			for(auto& p : mInhabitant) {
				if(p->health() == ePStatus::S)
					susceptibles.emplace_back(&*p);
			}
			if(!susceptibles.empty())
				gsl.flat(susceptibles)->deterministicIntroI();
		}
		if(gsl.bern(p_intro.value()))
			sim.introVisit(this);
	} else {
		if(gsl.bern(p_intro.value())) {
			vCls::staticPHome = this;	// Inform all mosquitoes that the chosen infection home is \c this;
			std::list<vCls*> mosquito;
			sim.prepareIntroducible(mosquito);
			if(!mosquito.empty())
				gsl.insSeqFlat(mosquito)->deterministicIntroI();
		}
	}
}

void resultsCls::init() {
	if(enableTL) {
		dayRes.clear();
		for(size_t i(measurementClock.dayArraySize()); i; --i)
			dayRes.emplace_back();
	}
	if(enableOverall) {
		overallRes = overallResCls();
		overallRes.ageGroup.clear();
		overallRes.ageGroup.resize(3);
		for(overallResCls::ageGroupResCls& ageGroup : overallRes.ageGroup) {
			ageGroup.clear();
			ageGroup.resize(simCls::nAgeGroups);
		}
	}
}

void simCls::interact() {
	if(simClock.day() % daysPerPupaScenario.value() == 0)
		for(homeCls& home : pPatch->mHome) home.setEmerges();
	for(pCls& person : mPerson) person.interact();
	for(homeCls& home : pPatch->mHome) home.interact();
	for(vCls& mosquito : mMosquito) mosquito.interact();
}

void simCls::simStep() {
	if(simClock.day() % 21 == 0)
		homeCls::p_introStep();
	for(auto i(mMosquito.begin()); i != mMosquito.end(); ++i) {
		if(i->remove()) {
			i->measureOverall();
			mMosquito.erase(i--);
		} else
			i->simStep();
	}
	for(auto i(mPerson.begin()); i != mPerson.end(); ++i) {
		if(i->remove()) {
			i->removePrepare();
			mPerson.erase(i--);
		} else
			i->simStep();
	}
	if((simClock.day() > introDay.value()) || !introduceInfection)
		introI();
	else if((simClock.day() == introDay.value())) {
		// Introduction is guaranteed for the first day.
		while(introduceInfection)
			introI();
	}
	for(homeCls& home : pPatch->mHome) home.simStep();
}

void simCls::emerge(int nMosquitoes, homeCls& home) {
	for(; nMosquitoes; --nMosquitoes)
		mMosquito.emplace_back(&home);
}

simCls::pItCls simCls::newPerson(homeCls* const home, const int ageDays) {
	mPerson.emplace_front(home, ageDays);
	return mPerson.begin();
}

void simCls::introVisit(homeCls* const home) {
	// A homeless person is created, visiting \c home; he will be removed on the next day;
	mPerson.emplace_back(home, -1);
}

const ageGroupCls& simCls::ageGroupsPool() const {
	return gsl.constFlat(pPatch->ageGroups);
}

int simCls::nPupaePool() const {
	return gsl.constFlat(pPatch->nPupaePosConts);
}

int simCls::nContainersPool() const {
	return gsl.constFlat(pPatch->nContainers);
}

int simCls::nPositiveContainersPool() const {
	return gsl.constFlat(pPatch->nPositiveContainers);
}

double simCls::distance(homeCls& h1, homeCls& h2) {
	return pointCls::distance(h1.position(), h2.position());
}

void simCls::buildHomes() {
	/// Read in "houselayout.txt" and define / fill in houselayout matrix. Expects a tab-delimitted text file
	std::ifstream hlayout("../input/houselayoutFachadaClusterFabio28March2012.csv");
	std::string buf;
	std::vector<std::vector<int> > patchmatrix;
	int inrows;
	int incols;
	hlayout >> inrows >> incols;

	// Define patch matrix
	patchmatrix.resize(inrows);
	for(std::vector<int>& row : patchmatrix) {
		row.resize(incols);
		for(int& cell : row)
			hlayout >> cell;
	}
	hlayout.close();
	mPerPatch.clear();
	mEnvelope = envelopeCls();	// It doesn't matter mEnvelope is reused without reinitialization, because the simulation is the same for all patches.
	const double scale(scalerange.value());    //Factor by which houses are scaled
	const double frontwidth(homefront.value() * scale);	//Length of the front of the theoretical home (y direction).
	const double sidewidth(homeside.value() * scale);   //Length of the side of the house (x direction).
	// Build one neighborhood for each patch; the neighborhoods are the same for all patches; they will be different for each patch when they are filled in from the pools
	for(patch.start(); patch.cont(); patch.step()) {
		std::list<homeCls>& mHome(mPerPatch[patch.value()].mHome);
		envelopeCls homeEnvelope({0, 0}, sidewidth, frontwidth);
		pointCls& homeCenter(homeEnvelope.center());
		mVisitZone.clear();
		/// Reserve memory for visit zones; this is not strict, incols is more than enough.
		mVisitZone.reserve(incols + 2);
		mVisitZone.emplace_back();
		auto visitZone(mVisitZone.rbegin());
		for(int i(-1); i <= incols ; ++i, homeCenter.x += sidewidth) {
			homeCenter.y = 0;
			if(i % 2 == 0) {
				mVisitZone.emplace_back();
				visitZone = mVisitZone.rbegin();
				visitZone->reserve(inrows + 2);
			}
			mEnvelope.expandToIncludeX(homeEnvelope);
			for(int k(-1); k <= inrows; ++k, homeCenter.y += frontwidth) {
				int IDtoadd = ((i == -1 || i == incols || k == -1 || k == inrows) ? -1 : patchmatrix[k][i]);
				const bool isPartOfResults(IDtoadd != -1);
				mHome.emplace_back(IDtoadd, homeEnvelope, isPartOfResults);
				mEnvelope.expandToIncludeY(homeEnvelope);
				visitZone->emplace_back(&mHome.back());
			}
		}
		// After the mVisitZone vector is created (maybe relocated),
		// report the zones to each of the homes
		for(auto& vZone : mVisitZone) {
			for(homeCls* homePtr : vZone)
				homePtr->setVisitZone(&vZone);
		}
	}
}

void simCls::prepareIntroducible() {
	pCls::introducible.clear();
	pCls::introducible.reserve(mPerson.size());  //Approximate
	for(pCls& person : mPerson) person.prepareIntroducible();
	if(pCls::introducible.empty()) {
		cerr << __LINE__ << " " << __PRETTY_FUNCTION__ << " No persons found in non-buffer homes; please debug." << endl;
		std::exit(1);
	}
}

void simCls::introI() {
	if(deterministic.value()) {
		prepareIntroducible();
		gsl.insecureFlat(pCls::introducible)->deterministicIntroI();
	} else
		for(homeCls& home : pPatch->mHome) home.introI();
}

bool simCls::canTravel(vCls& mosquito) {
	/// Determine what home will be the destination of the flight of the mosquito.
	for(homeCls& home : pPatch->mHome) {
		if(home.willBeInside(mosquito)) {
			mosquito.setHome(&home);
			return true;
		}
	}
	/// At this point the mosquito is not flying to any home, so it is flying to the street.
	/// Determine if the mosquito stays within neighborhood limits.
	if(mEnvelope.contains(mosquito.newPosition())) {
		mosquito.setHome(NULL);
		return true;
	} else
		return false;
}

void simCls::setVisitHomes(pCls& person) {
	auto& homeSet(person.homeSet());
	homeCls* ownHome(person.ownHome());
	homeSet.clear();
	homeSet.reserve(pPatch->mHome.size() * (person.pindep() + person.cd()));
	for(homeCls& home : pPatch->mHome) {
		if(&home == ownHome)
			continue;
		double p(person.pindep());
		if(home.visitZone() == ownHome->visitZone())
			p += person.cd() / sqrt(distance(*ownHome, home));
		if(gsl.bern(p))
			homeSet.emplace_back(&home);
	}
}

resultsCls& simCls::result() {
	return mRes.result();
}

void simCls::printDescription() const {
	descrfile << "\tIndividual interaction parameter\tMin\tstepSize\tnSteps\tMax\tValues\n";
	pCls::t_h_perP.printDescription();
	pCls::p_r_perP.printDescription();
	pCls::seis.printDescription();
	pCls::allInfections.printDescription();
	pCls::p_pcroff_perP.printDescription();
	pCls::p_inf_h.printDescription();
	pCls::pIndepPerP.printDescription();
	pCls::Cd.printDescription();
	pCls::visitW.printDescription();
	pCls::nVisitsLambdaAdult.printDescription();
	pCls::nVisitsLambdaYoung.printDescription();
	vCls::t_m_mean.printDescription();
	vCls::t_m_width.printDescription();
	vCls::g.printDescription();
	vCls::biteLimitPerV.printDescription();
	vCls::p_full_mean.printDescription();
	vCls::p_full_width.printDescription();
	vCls::p_inf_m.printDescription();
	vCls::p_same.printDescription();
	vCls::sigma_mean.printDescription();
	vCls::sigma_width.printDescription();
	vCls::s_v_Alpha.printDescription();
	vCls::s_v_Beta.printDescription();
	vCls::s_v_mean.printDescription();
	vCls::s_v_width.printDescription();
	homeCls::femaleP.printDescription();
	homeCls::pupalemergence.printDescription();
	homeCls::pupalsurvival.printDescription();
	if(!deterministic.value()) homeCls::p_introConn.printDescription();
	homeCls::t_dwell.printDescription();
	// homeCls::p_migration.printDescription();	// This is a function of t_dwell, no description is necessary.
	homeCls::refr_non_disp.printDescription();
	homeCls::disp_non_biting.printDescription();
	homeCls::yearBirthPacrossTrials.printDescription();
	deterministic.printDescription();
	homeside.printDescription();
	homefront.printDescription();
	scalerange.printDescription();
	simTime.printDescription();
	trial.printNDescription();
	daysPerPupaScenario.printDescription();
	simCls::patch.printDescription();
	// Description of outputs
	descrfile << "\nSummary output\tDescription" << endl;
	descrfile << "Days with 0 PCR people\tNumber of days (of PostEpidemic time) that 0 PCR+ people was reported\n";	// pcr0days
	descrfile << "Days with 0 PCR cases\tNumber of days (of PostEpidemic time) that 0 PCR+ cases were reported\n";	// pcr0casesDays
	descrfile << "Total PCR cases\tNumber of PCR+ cases in postEpidemic time, including introductions\n";	// pcrCases
	descrfile << "2+ generation PCR cases, years m-n\tNumber of secondary and higher PCR+ cases in the years m-n, excluding introductions.\n";	// secondaryAndHigherpcrCases
	descrfile << "Days PCR>=N\tNumber of days (of PostEpidemic time) in which N or more PCR+ were reported\n";	// pcr16days pcr22days
	descrfile << "Niche breadth of PCR>=N\tNeighborhood niche breadth (in PostEpidemic time) at days of introductions that caused days of reports of N or more PCR+, averaged over introductions\n";	// pcr16nicheBreadth pcr22nicheBreadth
	descrfile << "Number of introductions that caused >=N PCR+\tAverage number of introductions responsible for the PCR+ on days that N or more PCR+ were reported.\n";	// pcr16introductions pcr22introductions
	descrfile << "HomeDays with more than N pupae\tReports of homes with more than N pupae, on all simulation days\n";	// freq100pupae freq50pupae freq15pupae
	descrfile << "nVisits/person-day\tNumber of visits per person-day, indisctinct\n";	// nVisits
	descrfile << "deathPfraction\t" << pCls::deathPfraction.description() << "\n";	// deathPfraction
	descrfile << "preEpidemicTime\tTime duration of the pre-epidemic time. Day that the post-epidemic was confirmed.";	//  preEpidemicTime
	descrfile << endl;
}

void simCls::setResultsPointers() {
	pRes = & (mRes.result());
	// The following is harmless even if !enableOverall.
	pOverallRes = & (pRes->overallRes);
}

void simCls::scenPrepare() {
	// This is common code for scenStart() and scenStep().
	if(keepResults)
		setResultsPointers();
	else
		pRes->init();
	if(patch.hasChanged())
		pPatch = &mPerPatch.at(patch.value());
	homeCls::t_dwell.setIndex(homeCls::scenarioNumber.index());
	homeCls::p_introConn.setIndex(homeCls::scenarioNumber.index());
	homeCls::p_migrationPrepare();
	homeCls::birthPprepare();
	pCls::deathPprepare();
}

void simCls::scenStart() {
	mRes.start();
	if(!keepResults)
		setResultsPointers();	//First time.
	homeCls::t_dwell.setIndex(homeCls::scenarioNumber.index());
	homeCls::p_introConn.setIndex(homeCls::scenarioNumber.index());
	homeCls::p_migrationPrepare();	// This is in case t_dwell never changes;
	scenPrepare();
}

bool simCls::scenStep() {
	return mRes.step();
}

void simCls::startMeasure() {
	// Overall measures and results are enabled from now on, if that was specified a compile time.
	enableOverallToday = enableOverall;
	enableTLResults = true;
}

void simCls::ageGroupH(const size_t ageIndex) {
	if(!enableOverall)
		return;
	pCls::ageGroupHcount.clear();
	pCls::ageGroupHcount.resize(nAgeGroups);
	for(pCls& person : mPerson) person.ageGroupH();
	std::vector<int>& ageGroupSize(mAgeGroupSizeV.at(ageIndex));
	for(size_t i(0); i < nAgeGroups; ++i)
		ageGroupSize.at(i) = pCls::ageGroupHcount.at(i).value();
}

void simCls::measureDay() {
	dayMeasures = {};
	for(pCls& person : mPerson) person.measureDay();
	for(vCls& mosquito : mMosquito) mosquito.measureDay();
	for(homeCls& home : pPatch->mHome) home.measureDay();
	const double pcr2plusFrac(dayMeasures.pcr2plus.value() / static_cast<double>(dayMeasures.pcr2plus.value() + dayMeasures.s.value()));
	if(enableTLResults) {
		pDayRes->pcr(dayMeasures.pcr.value());
		const double sFrac(dayMeasures.s.value() / static_cast<double>(dayMeasures.h.value()));
		pDayRes->sFrac(sFrac);
	}
	switch(overallMeasures.PCRextinction.state) {
		case epiExtinctionCls::stateCls::nonEpidemic:
			if(pcr2plusFrac > 0.05)
				overallMeasures.PCRextinction.state = epiExtinctionCls::stateCls::inProgress;
			break;
		case epiExtinctionCls::stateCls::inProgress:
			if(dayMeasures.pcr2plus.value() == 0) {
				overallMeasures.PCRextinction.postEpidemicDay = simClock.day() + 10;	// The program waits and sees whether this day is reached;
				overallMeasures.PCRextinction.state = epiExtinctionCls::stateCls::extinguished;
			}
			break;
		case epiExtinctionCls::stateCls::extinguished:
			if(dayMeasures.pcr2plus.value()) {
				overallMeasures.PCRextinction.state = epiExtinctionCls::stateCls::inProgress;
				break;
			}
			if(simClock.day() != overallMeasures.PCRextinction.postEpidemicDay)
				break;
			overallMeasures.PCRextinction.state = epiExtinctionCls::stateCls::postEpidemic;
			mPreEpidemicTime = simClock.day();
			// No break, we continue below.
		case epiExtinctionCls::stateCls::postEpidemic:
			break;
	}
	switch(simClock.day()) {
		case 0:
			ageGroupH(0);
			break;
		case 10 * 365:
			ageGroupH(1);
			startMeasure();
			break;
		case 20 * 365:
			ageGroupH(2);
			break;
		default:
			break;
	}
	if(enableOverall) {
		const double pcrIntroIDs(((dayMeasures.pcr.value() >= 22) || (enableOverallToday && (dayMeasures.pcr.value() >= 16))) ? dayMeasures.pcrIntroID.size() : std::numeric_limits<double>::quiet_NaN());
		if(dayMeasures.pcr.value() >= 22)	// There are no PCR+ at day 0;
			mFiveYearRes.back().pcr22introductions(pcrIntroIDs);
		if(simClock.day() && (simClock.day() % (5 * 365) == 0)) {
			overallResCls::fiveYearResCls& fiveYrRes(mFiveYearRes.back());
			fiveYrRes.secondaryAndHigherpcrCases(fiveYearMeasures.secondaryAndHigherpcrCases.value());
			// This line is left for last;
			fiveYearMeasures = {};
			if(simClock.day() != simTime.value())
				mFiveYearRes.emplace_back();
		}
		if(enableOverallToday) {
			if(dayMeasures.pcr.value() == 0)
				++overallMeasures.pcr0days;
			if(dayMeasures.pcrCases.value() == 0)
				++overallMeasures.pcr0casesDays;
			if(dayMeasures.pcr.value() >= 16) {
				++overallMeasures.pcr16days;
				for(const introCls& intro : dayMeasures.pcrIntroID)
					overallMeasures.pcr16intro.insert(intro);
				pOverallRes->pcr16introductions(pcrIntroIDs);
			}
			if(dayMeasures.pcr.value() >= 22) {
				++overallMeasures.pcr22days;
				for(const introCls& intro : dayMeasures.pcrIntroID)
					overallMeasures.pcr22intro.insert(intro);
				pOverallRes->pcr22introductions(pcrIntroIDs);
			}
		}
	}
}

void simCls::printTLheading() {
	mRes.printHorizontalHeading(tlfile, 2, TLomitParams);
	homeCls::t_dwell.printHeading(tlfile, false);
	mRes.start(TLomitParams);
	while (true) {
		tlfile << "\t" << homeCls::t_dwell.value() << "\t" << homeCls::t_dwell.value();
		if (mRes.step(TLomitParams))
			break;
	}
	tlfile << endl;
	homeCls::p_introConn.printHeading(tlfile, false);
	mRes.start(TLomitParams);
	while (true) {
		tlfile << "\t" << homeCls::p_introConn.value() << "\t" << homeCls::p_introConn.value();
		if (mRes.step(TLomitParams))
			break;
	}
	tlfile << endl;
	tlfile << "yearBirthP";
	mRes.start(TLomitParams);
	while (true) {
		tlfile << "\t" << homeCls::yearbirthP() << "\t" << homeCls::yearbirthP();
		if (mRes.step(TLomitParams))
			break;
	}
	tlfile << endl;
	tlfile << "Day";
	mRes.start(TLomitParams);
	while (true) {
		tlfile << "\tHsFrac\tPCR+";
		if (mRes.step(TLomitParams))
			break;
	}
	tlfile << endl;
}

void simCls::printTL() {
	mRes.start();
	while(true) {
		std::stringstream fileNameStream;
		fileNameStream << "tl";
		mRes.produceFileName(fileNameStream, TLomitParams);
		fileNameStream << ".tsv";
		std::string tlfilename;
		fileNameStream >> tlfilename;
		cerr << "Writing timeline to file " << tlfilename << endl;
		tlfile.open(tlfilename);
		printTLheading();
		measurementClock.start();
		while(true) {
			mRes.start(TLomitParams);
			tlfile << measurementClock.day();
			while (true) {
				setResultsPointers();
				const dayResCls* pDayRes(&(pRes->dayRes[measurementClock.dayIndex()]));
				pDayRes->sFrac.printMean(tlfile);
				pDayRes->pcr.printMean(tlfile);
				if (mRes.step(TLomitParams))
					break;
			}
			tlfile << endl;
			if(measurementClock.simStep())
				break;
		}
		tlfile.close();
		if(mRes.stepOmitted(TLomitParams))
			break;
	}
}

int simCls::nonBufferH() {
	int h(0);
	for(pCls& person : mPerson) {
		if(person.isPartOfResults())
			++h;
	}
	return h;
}

void simCls::measureOverall() {
	for(pCls& person : mPerson) person.measureOverall();
	for(vCls& mosquito : mMosquito) mosquito.measureOverall();
	for(homeCls& home : pPatch->mHome) home.measureOverall();
	overallResCls& r(*pOverallRes);
	if(overallMeasures.PCRextinction.state == epiExtinctionCls::stateCls::postEpidemic) {
		r.postEpidemic(1);
		if(mPreEpidemicTime < 10 * 365)
			r.postEpidemic10yr(1);
		else
			r.postEpidemic10yr(0);
		r.preEpidemicTime(mPreEpidemicTime);
	} else {
		r.postEpidemic(0);
		r.postEpidemic10yr(0);
	}
	r.pcr0days(overallMeasures.pcr0days.value());
	r.pcr0casesDays(overallMeasures.pcr0casesDays.value());
	r.pcrCases(overallMeasures.pcrCases.value());
	r.pcr16days(overallMeasures.pcr16days.value());
	for(const introCls& intro : overallMeasures.pcr16intro)
		pOverallRes->pcr16nicheBreadth(intro.nicheBreadth());
	if(pOverallRes->pcr16introductions.n() == 0)
		pOverallRes->pcr16introductions(0);
	r.pcr22days(overallMeasures.pcr22days.value());
	for(const introCls& intro : overallMeasures.pcr22intro)
		pOverallRes->pcr22nicheBreadth(intro.nicheBreadth());
	if(pOverallRes->pcr22introductions.n() == 0)
		pOverallRes->pcr22introductions(0);
	r.freq15pupae(overallMeasures.freq15pupae.value());
	r.freq50pupae(overallMeasures.freq50pupae.value());
	r.freq100pupae(overallMeasures.freq100pupae.value());
	r.postEpiIntroductions(overallMeasures.postEpiIntroductions.value());
	r.postEpiInhabitantIntroductions(overallMeasures.postEpiInhabitantIntroductions.value());
	r.yearBirthRate(homeCls::yearbirthP());
	r.deathPfraction(pCls::deathPfraction.value());
	r.h0(mH0);
	const double hFinal(nonBufferH());
	r.hFinal(hFinal);
	r.hVariationFrac((hFinal - mH0) / mH0);
	for(size_t ageIndex(0); ageIndex < 3; ++ageIndex) {
		const ageGroupSizeCls& ageGroupMeasure(mAgeGroupSizeV.at(ageIndex));
		overallResCls::ageGroupResCls& ageGroupRes(pOverallRes->ageGroup.at(ageIndex));
		for(size_t groupIndex(0); groupIndex < nAgeGroups; ++groupIndex)
			ageGroupRes.at(groupIndex)(ageGroupMeasure.at(groupIndex));
	}
	for(overallResCls::fiveYearResCls& fiveYrTemp : mFiveYearRes) {
		r.fiveYearMeasurementVector.emplace_back();
		overallResCls::fiveYearResCls& fiveYrRes(r.fiveYearMeasurementVector.back());
		fiveYrRes.pcr22introductions.combine(fiveYrTemp.pcr22introductions);
		fiveYrRes.secondaryAndHigherpcrCases.combine(fiveYrTemp.secondaryAndHigherpcrCases);
	}
}

void simCls::printOverallHeading() {
	mRes.printHeading(overallfile);
	overallfile << "\tt_dwell\tp_introConn";
	overallfile << "\tpostEpidemic";	// postEpidemic
	overallfile << "\tpostEpidemic before 10yr";	// postEpidemic10yr
	overallfile << "\tpostEpidemicTime";	// postEpidemicTime
	overallfile << "\tDays with 0 PCR people";	// pcr0days
	overallfile << "\tDays with 0 PCR cases";	// pcr0casesDays
	overallfile << "\tTotal PCR cases";	// pcrCases
	const int nYears(simTime.value() / 365);
	for(int year(5); year <= nYears; year += 5)
		overallfile << "\t2+ generation PCR cases, years " << year - 5 << "-" << year;	// secondaryAndHigherpcrCases
	overallfile << "\tDays PCR>=16";	// pcr16days
	overallfile << "\tNiche breadth of PCR>=16";	// pcr16nicheBreadth
	overallfile << "\tNumber of introductions that caused >=16 PCR+";	// pcr16introductions
	overallfile << "\tDays PCR>=22";	// pcr22days
	overallfile << "\tNiche breadth of PCR>=22";	// pcr22nicheBreadth
	overallfile << "\tNumber of introductions that caused >=22 PCR+";	// pcr22introductions
	for(int year(5); year <= nYears; year += 5)
		overallfile << "\tNumber of introductions that caused >=22 PCR+, years " << year - 5 << "-" << year;	// secondaryAndHigherpcrCases
	overallfile << "\tNumber of post-epidemic introductions";	// postEpiIntroductions
	overallfile << "\tPost-epidemic inhabitant introductions";	// postEpiInhabitantIntroductions
	overallfile << "\tHomeDays with more than 100 pupae";	// freq100pupae
	overallfile << "\tHomeDays with more than 50 pupae";	// freq50pupae
	overallfile << "\tHomeDays with more than 15 pupae";	// freq15pupae
	overallfile << "\tnVisits/person-day";	// nVisits
	overallfile << "\tyearBirthRate";	// yearBirthRate
	overallfile << "\tdeathPfraction";	// deathPfraction
	overallfile << "\tStarting nonBuffer human population";	// h0
	overallfile << "\tFinal nonBuffer human population";	// hFinal
	overallfile << "\tHuman nonBuffer population variation (20 years)";	// hVariationFrac
	for(int ageIndex(0); ageIndex < 3; ++ageIndex) {
		typedef std::vector<int>::const_iterator ageIterator;
		ageIterator ageIt(age.begin());
		ageIterator ageNextIt(ageIt);
		++ageNextIt;
		for(; ageIt != age.end(); ++ageIt) {
			overallfile << "\tH(age" << *ageIt;
			if(ageNextIt == age.end())
				overallfile << "+";
			else
				overallfile << "-" << *(ageNextIt++) - 1;
			overallfile << ",t=" << ageIndex * 10 << "yr)";
		}
	}
	overallfile << endl;
}

void simCls::printOverall() {
	const overallResCls& r(*pOverallRes);
	mRes.printValue(overallfile);
	overallfile << "\t" << homeCls::t_dwell.value() << "\t" << homeCls::p_introConn.value();
	r.postEpidemic.printMean(overallfile);
	r.postEpidemic10yr.printMean(overallfile);
	r.preEpidemicTime.printMean(overallfile);
	r.pcr0days.printMean(overallfile);
	r.pcr0casesDays.printMean(overallfile);
	r.pcrCases.printMean(overallfile);
	for(const overallResCls::fiveYearResCls& fiveYrMeasure : r.fiveYearMeasurementVector)
		fiveYrMeasure.secondaryAndHigherpcrCases.printMean(overallfile);
	r.pcr16days.printMean(overallfile);
	r.pcr16nicheBreadth.printMean(overallfile);
	r.pcr16introductions.printMean(overallfile);
	r.pcr22days.printMean(overallfile);
	r.pcr22nicheBreadth.printMean(overallfile);
	r.pcr22introductions.printMean(overallfile);
	for(const overallResCls::fiveYearResCls& fiveYrMeasure : r.fiveYearMeasurementVector)
		fiveYrMeasure.pcr22introductions.printMean(overallfile);
	r.postEpiIntroductions.printMean(overallfile);
	r.postEpiInhabitantIntroductions.printMean(overallfile);
	r.freq100pupae.printMean(overallfile);
	r.freq50pupae.printMean(overallfile);
	r.freq15pupae.printMean(overallfile);
	r.nVisits.printMean(overallfile);
	r.yearBirthRate.printMean(overallfile);
	r.deathPfraction.printMean(overallfile);
	r.h0.printMean(overallfile);
	r.hFinal.printMean(overallfile);
	r.hVariationFrac.printMean(overallfile);
	for(int ageIndex(0); ageIndex < 3; ++ageIndex) {
		for(size_t i(0); i < nAgeGroups; ++i)
			pOverallRes->ageGroup.at(ageIndex).at(i).printMean(overallfile);
	}
	overallfile << endl;
}

void simCls::start() {
	enableOverallToday = false;
	enableTLResults = false;
	fiveYearMeasures = {};
	overallMeasures = {};
	simClock.start();
	measurementClock.start();
	pDayRes = pRes->dayRes.begin();
	mMosquito.clear();
	mPerson.clear();
	pCls::idCounter = 0;
	vCls::idcounter = 0;
	homeCls::p_introStart();
	for(homeCls& home : pPatch->mHome) home.start();
	introduceInfection = true;
	mH0 = nonBufferH();
	mFiveYearRes.clear();
	mFiveYearRes.emplace_back();
}

void simCls::draw() const {
	gnuplot << "set terminal png large size 1000,600\nset output 'town" << simClock.day() << ".png'\n";
	gnuplot << "reset\n";
	gnuplot << "unset key\n";
	gnuplot << "unset xtics\n";
	gnuplot << "unset ytics\n";
	gnuplot << "set xrange [" << mEnvelope.minX() - 1 << ":" << mEnvelope.maxX() + 1 << "]\n";
	gnuplot << "set yrange [" << mEnvelope.minY() - 1 << ":" << mEnvelope.maxY() + 1 << "]\n";
	gnuplot << "set cbrange [21:36]\n";	// Patch numbers for Armenia
	gnuplot << "set cblabel 'Patch'\n";
	gnuplot << "set palette defined ( 21 '#00dcff', 22 '#e6e6ff', 23 '#9966cc', 24 '#0084d1', 25 '#eb613d', 26 '#99ccff', 27 '#e6ff00', 28 '#ffff99', 29 '#000080', 30 '#314004', 31 '#99284c', 32 '#4c1900', 33 '#800080', 34 '#ffcc99', 35 '#0000ff', 36 '#ffff00' )\n";
	gnuplot << "set pm3d\n";
	for(const homeCls& home : pPatch->mHome) home.draw();
	gnuplot << "plot -10 with points linecolor rgb 'white' pointsize 0\nset terminal dumb nofeed\n";
}

void simCls::operator()() {
	// Checks
	gsl.seedSalt(trial.min());
	// Initializations
	ageDays.clear();
	for(int ageYears : age)
		ageDays.emplace_back(ageYears * 365);
	mAgeGroupSizeV.resize(3);
	for(ageGroupSizeCls& ageGroup : mAgeGroupSizeV)
		ageGroup.resize(nAgeGroups);
	mFiveYearRes.reserve(simTime.value() / (5 * 365));
	buildHomes();
	readSurveyFiles();
	simClock.init(simTime.value());
	measurementClock.init(3650);
	mRes.init();
	printDescription();
	if(enableHeadings && enableOverall)
		printOverallHeading();
	const int nScenarios(mRes.nScenarios());
	cerr << " = " << nScenarios << " scenarios.\n";
	mETA.start(nScenarios);
	pCls::maxIgeneration = 0;
	scenStart();
	while(true) {
		mRes.info();
		mETA.info();
		// The simulation is repeated nPostEpiTrial times or until a post-epidemic condition is achieved.
		// Next line is needed to ensure a proper enableTLResults value at the start of each scenario.
		overallMeasures.PCRextinction.state = epiExtinctionCls::stateCls::nonEpidemic;
		bool keepTrying(true);
		for(int nPostEpiTrial((simTime.value() > 20) ? 30 : 1); nPostEpiTrial && keepTrying; --nPostEpiTrial) {
			start();
			while(true) {
				// Measure for the current day; enableTL and enableOverall are evaluated at object scope;
				measureDay();
				if(enableTLResults) {
					simClock.simStep();
					if(measurementClock.simStep())
						break;
				} else if(simClock.simStep())
					break;
				if(simClock.day() == 3650) {
					if(overallMeasures.PCRextinction.state == epiExtinctionCls::stateCls::postEpidemic)
						keepTrying = false;
					else if(nPostEpiTrial > 1)
						break;
				}
				// Prepare pointer for results for the new interval
				if(enableTLResults)
					++pDayRes;
				// Interact;
				interact();
				simStep();
			}
		}
		if(enableOverall) {
			measureOverall();
			printOverall();
		}
		if(scenStep())
			break;
		mETA.step();
		scenPrepare();
	}
	if(keepResults && enableTL)
		printTL();
	descrfile << "Analysis\t" << mETA.nScenarios() << " scenarios.\n";
	boost::posix_time::ptime endTime(boost::posix_time::second_clock::local_time());
	descrfile << "Finished\t" << endTime << "\n";
	boost::posix_time::time_duration duration(endTime - mETA.startingTime());
	descrfile << "Execution time\t" << duration << endl;
}

int simCls::poolPupae() const {
	return gsl.constFlat(pPatch->nPupaePosConts);
}

void simCls::ignore(std::ifstream& input, int nFields) {
	while(nFields && input.good()) {
		switch(input.peek()) {
			case '\n':
			case '\r':
				return;	// No further parsing or seeking in the survey file.
			case ',':
				--nFields;
				// Do not break, continue below;
			default:
				input.ignore();
		}
	}
}

template<class dataT> void simCls::readField(std::ifstream& input, dataT& result) {
	switch(input.peek()) {
		case '\n':
		case '\r':
			break;	// No parsing or seeking in the survey file.
		case ',':
			input.ignore();	// Now the stream is ready to read the next field.
			break;
		default:
			input >> result;
			ignore(input, 1);	// This is to put the stream at the beginning of the next field.
	}
}

void simCls::readField(std::ifstream& input, std::string& result) {
	while(input.good()) {
		std::ifstream::int_type c(input.peek());
		switch(c) {
			case '\n':
			case '\r':
				return;	// No parsing or seeking in the survey file.
			case ',':
				input.ignore();	// Now the stream is ready to read the next field.
				return;
			default:
				result += static_cast<char>(c);
				input.ignore();
		}
	}
}

void simCls::readField(std::ifstream& input, bool& result) {
	int iResult;
	switch(input.peek()) {
		case '\n':
		case '\r':
			break;	// No parsing or seeking in the survey file.
		case ',':
			input.ignore();	// Now the stream is ready to read the next field.
			break;
		default:
			input >> iResult;
			result = iResult;
			ignore(input, 1);	// This is to put the stream at the beginning of the next field.
	}
}

void simCls::nextLine(std::ifstream& input) {
	bool newLine(false);
	while(input.good()) {
		switch(input.peek()) {
			case '\n':
			case '\r':
				newLine = true;
				input.ignore();
				continue;
			default:
				if(newLine)
					return;
				input.ignore();
		}
	}
}

void simCls::readSurveyFiles() {
	// Typedef for the container of temporary versions of the resampling pools. These use std::list for fast construction; member mHome is unused.
	typedef std::map<int, perPatchCls<std::list> > tmpPatchPoolCls;
	// Temporary versions of the resampling pools
	tmpPatchPoolCls tmpPool;
	// Pointer to the temporary version of the resampling pools for the current patch
	tmpPatchPoolCls::mapped_type* tmpPatchPool;
	for(patch.start(); patch.cont(); patch.step()) {
		std::ifstream census("../input/fachadaCluster5Jun2012.csv");	// Manzana 72 belongs to patch 28	while Manzana 68 is absent from the Armenia data.
		// Full:    Barrio,  ID   , mz  ,Casa ,Tipo ,Inspect,Muest,PCR- ,PCR+ ,  0-5yr, 6-15y ,16-25yr,26-35yr,36-45yr,>45 yr , Human ,Ocupa,Ocupa,Ocupa,TimeH,TimeR,Visit,Visit,Visit,Visit,Contnrs,+Contnr,+Pupa,Pupae  ,Febrs,PCR+K,PCR-K
		// Parsed:        ,  ID   ,     ,     ,     ,Inspect,     ,     ,     ,  0-5yr, 6-15y ,16-25yr,26-35yr,36-45yr,>45 yr , Human ,     ,     ,     ,     ,     ,     ,     ,     ,     ,Contnrs,+Contnr,     ,Pupae  ,     ,     ,
		tmpPatchPool = &tmpPool[patch.value()];
		pPatch = &mPerPatch.at(patch.value());
		// Line number of the census file.
		size_t line(1);
		for(nextLine(census); census.good(); nextLine(census)) {
			//First line is the header, which is ignored.
			++line;
			ignore(census, 1);	// Barrio;
			int id(-1); readField(census, id); if(id < 0) { cerr << "Ignoring invalid cluster line " << line << " with homeID " << id << "." << endl; continue; }
			ignore(census, 3);	//  mz  ,Casa ,Tipo;
			bool inspected(false); readField(census, inspected);
			ignore(census, 3);	// Muest,PCR- ,PCR+ ;
			ageGroupCls ageGroup(age);	// Contains age group data. Index: Age group index, value: Number of persons in the age group;
			bool ageDataFound(false);	// Determines whether any age data has been read from the input file;
			const size_t nAgeGroups(age.size());
			ageGroup.reserve(nAgeGroups);
			for(size_t j(0); j < nAgeGroups; ++j) {
				int persons(-1); readField(census, persons);
				if(persons >= 0) {
					ageGroup(persons);
					ageDataFound = true;
				} else
					ageGroup.zero();
			}
			int nPersonsCheck(-1); readField(census, nPersonsCheck);
			if(nPersonsCheck >= 0) {
				ageDataFound = true;
				if(ageGroup != nPersonsCheck)
					cerr << "Using age data of inconsistent home " << id << " at line " << line << ": " << ageGroup.H() << " persons counted, " << nPersonsCheck << " reported." << endl;
			}
			if(ageDataFound)
				tmpPatchPool->ageGroups.emplace_back(ageGroup);	//Do not move;
			ignore(census, 9);	// Ocupa,Ocupa,Ocupa,TimeH,TimeR,Visit,Visit,Visit,Visit;
			int nCont(-1); readField(census, nCont);
			if(nCont >= 0)
				tmpPatchPool->nContainers.emplace_back(nCont);
			int nPositiveContainers(-1); readField(census, nPositiveContainers);
			if((nPositiveContainers > 0) && !inspected) {
				cerr << nPositiveContainers << " positive containers at uninspected home " << id << "; home is marked as inspected.\n";
				inspected = true;
			}
			ignore(census, 1);	// +Pupa;
			int nPup(-1); readField(census, nPup);	// Last parsed field;
			if(nPositiveContainers > 0) {
				if (nPup >= 0)
						tmpPatchPool->nPupaePosConts.emplace_back(nPup);
			} else if(nPositiveContainers == 0) {
				if (nPup > 0) {
					cerr << "Reported " << nPup << " pupae with " << nPositiveContainers << " positive containers at home " << id << " at line " << line << "; Will create positive containers." << endl;
					nPositiveContainers = -1;
				}
			}
			if (nPositiveContainers > -1)
				tmpPatchPool->nPositiveContainers.emplace_back(nPositiveContainers);
			// Reminder: The homes are already built by simCls::buildHomes();
			bool notBuilt(true);  // Checker variable;
			for(homeCls& home : pPatch->mHome) {
				if(home.id() == id) {
					home.save(std::move(ageGroup), nPup, nCont, nPositiveContainers);
					notBuilt = false;
					break;
				}
			}
			if(notBuilt)
				cerr << "Discarding surveyed home " << id << " from the simulation because it was not built from the geography files.\n";
		}
		if(tmpPatchPool->ageGroups.empty()) {
			cerr << "No data for persons. Please review the input files.\n";
			std::exit(1);
		}
		if(tmpPatchPool->nPupaePosConts.empty()) {
			cerr << "No data for pupae. Please review the input files.\n";
			std::exit(1);
		}
		if(tmpPatchPool->nContainers.empty()) {
			cerr << "No data for containers. Please review the input files.\n";
			std::exit(1);
		}
		if(tmpPatchPool->nPositiveContainers.empty()) {
			cerr << "No data for positive containers was read. Please review the input files.\n";
			std::exit(1);
		}
		if(pPatch->mHome.empty()) {
			cerr << "No field data. Please review the input file.\n";
			std::exit(1);
		}
	}
	{
		std::ifstream surveyFile("../input/armenia_data_AR_ID_26jul2011.csv");
		// Full header:           CITY ,IDuniq ,barrio ,manza,ptcht, patch ,direc,tipo lc,Exist,adlt7,chdn7,H2007,H 2009 ,2009h0 ,2009h1 ,2009h2 ,2009h3 ,2009h4 ,2009h5 ,09h6-10,9h11-15,9h16-50,9h51-60,9h61-70,9h71+  , <15 , >15 ,9hbad, 10h0  , 10h1  , 10h2  , 10h3  , 10h4  , 10h5  , 10h6  , 10h7  , 10h8  , 10h9  , 10h10 , 10h11 , 10h16 , 10h21 , 10h51 , 10h61 ,10h71+ , <15 , >15 ,  10h  ,Inspect,Posi1,Posi2,Posi3,Posi4,Posi5,Posi6,Posi7,Positiv,pupa1,pupa2,pupa3,pupa4,pupa5,pupa6,pupa7,Total pupae
		// Parenthesis:                ,IDuniq ,barrio ,     ,     , patch ,     ,tipo lc,     ,     ,     ,     ,H 2009 ,2009h0 ,2009h1 ,2009h2 ,2009h3 ,2009h4 ,2009h5 ,09h6-10,9h11-15,9h16-50,9h51-60,9h61-70,9h71+  ,     ,     ,     , 10h0  , 10h1  , 10h2  , 10h3  , 10h4  , 10h5  , 10h6  , 10h7  , 10h8  , 10h9  , 10h10 , 10h11 , 10h16 , 10h21 , 10h51 , 10h61 ,10h71+ ,     ,     ,  10h  ,Inspect,     ,     ,     ,     ,     ,     ,     ,Positiv,     ,     ,     ,     ,     ,     ,     ,Total pupae
		// st::regex surveyRegex("[^,]*,([^,]*),([^,]*),[^,]*,[^,]*,[^,]*,[^,]*,([^,]*),[^,]*,[^,]*,[^,]*,[^,]*,([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),[^,]*,[^,]*,[^,]*,([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),[^,]*,[^,]*,([^,]*),([^,]*),[^,]*,[^,]*,[^,]*,[^,]*,[^,]*,[^,]*,[^,]*,([^,]*),[^,]*,[^,]*,[^,]*,[^,]*,[^,]*,[^,]*,[^,]*,([^,]*)", boost::regex_constants::collate);
		size_t line(1);
		for(nextLine(surveyFile); surveyFile.good(); nextLine(surveyFile)) {
			//First line is the header, which is ignored.
			++line;
			ignore(surveyFile, 1);	// CITY;
			std::string id; readField(surveyFile, id); if(id.empty()) { cerr << "Ignoring invalid Armenia line " << line << " with homeID " << id << "." << endl; continue; }
			std::string neighborhood; readField(surveyFile, neighborhood); if(neighborhood != "La Fachada") continue;
			ignore(surveyFile, 2);	// manza,ptcht
			int patchNumber(-1); readField(surveyFile, patchNumber);
			if((patchNumber < 21) || (patchNumber > 36))
				continue;
			tmpPatchPool = &tmpPool[patchNumber];
			ignore(surveyFile, 1);	// direc,
			std::string premiseType; readField(surveyFile, premiseType);
			boost::to_lower(premiseType);
			boost::trim(premiseType);
			const bool isPublic(
					(premiseType.find("comercial") != std::string::npos) ||
					(premiseType.find("local") != std::string::npos) ||
					(premiseType.find("empresa") != std::string::npos) ||
					(premiseType.find("industria") != std::string::npos) ||
					(premiseType.find("consultorio") != std::string::npos) ||
					(premiseType.find("peluquer") != std::string::npos) ||
					(premiseType.find("compra") != std::string::npos) ||
					(premiseType.find("venta") != std::string::npos) ||
					(premiseType.find("academia") != std::string::npos) ||
					(premiseType.find("gasolina") != std::string::npos) ||
					(premiseType.find("servicio") != std::string::npos) ||
					(premiseType.find("carn") != std::string::npos) ||
					(premiseType.find("coope") != std::string::npos) ||
					(premiseType.find("rest") != std::string::npos) ||
					(premiseType.find("taberna") != std::string::npos) ||
					(premiseType.find("tienda") != std::string::npos) ||
					(premiseType.find("acerradero") != std::string::npos) ||
					(premiseType.find("almac") != std::string::npos) ||
					(premiseType.find("silla") != std::string::npos) ||
					(premiseType.find("miscel") != std::string::npos) ||
					(premiseType.find("asadero") != std::string::npos) ||
					(premiseType.find("asadero") != std::string::npos) ||
					(premiseType == "bar") ||
					(premiseType.find("billar") != std::string::npos) ||
					(premiseType.find("internet") != std::string::npos) ||
					(premiseType.find("bolera") != std::string::npos) ||
					(premiseType.find("carpinter") != std::string::npos) ||
					(premiseType.find("centro") != std::string::npos) ||
					(premiseType.find("club") != std::string::npos) ||
					(premiseType.find("colch") != std::string::npos) ||
					(premiseType.find("colegio") != std::string::npos) ||
					(premiseType.find("diesel") != std::string::npos) ||
					(premiseType.find("disco") != std::string::npos) ||
					(premiseType.find("drog") != std::string::npos) ||
					(premiseType.find("educa") != std::string::npos) ||
					(premiseType.find("ejes") != std::string::npos) ||
					(premiseType.find("evento") != std::string::npos) ||
					(premiseType.find("estadero") != std::string::npos) ||
					(premiseType.find("brica") != std::string::npos) ||
					(premiseType.find("farmacia") != std::string::npos) ||
					(premiseType.find("ferreter") != std::string::npos) ||
					(premiseType.find("florister") != std::string::npos) ||
					(premiseType.find("fotocopia") != std::string::npos) ||
					(premiseType.find("frut") != std::string::npos) ||
					(premiseType.find("gimnasio") != std::string::npos) ||
					(premiseType.find("granero") != std::string::npos) ||
					(premiseType.find("helad") != std::string::npos) ||
					(premiseType.find("hotel") != std::string::npos) ||
					(premiseType.find("iglesia") != std::string::npos) ||
					(premiseType.find("inmobiliaria") != std::string::npos) ||
					(premiseType.find("infantil") != std::string::npos) ||
					(premiseType == "l") ||
					(premiseType == "lc") ||
					(premiseType.find("llant") != std::string::npos) ||
					(premiseType.find("lubrica") != std::string::npos) ||
					(premiseType.find("mercad") != std::string::npos) ||
					(premiseType.find("moneda") != std::string::npos) ||
					(premiseType.find("monti") != std::string::npos) ||
					(premiseType.find("motel") != std::string::npos) ||
					(premiseType.find("muebl") != std::string::npos) ||
					(premiseType.find("negocio") != std::string::npos) ||
					(premiseType.find("plaza") != std::string::npos) ||
					(premiseType.find("radiad") != std::string::npos) ||
					(premiseType.find("salud") != std::string::npos) ||
					(premiseType.find("pizz") != std::string::npos) ||
					(premiseType.find("rectific") != std::string::npos) ||
					(premiseType.find("reposter") != std::string::npos) ||
					(premiseType.find("salsamentaria") != std::string::npos) ||
					(premiseType.find("junim") != std::string::npos) ||
					(premiseType.find("servientrega") != std::string::npos) ||
					(premiseType.find("sintra") != std::string::npos) ||
					(premiseType.find("sociedad") != std::string::npos) ||
					(premiseType.find("taller") != std::string::npos) ||
					boost::starts_with(premiseType, "bco") ||
					boost::starts_with(premiseType, "am") ||
					boost::starts_with(premiseType, "of") ||
					boost::starts_with(premiseType, "pan") ||
					boost::starts_with(premiseType, "sai") ||
					boost::starts_with(premiseType, "l ")
			);
			if(isPublic)
				continue;
			ignore(surveyFile, 4);	// Exist,adlt7,chdn7,H2007,
			{
				// Year 2009
				// Determines whether any age data has been read from the input file for year 2009.
				bool ageDataFound(false);
				int nPersonsCheck(-1); readField(surveyFile, nPersonsCheck);
				// Contains age group data. Index: Age group index, value: Number of persons in the age group.
				ageGroupCls ageGroup(age09);
				const size_t nAgeGroups(age09.size());
				ageGroup.reserve(nAgeGroups);
				for(size_t j(0); j < nAgeGroups; ++j) {
					int persons(-1); readField(surveyFile, persons);
					if(persons >= 0) {
						ageGroup(persons);
						ageDataFound = true;
					} else
						ageGroup.zero();
				}
				if(nPersonsCheck >= 0) {
					ageDataFound = true;
					if(ageGroup != nPersonsCheck)
						cerr << "Using age data of inconsistent home " << id << " at line " << line << ": " << ageGroup.H() << " persons counted, " << nPersonsCheck << " reported." << endl;
				}
				if(ageDataFound)
					tmpPatchPool->ageGroups.emplace_back(std::move(ageGroup));
			}
			ignore(surveyFile, 3); //  <15 , >15 ,9hbad;
			{
				// Year 2010
				// Determines whether any age data has been read from the input file for year 2009.
				bool ageDataFound(false);
				// Contains age group data. Index: Age group index, value: Number of persons in the age group.
				ageGroupCls ageGroup(age10);
				const size_t nAgeGroups(age10.size());
				ageGroup.reserve(nAgeGroups);
				for(size_t j(0); j < nAgeGroups; ++j) {
					int persons(-1); readField(surveyFile, persons);
					if(persons >= 0) {
						ageGroup(persons);
						ageDataFound = true;
					} else
						ageGroup.zero();
				}
				ignore(surveyFile, 2); //  <15 , >15 ,
				int nPersonsCheck(-1); readField(surveyFile, nPersonsCheck);
				if(nPersonsCheck >= 0) {
					ageDataFound = true;
					if(ageGroup != nPersonsCheck)
						cerr << "Using age data of inconsistent home " << id << " at line " << line << ": " << ageGroup.H() << " persons counted, " << nPersonsCheck << " reported." << endl;
				}
				if(ageDataFound)
					tmpPatchPool->ageGroups.emplace_back(std::move(ageGroup));
			}
			bool inspected(false); readField(surveyFile, inspected);
			ignore(surveyFile, 7); // Posi1,Posi2,Posi3,Posi4,Posi5,Posi6,Posi7,
			int nPositiveContainers(-1); readField(surveyFile, nPositiveContainers);
			if((!inspected) && (nPositiveContainers > 0)) {
				cerr << "Using " << nPositiveContainers << " positive containers of uninspected home " << id << ".\n";
				inspected = true;
			}
			bool pupaeFound(false);
			ageGroupCls nPupae;	//Reuse for 7 pupae surveys
			nPupae.reserve(7);
			for(int survey(0); survey < 7; ++survey) {
				int nPup(-1); readField(surveyFile, nPup);
				if(nPup > 0)
					pupaeFound = true;
				if(nPup < 0) {
					cerr << "Home " << id << " has one survey with " << nPup << " pupae, correcting to 0.\n";
					nPup = 0;
				}
				nPupae(nPup);
			}
			int nPupCheck(-1); readField(surveyFile, nPupCheck);
			// The check on nPup should never fails because this column always has data. Because of this, the check nPup < 0 is correct.
			if(nPupCheck < 0) {
				cerr << "Home " << id << " has " << nPupCheck << " pupae, correcting.\n";
				nPupCheck = 0;
			}
			if(nPupae != nPupCheck)
				cerr << "Using pupae data of inconsistent home " << id << " at line " << line << ": " << nPupae.H() << " pupae counted, " << nPupCheck << " reported." << endl;
			if(inspected) {
				if(pupaeFound && (nPositiveContainers == 0)) {
					cerr << "Reported " << nPupCheck << " pupae with " << nPositiveContainers << " positive containers at home " << id << " at line " << line << "; Discarding positive containers to avoid a bias in the distribution." << endl;
					nPositiveContainers = -1;
				}
			}
			if((pupaeFound) || (nPositiveContainers == 0))
				tmpPatchPool->nPupaePosConts.insert(tmpPatchPool->nPupaePosConts.end(), nPupae.begin(), nPupae.end());
			if(nPositiveContainers > -1)
				tmpPatchPool->nPositiveContainers.emplace_back(nPositiveContainers);
		}
	}
	for(patch.start(); patch.cont(); patch.step()) {
		pPatch = &mPerPatch.at(patch.value());
		tmpPatchPoolCls::mapped_type& tmpPatchPool(tmpPool[patch.value()]);
		if(tmpPatchPool.ageGroups.empty()) {
			cerr << __FILE__ << " " << __LINE__ << " " << __PRETTY_FUNCTION__ << "() No age groups in patch " << patch.value() << ", using a zero." << endl;
			ageGroupCls dummyAge;
			for(int i(age10.size()); i; --i)
				dummyAge(0);
			tmpPatchPool.ageGroups.emplace_back(std::move(dummyAge));
		}
		if(tmpPatchPool.nPupaePosConts.empty()) {
			cerr << __FILE__ << " " << __LINE__ << " " << __PRETTY_FUNCTION__ << "() No pupae at positive containers in patch " << patch.value() << ", using a zero." << endl;
			tmpPatchPool.nPupaePosConts.emplace_back(0);
		}
		if(tmpPatchPool.nContainers.empty()) {
			cerr << __FILE__ << " " << __LINE__ << " " << __PRETTY_FUNCTION__ << "() No containers in patch " << patch.value() << ", using a zero." << endl;
			tmpPatchPool.nContainers.emplace_back(0);
		}
		if(tmpPatchPool.nPositiveContainers.empty()) {
			cerr << __FILE__ << " " << __LINE__ << " " << __PRETTY_FUNCTION__ << "() No pupae containers in patch " << patch.value() << ", using a zero." << endl;
			tmpPatchPool.nPositiveContainers.emplace_back(0);
		}
		pPatch->movePools(std::move(tmpPatchPool));
		for(homeCls& home : pPatch->mHome) home.fill();
	}
}

void simCls::prepareIntroducible(std::list< vCls* >& mosquito) {
	for(vCls& v : mMosquito) {
		if(v.isIntroducible())
			mosquito.emplace_back(&v);
	}
}

int main(int argc, char** argv) {
	std::string progName;
	if(argc) {
		progName = {argv[0]};
		const size_t exePosition(progName.find(".exe"));
		if(exePosition != std::string::npos)
			progName.erase(exePosition);
	} else
		progName = "dengue";
	if(enableOverall)
		overallfile.open(progName + "overall.tsv");
	descrfile.open(progName + "descr.tsv");
	gnuplot.open(progName + ".gplot");
	sim();
	descrfile.close();
	overallfile.close();
	cerr << "Simulation has ended.\n";
	return 0;
}

/** \mainpage
\author Harish Padmanabha
\author Fabio Correa
\date 2010 - 2012
\copyright This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
\warning This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

\section Description
This program simulates the interactions of Aedes aegypti with humans in a town.
*/


