/* This file is part of dengue: Dengue fever individual propagation model
Copyright (C) 2012 Harish Padmanabha, Fabio Correa

This project has been developed by Harish Padmanabha<harishpadmanabha@gmail.com> and Fabio Correa<facorread@gmail.com>
This code has been delivered to the Public Library of Science, on February 16, 2015.
We, the developers, grant you a license to review and modify this source code for your own use.
We do not grant you license to distribute or publish this program or any derivative work, any of its output, the ideas or algorithms contained within, in any form.
Anybody who wishes to get a copy of this source code or to get this project under a different license should email the developers directly.

*/

/// \file param.h Classes that represent simulation parameters and simulation time.

/// \addtogroup parameters Parameters
/// \addtogroup simTime Simulation time

//Check for C++0x
#if (__cplusplus <= 199711L) && !defined(__GXX_EXPERIMENTAL_CXX0X__)
#error Please use a C++11 compiler, such as GCC-4.6 or later. On Linux, use a recent distribution. On Windows, use MinGW or Cygwin. On Mac OS, use macports.
#endif

/// \def _wur_
/// \brief Warns about unused results of functions
/// \details _wur_ can be used in most function declarations so that the compiler warns about unused results of these functions. _wur_ is automatically enabled on GCC only; in other compilers it should do nothing.
#if defined(__GXX_EXPERIMENTAL_CXX0X__)
#define _wur_ __attribute__ (( warn_unused_result ))
#else
#define _wur_
#endif

#include <iostream>
#include <fstream>
#include <vector>
#include <cstdlib>
#include <algorithm>

using std::cerr;
using std::cout;
using std::endl;

class dayClockCls;
class intervalClockCls;
class paramIdCls;
template<class paramT> class indPCls;
template<class paramT> class paramCls;
template<class paramT> class paramIntervalCls;
template<class paramT> class paramRangeCls;
template<class paramT> class paramArrayCls;
class boolRangeCls;

/// Represents a simulation clock with days only. \ingroup simTime
class dayClockCls {
	protected:
		/// Simulation time in days.
		size_t mLastDay;
		/// Current day; the same as the time index for now.
		size_t mDay;
	public:
		/// Proper initialization.
		dayClockCls() : mLastDay(0), mDay(0) {}
		/// Initializes the clock.
		void init(size_t lastDay) { mLastDay = lastDay; }
		/// Starts the clock.
		void start() { mDay = 0; }
		/// Steps the clock one day. Returns whether this is the last evolution step.
		bool simStep() { return mDay++ == mLastDay; }
		/// Returns the current day.
		size_t day() const _wur_ { return mDay; }
		/// Returns the current clock index according to the current time.
		size_t dayIndex() const _wur_ { return mDay; }
		/// Returns the size of arrays which contain time series.
		size_t dayArraySize() _wur_ { return mLastDay + 1; }
};

/// Represents a simulation clock with days and intervals. \ingroup simTime
class intervalClockCls : public dayClockCls {
	protected:
		/// Number of intervals per day.
		size_t mIntervalsPerDay;
		/// Inverse of the time interval, for miscellanea.
		double mInvInterval;
		/// Current time interval; the same as the stepwise time index for now.
		size_t mInterval;
		/// Stores whether the clock is at the start of a new day.
		bool mIsANewDay;
	public:
		/// Proper initalization.
		intervalClockCls() : dayClockCls(), mIntervalsPerDay(1), mInvInterval(1), mInterval(0), mIsANewDay(true) {}
		/// Initializes the clock.
		void init(size_t lastDay, size_t intervalsPerDay);
		/// Starts the clock.
		void start() { mDay = 0; mInterval = 0; mIsANewDay = true; }
		/// Steps the clock one day. Returns whether this is the last evolution step.
		bool simStep() _wur_;
		/// Returns whether this is a new day, 12:00AM
		bool isANewDay() const _wur_ { return mIsANewDay; }
		/// Returns the current time step.
		size_t interval() const _wur_ { return mInterval; }
		/// Returns the current day with the current fraction.
		double dayFull() const _wur_ { return mInterval * mInvInterval; }
		/// Returns the current clock index for intervalic arrays, according to the current time.
		size_t intervalIndex() const _wur_ { return mInterval; }
		/// Returns the size of arrays which contain intervalic time series.
		size_t intervalArraySize() _wur_ { return mLastDay * mIntervalsPerDay + 1; }
		/// Returns the length of a time interval.
		double invInterval() const _wur_ { return mInvInterval; }
};

/// Contains the name and description of a simulation parameter. \ingroup parameters
class paramIdCls {
	protected:
		/// Holds the column heading corresponding to the parameter.
		std::string heading;
		/// Holds the parameter description string.
		std::string descr;
	public:
		/// Initializes the parameter strings. @param [in] Heading Short string representing the column heading for data output. @param [in] Descr Parameter description.
		paramIdCls(const char* Heading, const char* Descr) : heading(Heading), descr(Descr) {}
		/// Returns the heading of the parameter.
		const std::string& name() const _wur_ { return heading; };
		/// Returns the description of the parameter.
		const std::string& description() const _wur_ { return descr; };
		/// Prints the column heading corresponding to the parameter. @param [in] outfile Output file.
		void printHeading(std::ofstream& outfile) const;
};

/// Represents an a simulation parameter for an individual; it has neither name or description to save memory. \ingroup parameters
template<class paramT = double> class indPCls {
	protected:
		/// Holds the parameter value.
		paramT mValue;
	public:
		/// Initializes the parameter to a number.
		indPCls(const paramT val) : mValue(val) {}
		/// Initializes the parameter to 0.
		indPCls() : mValue(0) {}
		/// Sets the parameter value.
		void set(const paramT value) { mValue = value; }
		/// Sets the parameter value from a paramRangeCls object.
		void set(const paramRangeCls<paramT>& p);
		/// Adds to a parameter value.
		void add(const paramT value) { mValue += value; }
		/// Returns the parameter value.
		paramT value() const _wur_ { return mValue; }
		/// Returns that the parameter does not vary.
		bool varies() const _wur_ { return false; }
		/// Returns that the parameter never changes.
		bool hasChanged() const _wur_ { return false; }
		/// Reports a design error because this function is for varying parameters.
		void noChange() {
			cerr << __LINE__ << " " << __PRETTY_FUNCTION__ << " is for use in varying parameters only.\n";
			std::exit(1);
		}
		/// Returns whether the parameter is finite
		bool isFinite() const { return isfinite(mValue); }
};

/// Represents a simulation parameter. @tparam paramT underlying parameter type. \ingroup parameters
template<class paramT = double> class paramCls : public paramIdCls, public indPCls<paramT> {
	protected:
		/// Temporarily holds the parameter value.
		paramT mTemp;
	public:
		/// Initializes the parameter. @param [in] initValue Initial value. @param [in] Heading Short string representing the column heading for data output. @param [in] Descr Parameter description.
		paramCls(const paramT initValue, const char* Heading, const char* Descr) : paramIdCls(Heading, Descr), indPCls<paramT>(initValue) {}
		/// Saves temporarily the value.
		void push(const paramT temp) { mTemp = this->value(); this->set(temp); }
		/// Pops the saved value.
		void pop() { this->set(mTemp); }
		/// Prints the parameter scenario info to cerr.
		void info() const { cerr << " " << name() << " " << this->value(); }
		/// Prints the parameter description to the outfile.
		void printDescription() const;
		/// Checks that the parameter has a positive value. If not, prints an error message and ends the program.
		void checkPositive() const;
};

/// Represents a simulation parameter interval. @tparam paramT underlying parameter type. \ingroup parameters
template<class paramT = double> class paramIntervalCls : public paramIdCls {
	protected:
		/// Holds the lower bound of the interval.
		paramT mMin;
		/// Holds the upper bound of the interval.
		paramT mMax;
		/// Tracks whether parameter-related initializations are executed; they should be run only once.
		bool mChanges;
	public:
		/// Interval constructor. @param [in] min Minimum value. @param [in] max Maximum value. @param [in] Heading Short string representing the column heading for data output. @param [in] Descr Parameter description.
		paramIntervalCls(const paramT min, const paramT max, const char* Heading, const char* Descr) : paramIdCls(Heading, Descr), mMin(min), mMax(max), mChanges(true) {}
		/// Returns whether the interval of the range contains the value \c a @param [in] a Value to test
		bool intervalContains(const paramT a) const _wur_ { return (a >= mMin) && (a <= mMax); }
		/// Returns that the interval does not vary.
		bool varies() const _wur_ { return false; }
		/// Returns that the interval never changes.
		bool hasChanged() _wur_;
		/// Returns the minimum of the parameter.
		paramT min() const _wur_ { return mMin; }
		/// Returns the maximum value the parameter can take.
		paramT max() const _wur_ { return mMax; }
		/// Returns the width of the interval.
		paramT width() const _wur_ { return mMax - mMin; }
		/// Prints the parameter description to the description file.
		void printDescription() const;
};

/// Represents a simulation parameter range for sensibility analyzes. @tparam paramT underlying parameter type. \ingroup parameters
template<class paramT = double> class paramRangeCls : public paramCls<paramT> {
	protected:
		/// Minimum parameter value.
		paramT mMin;
		/// Parameter step
		paramT mStep;
		/// Number of steps.
		size_t mNSteps;
		/// Current parameter index.
		size_t mIndex;
		/// Returns the index of the lower bound of the interval that contains a value. \param [in] val Value to bracket.
		long int bracket(paramT val) { return lroundf((val - mMin) / mStep); }
		/// Tracks whether the parameter changes from the previous scenario to the current scenario.
		bool mChanges;
	public:
		/// Starts the simulation.
		void start() { paramCls<paramT>::set(mMin); mIndex = 0; mChanges = true; }
		/// Constructor.
		paramRangeCls(const paramT min, const paramT step, const size_t nsteps, const char* Heading, const char* Descr) : paramCls<paramT>(min, Heading, Descr), mMin(min), mStep(step), mNSteps(nsteps), mChanges(true) {
			if(nsteps == 0) {
				cerr << __LINE__ << " " << __PRETTY_FUNCTION__ << " Parameter " << this->heading << " initialized with zero steps." << endl;
				std::exit(1);
			}
			start();
		}
		/// Returns the current index.
		size_t index() const _wur_ { return mIndex; }
		/// Returns the very index; this is implemented for recursion.
		size_t meanIndex() const _wur_ { return index(); }
		/// Initializes the parameter again.
		void init(const paramT min, const paramT step, const size_t nsteps) { mMin = min; mStep = step; mNSteps = nsteps; start(); }
		/// Returns whether the simulation should continue.
		bool cont() const _wur_ { return mIndex < mNSteps; }
		/// Steps the simulation.
		void step() {
			++mIndex;
			this->mValue += mStep;
			mChanges = true;
		}
		/// Sets the simulation to a specific step.
		void setIndex(const size_t index) {
			if(index >= mNSteps) {
				cerr << __LINE__ << " " << __PRETTY_FUNCTION__ << " Parameter " << this->heading << " is beyond limits." << endl;
				std::exit(1);
			}
			mIndex = index;
			this->mValue = mMin + mStep * mIndex;
			mChanges = true;
		}
		/// Sets the simulation to the last step.
		void setIndexLast() {
			mIndex = mNSteps - 1;
			this->mValue = mMin + mStep * mIndex;
			mChanges = true;
		}
		/// Inits the range to a simple step, because the paramRangeCls is being used as a paramCls
		void set(const paramT value) {
			cerr << __LINE__ << " " << __PRETTY_FUNCTION__ << " Tried to set parameter " << this->heading << "." << endl;
			std::exit(1);
			init(value, 0, 1);
		}
		/// Produces an error.
		void add(const paramT) {
			cerr << __LINE__ << " " << __PRETTY_FUNCTION__ << " Tried to add to parameter " << this->heading << "." << endl;
			std::exit(1);
		}
		/// Returns the number of steps of the parameter.
		size_t nsteps() const _wur_ { return mNSteps; }
		/// Sets the number of steps of the container.
		void nsteps(size_t n) { mNSteps = n; }
		/// Returns the minimum of the parameter.
		paramT min() const _wur_ { return mMin; }
		/// Returns the maximum value the parameter can take.
		paramT max() const _wur_ { return mMin + (mNSteps - 1) * mStep; }
		/// Returns the minimum of the parameter, with a padding for graphs.
		paramT padMin() const _wur_ { return mMin - 0.5 * mStep; }
		/// Returns the maximum of the parameter, with a padding for graphs.
		paramT padMax() const _wur_ { return mMin + (mNSteps - 0.5) * mStep; }
		/// Returns the size of arrays related to the parameter
		size_t arraySize() const _wur_ { return mNSteps; }
		/// Returns the total number of parameter values that are evaluated
		size_t nValues() const _wur_ { return mNSteps; }
		/// Prints the column heading corresponding to the parameter. @param [in] outfile Output file. @param [in] prependTab Prepend a tabulator to the output.
		void printHeading(std::ofstream& outfile, const bool prependTab) const;
		/// Prints the parameter range description to the description file.
		void printDescription() const;
		/// Prints the number of parameter values to the description file.
		void printNDescription() const;
		/// Prints the parameter scenario info to cerr.
		void info() const { cerr << " " << this->heading << " " << this->value(); }
		/// Returns that the parameter varies.
		bool varies() const _wur_ { return true; }
		/// Returns the index of the lower bound of the interval that contains a value; if val is outside of the range, the nearest interval index from the range is returned. \param [in] val Value to bracket.
		size_t bracketChop(paramT val) {
			const long int r(bracket(val));
			if(r < 0)
				return 0;
			if(r >= mNSteps - 1)
				return mNSteps - 2;
			return r;
		}
		/// Returns whether the parameter changes from the previous scenario to the current scenario.
		bool hasChanged() const _wur_ { return mChanges; }
		/// Reports that the parameter has not changed from the previous scenario to the current scenario.
		void noChange() { mChanges = false; }
		/// Returns whether the interval of the range contains the value \c a @param [in] a Value to test
		bool intervalContains(const paramT a) const _wur_ { return (a >= min()) && (a <= max()); }
};

/// Represents a set of simulation parameter values. @tparam paramT underlying parameter type. \ingroup parameters
template<class paramT = double> class paramArrayCls : public paramCls<paramT> {
	protected:
		typedef std::vector<paramT> vecCls;
		/// Stores the parameter values.
		vecCls mVector;
		/// Current parameter index.
		size_t mIndex;
		/// Tracks whether the parameter changes from the previous scenario to the current scenario.
		bool mChanges;
	public:
		/// Starts the simulation.
		void start() { this->set(mVector.front()); mIndex = 0; mChanges = true; }
		/// Constructor takes an initializer list as input value
		paramArrayCls(std::initializer_list<paramT> l, const char* Heading, const char* Descr) : paramCls<paramT>(*(l.begin()), Heading, Descr), mVector(l.begin(), l.end()), mIndex(0), mChanges(true) {
			if(l.size() == 0) {
				cerr << __FILE__ << ":" << __LINE__ << " " << __FUNCTION__ << "() zero steps.\n";
				std::exit(1);
			}
			this->set(mVector.front());
		}
		/// Constructor takes an array and its size as input value
		paramArrayCls(const paramT* p, size_t size, const char* Heading, const char* Descr) : paramCls<paramT>(Heading, Descr), mChanges(true) {
			if(size <= 0) {
				cerr << "Attempt to create a zero-sized parameter array at " << __FILE__ << ":" << __LINE__ << ":" << __FUNCTION__ << "().\n";
				std::exit(1);
			}
			mVector.assign(p, p + size);
			std::sort(mVector.begin(), mVector.end());
			start();
		}
		/// Returns the current index.
		size_t index() const _wur_ { return mIndex; }
		/// Returns the very index; this is implemented for recursion.
		size_t meanIndex() const _wur_ { return index(); }
		/// Returns whether the simulation should continue.
		bool cont() const _wur_ { return mIndex < mVector.size(); }
		/// Steps the simulation.
		void step() {
			++mIndex;
			if(cont())
				this->set(mVector[mIndex]);
			mChanges = true;
		}
		/// Returns the number of steps of the parameter.
		size_t nsteps() const _wur_ { return mVector.size(); }
		/// Sets the parameter to a specific step.
		void setIndex(const size_t index) {
			if(index >= nsteps()) {
				cerr << __LINE__ << " " << __PRETTY_FUNCTION__ << " Parameter " << this->heading << " is beyond limits." << endl;
				std::exit(1);
			}
			mIndex = index;
			this->set(mVector[mIndex]);
			mChanges = true;
		}
		/// Returns the size of arrays related to the parameter
		size_t arraySize() const _wur_ { return nsteps(); }
		/// Prints the column heading corresponding to the parameter.
		void printHeading(std::ofstream& outfile, const bool prependTab) const;
		/// Prints the parameter range description to the description file.
		void printDescription() const;
		/// Prints the parameter scenario info to cerr.
		void scenarioInfo() const { cerr << " " << this->heading << " " << this->value(); }
		/// Returns that the parameter varies.
		bool varies() const _wur_ { return true; }
		/// Returns whether the parameter changes from the previous scenario to the current scenario.
		bool hasChanged() const _wur_ { return mChanges; }
		/// Reports that the parameter has not changed from the previous scenario to the current scenario.
		void noChange() { mChanges = false; }
};

/// Represents a simulation Boolean parameter for sensibility analyzes; Both true and false will be analized. \ingroup parameters
class boolRangeCls : public paramCls<bool> {
	private:
		/// Current parameter index.
		size_t mIndex;
		/// Tracks whether the parameter changes from the previous scenario to the current scenario.
		bool mChanges;
	public:
		/// Starts the simulation.
		void start() { paramCls<bool>::set(false); mIndex = 0; mChanges = true; }
		/// Constructor.
		boolRangeCls(const char* Heading, const char* Descr) : paramCls<bool>(false, Heading, Descr), mChanges(true) {
			paramCls<bool>::set(false);
		}
		/// Returns the current index.
		size_t index() const _wur_ { return mIndex; }
		/// Returns the very index; this is implemented for recursion.
		size_t meanIndex() const _wur_ { return index(); }
		/// Returns whether the simulation should continue.
		bool cont() const _wur_ { return mIndex < 2; }
		/// Steps the simulation.
		void step() {
			++mIndex;
			this->mValue = !(this->mValue);
			mChanges = true;
		}
		/// Produces an error.
		void add(const bool) {
			cerr << __LINE__ << " " << __PRETTY_FUNCTION__ << " Tried to add to parameter " << this->heading << "." << endl;
			std::exit(1);
		}
		/// Returns the number of steps of the parameter.
		size_t nsteps() const _wur_ { return 2; }
		/// Returns the minimum of the parameter.
		bool min() const _wur_ { return false; }
		/// Returns the maximum value the parameter can take.
		bool max() const _wur_ { return true; }
		/// Returns the size of arrays related to the parameter
		size_t arraySize() const _wur_ { return 2; }
		/// Returns the total number of parameter values that are evaluated
		size_t nValues() const _wur_ { return 2; }
		/// Prints the column heading corresponding to the parameter. @param [in] outfile Output file. @param [in] prependTab Prepend a tabulator to the output.
		void printHeading(std::ofstream& outfile, const bool prependTab) const;
		/// Prints the parameter range description to the description file.
		void printDescription() const;
		/// Prints the number of parameter values to the description file.
		void printNDescription() const;
		/// Prints the parameter scenario info to cerr.
		void info() const { cerr << " " << this->heading << " " << mIndex; }
		/// Returns that the parameter varies.
		bool varies() const _wur_ { return true; }
		/// Returns whether the parameter changes from the previous scenario to the current scenario.
		bool hasChanged() const _wur_ { return mChanges; }
		/// Reports that the parameter has not changed from the previous scenario to the current scenario.
		void noChange() { mChanges = false; }
};

/// Description file that the user defines in the main file.
extern std::ofstream descrfile;

template<class paramT>
void indPCls<paramT>::set(const paramRangeCls<paramT>& p) {
	set(p.value());
}

template<class paramT>
void paramCls<paramT>::printDescription() const {
	descrfile << this->heading << "\t" << descr << "\t\t\t\t\t" << this->value() << "\n";
}

template<class paramT>
void paramCls<paramT>::checkPositive() const {
	if(!(this->value() > 0)) {
		cerr << __LINE__ << " " << __PRETTY_FUNCTION__ << " Parameter " << this->heading << " has value " << this->value() << " which must be greater than zero. Check the source code." << endl;
		std::exit(1);
	}
}

template<class paramT>
void paramIntervalCls<paramT>::printDescription() const {
	descrfile << this->heading << "\t" << descr << "\t" << this->min() << "\t\t\t" << this->max() << "\n";
}

template<class paramT>
bool paramIntervalCls<paramT>::hasChanged() {
	if(mChanges) {
		mChanges = false;
		return true;
	}
	return false;
}

template<class paramT>
void paramRangeCls<paramT>::printHeading(std::ofstream& outfile, const bool prependTab) const {
	if(prependTab)
		outfile << "\t";
	outfile << this->heading;
}

template<class paramT>
void paramRangeCls<paramT>::printDescription() const {
	descrfile << this->heading << "\t" << this->descr;
	if(mNSteps > 1)
		descrfile << "\t" << min() << "\t" << mStep << "\t" << nsteps() << "\t" << max();
	else
		descrfile << "\t\t\t\t\t" << min();
	descrfile << "\n";
}

template<class paramT>
void paramRangeCls<paramT>::printNDescription() const {
	descrfile << this->heading << "\t" << this->descr << "\t\t\t" << this->nsteps() << "\n";
}

template<class paramT>
void paramArrayCls<paramT>::printDescription() const {
	descrfile << this->heading << "\t" << this->descr << "\t\t\t\t\t";
	const auto end(mVector.end());
	bool moreThanOne(false);
	for(auto& item : mVector) {
		if(moreThanOne)
			descrfile << "\t";
		else
			moreThanOne = true;
		descrfile << item;
	}
	descrfile << "\n";
}

template<class paramT>
void paramArrayCls<paramT>::printHeading(std::ofstream& outfile, const bool prependTab) const {
	if(prependTab)
		outfile << "\t";
	outfile << this->heading;
}

/** \brief Calculates a linear relation between two parameters which have max() and min(), such as a paramIntervalCls.
		@tparam Tx Data type of the X axis.
		@tparam Ty Data type of the Y axis.
		@param xInterval Interval at the X axis of the linear relation.
		@param yInterval Interval at the Y axis of the linear relation.
		@param x Point at the X axis of the linear relation, for which the corresponding y point is calculated.
		\return the \c y value that corresponds to the \c x point, in the linear function spanned by the two intervals \c xInterval, \c yInterval.
*/
template <class Tx, class Ty> double linearRelation(const Tx& xInterval, const Ty& yInterval, const double x) {
	// Slope
	const double m(yInterval.width() / xInterval.width());
	// Linear relation
	return yInterval.min() + m * (x - xInterval.min());
}

/// Defines a bool value whose truth depends on a static value. Use case: Threads iterate concurrently over a list of persons to modify some data. A member variable <tt>flipBool mChecked</tt> tracks what persons have been processed. At the start of an iteration start() is called, such that static variable trueValue is different to mValue, so value() is false for all persons. As the threads progress, they mark each person using flip(). Using value(), other threads can verify that a person has been already processed. When a new iteration over the list of people is about to begin, there is no need to reset the values of mChecked for all people; instead, the start() function flips the static variable trueValue and because of this value() becomes false for all. @tparam uniqueClass useful to have different versions of trueValue for every kind of flipBool variable. The containing class is a good candidate.
template<class uniqueClass> class flipBool {
	private:
		/// Truth value. At the start of the program, this should be set to the initially desired value for value().
		static bool trueValue;
		/// Local value.
		bool mValue;
	public:
		/// Constructor.
		flipBool() : mValue(false) {}
		/// Flips the true value to start a new iteration.
		static void start() { trueValue = !trueValue; }
		/// Toggles the value.
		void flip() { mValue = !mValue; }
		/// Returns the bool-value.
		bool value() { return mValue == trueValue; }
};
