/* This file is part of dengue: Dengue fever individual propagation model
Copyright (C) 2012 Harish Padmanabha, Fabio Correa

This project has been developed by Harish Padmanabha<harishpadmanabha@gmail.com> and Fabio Correa<facorread@gmail.com>
This code has been delivered to the Public Library of Science, on February 16, 2015.
We, the developers, grant you a license to review and modify this source code for your own use.
We do not grant you license to distribute or publish this program or any derivative work, any of its output, the ideas or algorithms contained within, in any form.
Anybody who wishes to get a copy of this source code or to get this project under a different license should email the developers directly.

*/

/// \file results.h Infrastructure for storing measurements and results for each simulation scenario.

/// \addtogroup results Results containers

//Check for C++0x
#if (__cplusplus <= 199711L) && !defined(__GXX_EXPERIMENTAL_CXX0X__)
#error Please use a C++11 compiler, such as GCC-4.6 or later. On Linux, use a recent distribution. On Windows, use MinGW or Cygwin. On Mac OS, use macports.
#endif

/// \def _wur_
/// \brief Warns about unused results of functions
/// \details _wur_ can be used in most function declarations so that the compiler warns about unused results of these functions. _wur_ is automatically enabled on GCC only; in other compilers it should do nothing.
#if defined(__GXX_EXPERIMENTAL_CXX0X__)
#define _wur_ __attribute__ (( warn_unused_result ))
#else
#define _wur_
#endif

#include <iostream>
#include <sstream>

// Boost includes
#include <boost/accumulators/accumulators.hpp>
#include <boost/accumulators/statistics/variance.hpp>
#include <boost/accumulators/statistics/skewness.hpp>
#include <boost/accumulators/statistics/max.hpp>
#include <boost/accumulators/statistics/min.hpp>

using std::cerr;
using std::endl;

/// \brief \c true to save all results to memory, or \c false to reuse a small memory region for results.
/// \details If \c true, all the simulation scenario results are kept in memory, so they can be compared and organized. The program can wait until all the simulations are finished before starting to save them to files.\n
/// If \c false, the simulation results are kept in memory one scenario at a time, saving memory, with the program being responsible of writing the results to files as soon as each scenario is finished. When a new scenario starts, the memory region is wiped and reused.
extern const bool keepResults;

class counterCls;
template<class C> class mCls;
/// Saves a set of data and calculates their mean. \ingroup results
typedef mCls<boost::accumulators::accumulator_set<double, boost::accumulators::features<boost::accumulators::tag::mean> > > meanMeasureCls;
/// Saves a set of data and calculates their mean and variance. \ingroup results
typedef mCls<boost::accumulators::accumulator_set<double, boost::accumulators::features<boost::accumulators::tag::mean, boost::accumulators::tag::lazy_variance> > > measureCls;
/// Saves a set of data and calculates their mean and minimum. \ingroup results
typedef mCls<boost::accumulators::accumulator_set<double, boost::accumulators::features<boost::accumulators::tag::mean, boost::accumulators::tag::min> > > meanMinCls;
/// Saves a set of data and calculates their mean and maximum. \ingroup results
typedef mCls<boost::accumulators::accumulator_set<double, boost::accumulators::features<boost::accumulators::tag::mean, boost::accumulators::tag::max> > > meanMaxCls;
/// Saves a set of data and calculates their mean and minimum. \ingroup results
typedef mCls<boost::accumulators::accumulator_set<double, boost::accumulators::features<boost::accumulators::tag::mean, boost::accumulators::tag::max> > > meanMaxCls;
/// Saves a set of data and calculates their mean and skewness. \ingroup results
typedef mCls<boost::accumulators::accumulator_set<double, boost::accumulators::features<boost::accumulators::tag::mean, boost::accumulators::tag::skewness> > > skewCls;
/// Safely accumulates the double minimum of a set of data. \ingroup results
typedef mCls<boost::accumulators::accumulator_set<double, boost::accumulators::features<boost::accumulators::tag::min> > > doubleMinCls;
/// Safely accumulates the double maximum of a set of data. \ingroup results
typedef mCls<boost::accumulators::accumulator_set<double, boost::accumulators::features<boost::accumulators::tag::max> > > doubleMaxCls;
/// Safely accumulates the double sum of a set of data. \ingroup results
typedef mCls<boost::accumulators::accumulator_set<double, boost::accumulators::features<boost::accumulators::tag::sum> > > doubleSumCls;
/// Safely accumulates the int minimum of a set of data. \ingroup results
typedef mCls<boost::accumulators::accumulator_set<int, boost::accumulators::features<boost::accumulators::tag::min> > > intMinCls;
/// Safely accumulates the int maximum of a set of data. \ingroup results
typedef mCls<boost::accumulators::accumulator_set<int, boost::accumulators::features<boost::accumulators::tag::max> > > intMaxCls;
/// Safely accumulates the int sum of a set of data. \ingroup results
typedef mCls<boost::accumulators::accumulator_set<int, boost::accumulators::features<boost::accumulators::tag::sum> > > intSumCls;
/// Safely accumulates the size_t minimum of a set of data. \ingroup results
typedef mCls<boost::accumulators::accumulator_set<size_t, boost::accumulators::features<boost::accumulators::tag::min> > > sizeMinCls;
/// Safely accumulates the size_t maximum of a set of data. \ingroup results
typedef mCls<boost::accumulators::accumulator_set<size_t, boost::accumulators::features<boost::accumulators::tag::max> > > sizeMaxCls;
/// Safely accumulates the size_t sum of a set of data. \ingroup results
typedef mCls<boost::accumulators::accumulator_set<size_t, boost::accumulators::features<boost::accumulators::tag::sum> > > sizeSumCls;
/// Contains simulation results; must be defined at main.cpp
class resultsCls;
template<class C, class rangeCls> class tableCls;

/// Represents a counter for numbers of individuals; in essence, it is a sophisticated version of an int. \ingroup results
class counterCls {
	private:
		/// Current counter value.
		int mCounter;
	public:
		/// Constructor guarantees initialization to zero.
		counterCls() : mCounter(0) {}
		/// Sets the counter value.
		void set(const int value) { mCounter = value; }
		/// Resets the counter.
		void start() { mCounter = 0; }
		/// Returns the counter value.
		int value() const { return mCounter; }
		/// Increases the counter value.
		void operator++() { ++mCounter; }
		/// Decreases the counter value.
		void operator--() { --mCounter; }
		/// Increases the counter value.
		void operator++(const int) { ++mCounter; }
		/// Decreases the counter value.
		void operator--(const int) { --mCounter; }
		/// Increases the counter by some value.
		void operator+=(const int a) { mCounter += a; }
		/// Increases the counter by the value of another counter.
		void operator+=(const counterCls& a) { mCounter += a.mCounter; }
		/// Decreases the counter by some value.
		void operator-=(const int a) { mCounter -= a; }
};

/// Saves a set of measurements or results and calculates statistical properties like mean or variance. @tparam C Underlying Boost accumulator. \ingroup results
template<class C> class mCls {
	private:
		/// Boost accumulator class
		C mAccum;
	public:
		///Checks whether the object has an expected number of elements.
		void checkEqualLess(const size_t expectedSize) const {
			using boost::accumulators::extract::count;
			if(n() <= expectedSize)
				return;
			cerr << __LINE__ << " " << __PRETTY_FUNCTION__ << " Expected at most " << expectedSize << " elements, but there are only " << count(mAccum) << "." << endl;
			std::exit(1);
		}
		///Checks whether the object has an expected number of elements.
		void checkEqual(const size_t expectedSize) const {
			if(n() == expectedSize)
				return;
			cerr << __LINE__ << " " << __PRETTY_FUNCTION__ << " Expected " << expectedSize << " elements, but there are only " << count(mAccum) << "." << endl;
			std::exit(1);
		}
		/// Adds a datum to the data set. \param [in] a Datum to add.
		void operator()(const typename C::sample_type a) { mAccum(a); }
		/// Returns the number of parameters of the data set.
		size_t n() const _wur_ { return boost::accumulators::extract::count(mAccum); }
		/// Returns the sum of the data.
		typename C::sample_type sum() const _wur_ { return boost::accumulators::extract::sum(mAccum); }
		/// Returns an unsafe mean of the data. If there is no data, \c NaN is returned.
		typename C::sample_type mean() const _wur_ { return boost::accumulators::extract::mean(mAccum); }
		/// Returns an unsafe maximum of the data. If there is no data, \c NaN is returned.
		typename C::sample_type max() const _wur_ { return boost::accumulators::extract::max(mAccum); }
		/// Returns an unsafe minimum of the data. If there is no data, \c NaN is returned.
		typename C::sample_type min() const _wur_ { return boost::accumulators::extract::min(mAccum); }
		/// Returns an unsafe standard deviation of the data. If there is no data, \c NaN is returned.
		typename C::sample_type sd() const _wur_ { return std::sqrt(boost::accumulators::extract::variance(mAccum)); }
		/// Returns an unsafe skewness of the data. If there is no data, \c NaN is returned.
		typename C::sample_type skewness() const _wur_ { return boost::accumulators::extract::skewness(mAccum); }
		/// Returns a safe mean of the data. @param [in] expectedSize to check the data number.
		typename C::sample_type mean(const int expectedSize) const _wur_ { checkEqual(expectedSize); return mean(); }
		/// Returns a safe mean of the data: If there is no data, 0 is returned.
		typename C::sample_type safeMean() const _wur_ { return n() ? mean() : 0; }
		/// Prints the mean of the data to the outfile.
		void printMean(std::ofstream& outfile) const;
		/// Prints the mean of the data to the outfile, with a coefficient.
		void printMean(std::ofstream& outfile, const double coefficient) const;
		/// Prints the minimum of the data to the outfile.
		void printMin(std::ofstream& outfile) const;
		/// Prints the maximum of the data to the outfile.
		void printMax(std::ofstream& outfile) const;
		/// Prints the skewness of the data to the outfile.
		void printSkewness(std::ofstream& outfile) const;
		/// Prints the variance coefficient (SD / mean) of the data to the outfile.
		void printVarianceCoeff(std::ofstream& outfile) const;
		/// Clears the accumulator.
		void clear() { mAccum = C(); }
		/// Returns whether the maximum value is valid.
		bool maxIsValid() const _wur_ { return max() > std::numeric_limits<typename C::sample_type>::lowest(); }
		/// Returns whether the minimum value is valid.
		bool minIsValid() const _wur_ { return min() < std::numeric_limits<typename C::sample_type>::max(); }
		/// Combines the contents of a meanMeasureCls with \c this; \c this should also be a meanMeasureCls.
		void combine(const meanMeasureCls& rhs) {
			int nElements(rhs.n());
			if(nElements) {
				operator()(rhs.sum());
				for(--nElements; nElements; --nElements)
					operator()(0);
			}
		}
};

/// \brief Recursively contains results for all scenarios of a simulation.
/// @tparam C Contained class, generally another instantiation of this template or the resultsCls class.
/// @tparam rangeCls Specifies the parameter on which the current container is working.
/// \ingroup results
/// \details This class is a sophisticated meta-program using standard C++ templates. It is designed to hold the results for all scenarios of a simulation. \n
/// Simulation scenarios are defined by the values of all the parameters, in all their possible combinations. \n
/// tableCls works closely with these parameters, using one instantiation per parameter. \n
/// tableCls works with changing parameters only, such as paramRangeCls, paramArrayCls, and boolRangeCls. \n
/// tableCls can store all the simulation results in RAM memory, if \ref keepResults is true. However, for simulations that require lots of memory, \ref keepResults may be set to false. In this case, only one set of results is saved in a small region of memory, and the program must write them to disk as soon as each scenario ends. When the next scenario begins, the region is reused for storing the new results. \n
/// tableCls is designed to be recursive. Each level of recursion corresponds to one of the parameters of the simulation. For simulations that have scenarios according to one parameter, one recursion level is used: \n
/// \code
/// class resultsCls;	// Results container defined by the user;
/// paramRangeCls<> simParam(1.1, 0.4, 15);
/// tableCls <resultsCls, simParam> results1Cls;
/// results1Cls simResults;
/// \endcode
/// \c simResults holds 15 results containers of the resultsCls class because the associated parameter simParam holds 15 values. The results can be accessed according to the current value of simParam. \n
/// In general, simulations require scenarios based on several parameters; let us see a complex example:
/// \code
/// class resultsCls;	// Results container defined by the user;
/// paramRangeCls<> simParamA(1.1, 0.4, 15);
/// paramRangeCls<> simParamB(0.2, 0.1, 8);
/// paramRangeCls<> simParamC(0.3, 0.1, 12);
/// tableCls <resultsCls, simParamA> results1Cls;
/// tableCls <results2Cls, simParamB> results2Cls;
/// tableCls <results3Cls, simParamC> results3Cls;
/// results3Cls simResults;
/// \endcode
/// In this example, simResults holds 15 x 8 x 12 = 1440 sets of results, one for each simulation scenario. The results for each scenario are accessed according to the values of the three parameters. Setting \ref keepResults to false has the effect of using only one resultsCls container to store the results for every scenario, with the program being responsible to produce the output at the end of each scenario before the resultsCls container is wiped at the beginning of the next scenario.\n
/// In this example, the intermediate classes results1Cls and results2Cls should not be used by the developer.
template<class C, class rangeCls> class tableCls {
	private:
		/// Specifies the range.
		static rangeCls& range;
		/// Contains the results.
		std::vector<C> mRes;
	public:
		/// Only allowed constructor.
		void init();
		/// Returns that this is not the resultsCls class.
		static bool isTableCls() { return true; }
		/// Returns a result element.
		resultsCls& result() _wur_;
		/// Prints the parameter value to a file. @param [in] outfile Output file. @param [in] prependTab Prepend a tabulator to the output.
		static void printValue(std::ofstream& outfile, const bool prependTab = false);
		/// Returns a value for unique file name generation.
		static std::string fileNameValue();
		/// Determines that the current parameter appears in a list with the purpose of omitting its output.
		static bool paramIsIn(const std::initializer_list<void*>& omitParam);
		/// Thread-unsafe: produces a file name from omitted parameters and their values. The file name is saved to filenamestr.
		static void produceFileName(std::stringstream& fileNameStream, const std::initializer_list< void* >& omitParam);
		/// Steps the parameter ranges and the higher parameters in the hierarchy as needed. \return Whether the scenario should change: if true, it reports the end-of-simulation for the current parameter.
		static bool step() _wur_;
		/// Steps the parameter ranges and the higher parameters in the hierarchy as needed. @param [in] omitParam List of pointers to the parameters that must be omitted from the operation. \return whether the scenario should change: if true, it reports the end-of-simulation for the current parameter.
		static bool step(const std::initializer_list< void* >& omitParam) _wur_;
		/// Steps the parameter ranges and the higher parameters in the hierarchy as needed, for parameters that were omitted from the main output only. @param [in] omitParam List of pointers to the parameters that were omitted from the operation. \return whether the scenario should change: if true, it reports the end-of-simulation for the current parameter.
		static bool stepOmitted(const std::initializer_list< void* >& omitParam) _wur_;
		/// Returns whether the range exploration should continue
		static bool cont() _wur_ { return range.cont(); }
		/// Restarts all parameters.
		static void start() { range.start(); C::start(); }
		/// Restarts all parameters, with exceptions. @param [in] omitParam List of pointers to the parameters that must be omitted from the operation.
		static void start(const std::initializer_list< void* >& omitParam);
		/// Print the column heading corresponging to the parameter.
		static void printHeading(std::ofstream& outfile, const bool prependTab = false);
		/// \name Horizontal layout parameters
		/// @{ Useful for printing simulation scenarios in the column headers

		/// Prints the parameter name corresponding to the indicated \c levelOffset. @param [in] outfile Output file. @param [in] levelOffset if zero, the parameter of the current tableCls level is printed. If nonzero, the function is recursed to the next level. @param [in] omitParam List of addresses of the parameters that should be omitted from the output.
		static void printHzParamName(std::ofstream& outfile, const size_t levelOffset, const std::initializer_list< void* >& omitParam);
		/// Prints the parameter value corresponding to the indicated \c levelOffset. @param [in] outfile Output file. @param [in] nColumns Number of columns of output for each scenario. @param [in] levelOffset if zero, the parameter of the current tableCls level is printed. If nonzero, the function is recursed to the next level. @param [in] omitParam List of addresses of the parameters that should be omitted from the output.
		static bool printHzParamValue(std::ofstream& outfile, size_t nColumns, const size_t levelOffset, const std::initializer_list< void* >& omitParam);
		/// Prints the column heading corresponding to the parameter in a horizontal layout. @param [in] outfile Output file. @param [in] nColumns Number of columns of output per scenario. @param [in] omitParam List of pointers to the parameters that must be omitted from the heading output and which should be put in the traditional vertical layout from the main.cpp file.
		static void printHorizontalHeading(std::ofstream& outfile, size_t nColumns, const std::initializer_list< void* >& omitParam);
		/// @}

		/// Prints scenario information to \c cerr.
		static void info();
		/// Prints information on number of scenarios to \c cerr.
		static int _wur_ nScenarios();
		/// Returns a pointer to the range.
		static rangeCls& r() { return range; }
};

template<class C> void mCls<C>::printMean(std::ofstream& outfile) const {
	outfile << "\t";
	if(n())
		outfile << mean();
}

template<class C> void mCls<C>::printMean(std::ofstream& outfile, const double coefficient) const {
	outfile << "\t";
	if(n())
		outfile << mean() * coefficient;
}

template<class C> void mCls<C>::printMin(std::ofstream& outfile) const {
	outfile << "\t";
	typedef typename C::sample_type sCls;
	const sCls m(min());
	if(m < std::numeric_limits<sCls>::max())
		outfile << m;
}

template<class C> void mCls<C>::printMax(std::ofstream& outfile) const {
	outfile << "\t";
	typedef typename C::sample_type sCls;
	const sCls m(max());
	if(m > std::numeric_limits<sCls>::lowest())
		outfile << m;
}

template<class C> void mCls<C>::printSkewness(std::ofstream& outfile) const {
	outfile << "\t";
	const double skew(skewness());
	if(std::isfinite(skew))
		outfile << skew;
}

template<class C> void mCls<C>::printVarianceCoeff(std::ofstream& outfile) const {
	outfile << "\t";
	if(n()) {
		const double coeff(sd() / mean());
		if(std::isfinite(coeff))
			outfile << coeff;
	}
}

template<class C, class rangeCls>
void tableCls<C, rangeCls>::init() {
	mRes.clear();
	size_t i(keepResults ? range.arraySize() : 1);
	for(mRes.reserve(i); i; --i) {
		mRes.emplace_back();
		mRes.back().init();
	}
}

template<class C, class rangeCls>
resultsCls& tableCls<C, rangeCls>::result() {
	if(keepResults)
		return mRes[range.index()].result();
	else
		return mRes.front().result();
}

template<class C, class rangeCls>
void tableCls<C, rangeCls>::printValue(std::ofstream& outfile, const bool prependTab) {
	if(prependTab)
		outfile << "\t";
	outfile << range.value();
	C::printValue(outfile, true);
}

template<class C, class rangeCls>
std::string tableCls<C, rangeCls>::fileNameValue() {
	std::stringstream winHack;
	winHack << range.index() << C::fileNameValue();
	return winHack.str();
}

template<class C, class rangeCls>
bool tableCls<C, rangeCls>::paramIsIn(const std::initializer_list<void*>& omitParam) {
	for(void* paramPtr : omitParam) {
		if(paramPtr == &range)
			return true;
	}
	return false;
}

template<class C, class rangeCls>
void tableCls<C, rangeCls>::produceFileName(std::stringstream& fileNameStream, const std::initializer_list<void*>& omitParam) {
	if(paramIsIn(omitParam))
		fileNameStream << "_" << range.name() << range.value();
	C::produceFileName(fileNameStream, omitParam);
}

template<class C, class rangeCls>
void tableCls<C, rangeCls>::info() {
	range.info();
	C::info();
}

template<class C, class rangeCls>
int tableCls<C, rangeCls>::nScenarios() {
	const int steps(range.nsteps());
	cerr << "x" << steps;
	return steps * C::nScenarios();
}

template<class C, class rangeCls>
bool tableCls<C, rangeCls>::step() {
	if(C::step()) {
		range.step();
		if(range.cont())
			return false;
		range.start();
		return true;
	}
	range.noChange();
	return false;
}

template<class C, class rangeCls>
bool tableCls<C, rangeCls>::step(const std::initializer_list< void* >& omitParam) {
	if(paramIsIn(omitParam))
		return C::step(omitParam);
	else if(C::step(omitParam)) {
		range.step();
		if(range.cont())
			return false;
		range.start();
		return true;
	}
	range.noChange();
	return false;
}

template<class C, class rangeCls>
bool tableCls<C, rangeCls>::stepOmitted(const std::initializer_list< void* >& omitParam) {
	if(!paramIsIn(omitParam))
		return C::stepOmitted(omitParam);
	else if(C::stepOmitted(omitParam)) {
		range.step();
		if(range.cont())
			return false;
		range.start();
		return true;
	}
	range.noChange();
	return false;
}

template<class C, class rangeCls>
void tableCls<C, rangeCls>::start(const std::initializer_list< void* >& omitParam) {
	if(!paramIsIn(omitParam))
		range.start();
	C::start(omitParam);
}

template<class C, class rangeCls>
void tableCls<C, rangeCls>::printHeading(std::ofstream& outfile, const bool prependTab) {
	range.printHeading(outfile, prependTab);
	C::printHeading(outfile, true);
}

template<class C, class rangeCls>
void tableCls<C, rangeCls>::printHzParamName(std::ofstream& outfile, const size_t levelOffset, const std::initializer_list<void*>& omitParam) {
	if(paramIsIn(omitParam))
		C::printHzParamName(outfile, levelOffset, omitParam);
	else if(levelOffset)
		C::printHzParamName(outfile, levelOffset - 1, omitParam);
	else
		range.printHeading(outfile, false);
}

template<class C, class rangeCls>
bool tableCls<C, rangeCls>::printHzParamValue(std::ofstream& outfile, size_t nColumns, const size_t levelOffset, const std::initializer_list<void*>& omitParam) {
	if(paramIsIn(omitParam)) {
		if(levelOffset || C::isTableCls())
			return C::printHzParamValue(outfile, nColumns, levelOffset, omitParam);
		else
			return false;
	} else if(levelOffset)
		return C::printHzParamValue(outfile, nColumns, levelOffset - 1, omitParam);
	else for(; nColumns; --nColumns)
		outfile << "\t" << range.value();
	return C::isTableCls();
}

template<class C, class rangeCls>
void tableCls<C, rangeCls>::printHorizontalHeading(std::ofstream& outfile, size_t nColumns, const std::initializer_list<void*>& omitParam) {
	// The following loop is intended to output each line;
	bool continuePrint(true);
	for(size_t line(0); continuePrint; ++line) {
		printHzParamName(outfile, line, omitParam);
		start(omitParam);
		while(true) {
			// Print the parameter value;
			continuePrint = printHzParamValue(outfile, nColumns, line, omitParam);
			if(step(omitParam))
				break;
		}
		outfile << endl;
	}
}
